/***********************************************************
*
*   ShareCom 1.0
*    Copyright (C)  1994 Identity Software Inc
*
* basket.h
* 
* Utility Data Structures. Linked List, FIFO
*
*/



#ifndef BASKET_H
#define BASKET_H


typedef struct _dsset {
  caddr_t data;
  struct _dsset * next;
} s_node_t;


typedef s_node_t * dk_set_t;


#define DO_SET(type, var, set) \
{                                   \
  type var;                         \
  s_node_t * iter = * set;          \
  s_node_t * nxt;                   \
  while (iter) {                    \
    var = (type) (iter->data);      \
      nxt = iter->next;


#define END_DO_SET()   \
    iter = nxt;        \
  };                   \
};


#define DO_DELETE(type, var, start)  \
{ type var = start;   \
  type * prev = & start;  \
  while (var) {

#define REMOVE_THIS(var, next) \
    * prev = next; \
    break;

#define DELETE_THIS(var, next) \
    * prev = next; \
    dk_free(var,sizeof(s_node_t)); \
    break;

#define END_DO_DELETE(var, next) \
    prev = & next; \
    var = next; \
  }; \
};



/* 
  struct basket_t

  This is a queue of tokens. The next_token member of the token is used 
  to link successive tokens when they are in a basket. A token is either 
  in no basket or in exactly one basket
*/

typedef struct _bsk {
  s_node_t * first_token;
  s_node_t  * last_token;
} basket_t;



extern void basket_add (basket_t *bsk, void *token);
extern void * basket_get (basket_t *bsk);
extern int basket_is_empty(basket_t * bsk);

extern void dk_set_push (s_node_t ** set, void * item);
extern void dk_set_pushnew (s_node_t ** set, void * item);

void* dk_set_pop (s_node_t ** set);
extern void dk_set_delete(dk_set_t * set, void * item);

dk_set_t dk_set_last (dk_set_t s1);

dk_set_t dk_set_conc (dk_set_t s1, dk_set_t s2);

dk_set_t dk_set_cons (caddr_t s1 ,dk_set_t s2);

/* this is like car or first in lisp */
#define DK_SET_FIRST(set) \
  ( *set ? (((dk_set_t) *set)->data) : NULL)


long dk_set_length (s_node_t * set);

void dk_set_free (s_node_t * set);

caddr_t * dk_set_to_array (s_node_t * node);

s_node_t * dk_set_member (s_node_t * set, void * elt);


#endif


 


