
/***************************************************************
 *
 *
 *   ShareCom 1.0   
 *    Copyright (C)  1994 Identity Software Inc
 *
 *  clitest.c
 * 
 *  Futures RPC Test Client
 
 *  This program also functions as a server, being its own client.
 */



#include <stdio.h>
#include <future.h>





SERVICE_1(fibo_service, _fs, "fi", DA_FUTURE_REQUEST,
	  DV_LONG_INT,   /* return type */
	  DV_LONG_INT);

SERVICE_1(blobi_service, _bis, "binc", DA_FUTURE_REQUEST,
	  DV_LONG_STRING,
	  DV_LONG_STRING);

SERVICE_1(blobi_1_service, _bis1, "binc_one_way",DA_FUTURE_REQUEST,
	  DV_LONG_STRING,
	  DV_LONG_STRING);

SERVICE_1(blob_resend_service, _bis1r, "blob_resend",   DA_FUTURE_REQUEST,
	  DV_LONG_STRING,
	  DV_LONG_STRING);

SERVICE_1(direct_io_blobi_service, _diobs, "direct_io_binc",
	  DA_DIRECT_IO_FUTURE_REQUEST, 
	  DV_LONG_INT,
	  DV_LONG_INT);

SERVICE_1(direct_io_resend_service, _diors, "direct_io_resend",
	  DA_FUTURE_REQUEST, 
	  DV_LONG_STRING,
	  DV_LONG_STRING);




SERVICE_2(many_answers_service, _ma, "many_answers", DA_FUTURE_REQUEST,
	  DV_LONG_INT,
	  DV_LONG_INT,
	  DV_C_STRING);

SERVICE_0 (drop_dead_service, "drop_dead", DA_FUTURE_REQUEST,
	   DV_SEND_NO_ANSWER);



/* sessions */

dk_session_t* server;
dk_session_t* server2;
dk_session_t * self;

dk_session_t * listen_session;
dk_session_t * listen2;

#ifdef WINDOWS
#define dk_exit(x) \
   PrpcLeave (); \
   exit (x);
#endif



#if defined (OS2)||defined(WINDOWS)
/* 16 bit */
#define BLOB_SIZE 10000
#else
#define BLOB_SIZE 100000 
#endif


future_t * futures [1000];
/* Global array of simultaneously pending RPC's */

/*##**********************************************************************
 * 
 *              sertest
 * 
 * Synchronously invoke the fi  service.
 * 
 *
 * Input params : n - count - The number of times the fi service is to be 
 *            - arg - The argument to pass to the fi service.
 *              verbose - Whether to print messages.
 * 
 * Return value : none
 * 
 *
 * Limitations 
 *
 * Globals used : - none
 */


void sertest (int count, int arg, int verbose)
{
  int n, val;
  for (n= 0; n<count; n++) {
    future_t* f = PrpcFuture (server, &fibo_service, (long) arg);
    if (verbose) {
      printf ("Request no %ld sent.\n", f -> ft_request_no);
      fflush (stdout);
    };
    val =  (int) PrpcValueOrWait (f);
    if (verbose) {
      printf ("Answer %ld received \n.",val);
      fflush (stdout);
    };
    PrpcFutureFree (f);
  };
}



/*##**********************************************************************
 * 
 *              partest
 * 
 * Asynchronously invoke the fi  service.
 * 
 *
 * Input params : n - count - The number of times the fi service is to be 
 *            - arg - The argument to pass to the fi service.
 *              verbose - Whether to print messages.
 * 
 * Return value : none
 * 
 *
 * Limitations 
 *
 * Globals used : - futures
 */

void partest (int count, int arg, int verbose)
{
  int n;
  future_t * f;
  for (n=0; n < count; n++) {
    futures[n] = f = PrpcFuture (server, &fibo_service, (long) arg);
    if (verbose) {
      printf ("Request no %ld sent.\n", f -> ft_request_no);
      fflush (stdout);
    };
  };
  for (n=0; n < count; n++) {
    long * v = (long *)PrpcValueOrWait (futures[n]);
    if (verbose) {
      printf ("received req no %ld.\n", futures[n]->ft_request_no);
      fflush (stdout);
    };
    PrpcFutureFree (futures[n]);
  };
}

/*##**********************************************************************
 * 
 *              blob_resend
 * 
 * This is a service called as a future from this process.
 * This in turn invokes the blob incremnt service on the server.
 * 
 * Input params :        - a DV_LONG_STRING
 * 
 * Output params:    - 
 * 
 * Return value :    The processed DV_LONG_STRING from the server
 * 
 * Limitations  : 
 * 
 * Globals used :    default controls
 */

caddr_t blob_resend (caddr_t blob)
{
  future_t* f = PrpcFuture (server, &blobi_service, blob);
  char * value = PrpcValueOrWait (f);
  PrpcFutureFree (f);
  return (value);
}


void post_blobi (caddr_t b, future_request_t * future)
     /* The first argument is the data sent by the client.
	After resending it as the answer, deallocate */
{
  dk_free_box(b);
}



/*##**********************************************************************
 * 
 *              partial_answer_test
 * 
 * Invoke a service that sends multiple answers. Get these answers and 
 * print the contents of the received data structure.
 * 
 * Input params :       - n - The number of answers tp reqiest from the server.
 * 
 * Output params:    - 
 * 
 * Return value :    The none
 * 
 * Limitations  : 
 * 
 * Globals used :    none
 */


void partial_answer_test(long count)
{
  caddr_t * res;
  caddr_t * res2;
  future_t* f = PrpcFuture (server, &many_answers_service,count, "ASCII text.");
  while (1) {
    res = (caddr_t *)PrpcFutureNextResult (f);
    if (FUTURE_IS_EXHAUSTED(f))
      break;
    /* The result, ifnon-null is an array with:
       small integer, large integer, float, double, array with
       ASCII string, binary string. */
    
    
    res2 = (caddr_t *) res[4];
    printf  ("%ld %ld, %f, %lf %s %s\n", 
	     unbox_num(res[0]),
	     unbox_num(res[1]),
	     unbox_float (res[2]),
	     unbox_double (res[3]),
	     unbox_string (res2[0]),
	     unbox_string (res2[1] ));
    
    printf ("Received partial answer = %ld \n", res); fflush(stdout);
    dk_free_tree (res);
    
    
  };
}





/*##**********************************************************************
 * 
 *              direct_io_resend
 * 
 * This is an internally invoked service that calls the direct io blobi
 * service on the server. This implements the client side of the direct IO
 * client server dialogue.
 * 
 * Input params :       - The blob
 * 
 * Output params:    - 
 * 
 * Return value :    The length of the blob
 * 
 * Limitations  : 
 * 
 * Globals used :    none
 */

long DK_CALLBACK direct_io_resend (caddr_t blob)
{
  long ok;
  caddr_t blob2;
  future_t* f
    = PrpcFuture (server2, &direct_io_blobi_service, box_length(blob));
  printf ("\nStarting direct io.\n"); fflush (stdout);
  PrpcWriteObject (server2, blob);
  printf ("\nWrote one blob.\n");     fflush (stdout);
  blob2 = PrpcReadObject (server2);
  printf ("\nRead one blob.\n");      fflush (stdout);
  SESSION_CHECK_IN (server2);
  ok = (long) PrpcValueOrWait (f);
  dk_free_box(blob2);
  PrpcFutureFree (f);
  printf ("\n ok = %d.\n", ok);      fflush (stdout);
  return (ok);
}

/* mty HUHTI missing * from *ses */
caddr_t  DK_CALLBACK service_request_hook_test(dk_session_t *ses, caddr_t request)
{
#ifdef OS2
  printf ("\n Service_request_hook: session = %p, request = %p.\n",
	  ses, request);
#else
  printf ("\n Service_request_hook: session = %x, request = %x.\n",
	  ses, request);
#endif
  fflush(stdout);   
  return (request);
}


#ifdef OS2
#define start_timing(var)
#define end_timing(var) 

#else

#define start_timing(var) \
{   \
      long var = get_msec_real_time();
    
#define end_timing(var) \
    printf ("Elapsed real time = %ld msec.", get_msec_real_time()-var); \
      fflush(stdout);   \
      }; 

#endif  /* OS2 */



/* 
  Test semaphore speed on the platform 
  */

void sem_timing ()
{
  semaphore_t * sem = semaphore_allocate (1);
  int n;
  printf ("\nSemaphore test 1000 enters and leaves.\n"); fflush (stdout);
  start_timing(temp);
  for (n=0; n<1000; n++) {
    semaphore_enter (sem);
    semaphore_leave (sem);
  };
  end_timing (temp);
}



/* 
  Test transmission of strings of different lengths .
  */
void length_test(int size)  
{
  
  caddr_t str = dk_alloc_box (size, DV_LONG_STRING);
  if (box_length (blob_resend (str)) != size)
    printf("stringtest with size = %d failed\n",size);
  else
    printf("stringtest with size = %d passed\n",size);
  
};


char * addr;

#ifdef UNIX
#include <errno.h>
extern int errno;

#else
#include <winsock.h>

#endif

void server_is_dead (dk_session_t *server)
{
  printf ("\n *** Server died at %s, exiting. Status =%d, errno =%d.\n", addr,server->dks_session->ses_status,
#if defined(WINNT) || defined(WINDOWS)
	  WSAGetLastError()
#else      
	  errno
#endif
	  );
  PrpcDisconnectAll();                                               
  dk_exit(1);
}

dk_thread_t * back_thread;
semaphore_t * back_sem;

char * srv_addr;


/*##**********************************************************************
 * 
 *              background
 * 
 * This function is started on a separate thhread by the main thread. It
 * connects to the server and carries out an independent dialogue with it
 *while the main thread is running the rest of the test. This function
 * intermittently sleeps and prints the real time each time the sleep ends. 
 * After a set number of iterations this terminates its thread.
 * 
 * Input params :       - The count of iterations before terminating itself.
 * 
 * Output params:    - 
 * 
 * Return value :    This function does not return.
 * 
 * Limitations  : 
 * 
 * Globals used :    none
 */


void DK_CALLBACK background (long count)
{
  dk_session_t * server = PrpcConnect (srv_addr, SESCLASS_TCPIP);
  timeout_t to;
  long n;
  long now = get_msec_real_time();
  to.to_sec = 0;
  to.to_usec = 500000;
  
  for (n = 0; n < count; n++) {
    long r = (long) PrpcSync (PrpcFuture (server, & fibo_service, 11));
    printf ("\n *** Background thread: It's %ld +, I got %ld.\n",
	    get_msec_real_time () - now, r);
    
    fflush (stdout);
    process_sleep (& to);
  };
  PrpcDisconnect (server);
  semaphore_leave (back_sem);
  /* Allow others to synchronize with this thread's termination. */
}


/*##**********************************************************************
 * 
 *              start_background
 * 
 * This creates an independent thread, assigns it a function and starts it.
 *  This returns after this, leaving the background thread tunning on its
 * own. * 
 * 
 * Input params :       - The count of iterations before terminating the 
 *                       background thread.
 * 
 * Output params:    - 
 * 
 * Return value :    void
 * 
 * Limitations  : 
 * 
 * Globals used :    back_thread
 */

void start_back_thread (long count)
{
  if (! back_sem) {
    back_sem = semaphore_allocate (1);
    back_thread = PrpcThreadAllocate (10000);
  };
  process_set_init_function (DK_THREAD_PROCESS (back_thread),
			     (init_func) background, (void *) count );
  semaphore_leave (DK_THREAD_SEM (back_thread));
  PROCESS_ALLOW_SCHEDULE ();
}


#define TEST_DIRECT
#ifndef WINDOWS

extern unsigned long last_future;

#endif

#if defined(WINDOWS_APP)
/* Windows App, not Quick Win */

int PASCAL WinMain(HANDLE hInstance, HANDLE hPrevInstance, LPSTR lpszCmdLine, int nCmdShow)
     
#else
     
     void main (int argc, char** argv)
     
#endif
{
  
  
  addr = "localhost 1177";
  srv_addr = "localhost 1111";
  if (argc > 1)
    addr = argv[1];

  if (argc > 2)
    srv_addr = argv[2];
 
  
  printf ("\nOwn listen address = %s, Server's address = %s", 
	  addr, srv_addr);
  
#ifdef WINDOWS
  PrpcSetThreadParams (10000, 10000, 10000, 1);
  
  /* The default is OK for others. */
#endif  
  PrpcInitialize();
  
  
  PrpcRegisterService ("blob_resend", (server_func) blob_resend,
		       NULL, DV_LONG_STRING, post_blobi);
  
  PrpcRegisterService ("direct_io_resend", (server_func) direct_io_resend,
		       NULL, DV_LONG_INT, NULL);
  
  
  
  /* Connect to the server application */
  
  server = PrpcConnect (srv_addr, SESCLASS_TCPIP);
  
  if (! DKSESSTAT_ISSET(server, SST_OK)) {
    printf ("\n*** Could not connect to server. Exiting."); fflush(stdout);
    dk_exit(0);
  };
  
  PrpcSetPartnerDeadHook (server, (io_action_func) server_is_dead);
  
  /* partial answers */
  
  printf ("\nTesting future with partial answers.\n");
  
  partial_answer_test(0);   
  
  partial_answer_test(10);
  
  
  /* very simple test, we use blob_resend so that we do not need to write 
     a special function for this test */
  
  sem_timing ();
  
  printf ("\n%d byte blob\n",BLOB_SIZE);
  
  blob_resend (dk_alloc_box(BLOB_SIZE, DV_LONG_STRING));
  
  
  
  /* simple tests */
  
  printf ("\n10 synchronous calls\n");
  sertest (10, 23, 1);
  
  printf ("\n50 asynchronous calls\n");
  
#ifndef WINDOWS
  start_back_thread (10);
#endif
  partest (50, 24, 1);
  
  PrpcDisconnect (server);
  printf ("*** Disconnect\n");
  
  server = PrpcConnect (srv_addr, SESCLASS_TCPIP);
  
  if (! DKSESSTAT_ISSET(server, SST_OK)) { 
    printf ("\n*** Could not connect to server. Exiting."); fflush(stdout);
    dk_exit(0);
  };
  
  printf ("Reconnected.\n"); fflush(stdout);
  
  
  /* Tests service_request_hook  */
  
  PrpcSetServiceRequestHook(service_request_hook_test);
  
  /* Become a server. Start listening. */
  
  listen_session = PrpcListen(addr, SESCLASS_TCPIP);
  
  if (! DKSESSTAT_ISSET (listen_session, SST_LISTENING)) {
    printf ("Failed to start listening");
    PrpcDisconnectAll();
    dk_exit(1);
  };
  listen2 = PrpcListen (addr, SESCLASS_TCPIP);
  /* Listen twive at same address. Do we get an error? */
#ifndef WINNT
  if (DKSESSTAT_ISSET (listen2, SST_LISTENING)) {
    printf ("Listened twice at the same address.");
    PrpcDisconnectAll();
    /*      dk_exit(1); */
  };
#endif
  
  self = PrpcConnect (addr, SESCLASS_TCPIP);
  
  if (! SESSTAT_ISSET(self->dks_session, SST_OK)) {
    printf ("\n*** Could not connect to itself. Exiting."); fflush(stdout);
    dk_exit(0);
  };
  
  
#ifndef WINDOWS
  /* two concurent blobs */
  
  printf ("\nBlob transfer %d bytes in 12 transfers\n", 12 * BLOB_SIZE);
  fflush(stdout);
  start_timing(t4);
  {
    caddr_t string_1 = (caddr_t) dk_alloc_box (BLOB_SIZE, DV_LONG_STRING);
    caddr_t string_2 = (caddr_t) dk_alloc_box (BLOB_SIZE, DV_LONG_STRING);
    future_t * bf1 = PrpcFuture (self, &blob_resend_service, string_1);
    future_t * bf2 = PrpcFuture (self, &blob_resend_service, string_2);
    PrpcValueOrWait (bf1);
    PrpcValueOrWait (bf2);
    PrpcFutureFree (bf2);
    PrpcFutureFree (bf1);
  };
  end_timing(t4);
  
#endif
  
  PrpcSetServiceRequestHook(NULL);
  
  
  /* timeout test */
  
  {
    caddr_t string = (caddr_t) dk_alloc_box(100, DV_SHORT_STRING);
    future_t * f;
    printf ("\nFuture with 10s timeout sent\n"); fflush (stdout);
    start_timing(t3);
    f = PrpcFuture (server, &blobi_1_service, string);
    PrpcFutureSetTimeout (f, 10000);
    PrpcValueOrWait (f);
    end_timing(t3);
    PrpcFutureFree (f);
  };
  
  
  
  /* Speed */
  
  
  printf ("\n500 synchronous calls\n");fflush (stdout);
  start_timing(t1);
  sertest (500,1,0);
  end_timing(t1);
  
  
  printf ("\n100 asynchronous calls\n");fflush (stdout);
  start_timing(t2);
  partest (100,1,0);
  end_timing(t2);
  
  printf ("\n100 asynchronous calls\n");
  start_timing(t2);
  partest (100,1,0);
  end_timing(t2);
  
  /* direct_io_future test */
  /* Open an other session to the server. */
  
  server2 = PrpcConnect (srv_addr, SESCLASS_TCPIP);
  
  if (! SESSTAT_ISSET(server2->dks_session, SST_OK)) {
    printf ("\n*** Could not make second connection to server. Exiting.");
    fflush(stdout);
    dk_exit(0);
  };
#ifndef WINDOWS
  printf ("\n Testing direct io transfer of blob. \n" );
  fflush(stdout);
  {
    long ok;
#ifdef UNIX
    caddr_t string = (caddr_t) dk_alloc_box (10*BLOB_SIZE, DV_LONG_STRING);
#else
    caddr_t string = (caddr_t) dk_alloc_box (BLOB_SIZE, DV_LONG_STRING);
#endif
    future_t * bf = PrpcFuture (self, &direct_io_resend_service, string);
    partest (500,1,0);
    ok = (long) PrpcValueOrWait (bf);
    PrpcFutureFree (bf);
    printf ("\n Received answer ok = %d.\n", ok);
    fflush(stdout); 
  };
#endif /* WINDOWS */
  
  /* test if server2 is ok after direct io. */ 
  {
    int val;
    future_t* f = PrpcFuture (server2, &fibo_service, 12L);
    val = (int) PrpcValueOrWait(f);
    printf ("\n server2 ok.\n" );
    fflush(stdout);    
  };
  
  
  
  /* Test request number range. 10000 32K, 64K */
  printf ("/bRequest bo. Ranges."); fflush (stdout);
#ifndef WINDOWS
  last_future = 9980;
#endif
  partest (50,1,1);
#ifndef WINDOWS
  last_future = ((long)1<<15) - 20;
#endif
  partest (50,1,1);
#ifndef WINDOWS
  last_future = ((long)1<<16) - 20;
#endif
  partest (50,1,1);
  
  /* Test strings of different lengths */
  length_test(255);
  length_test(256);
  length_test(257);
  length_test(127);
  length_test(128);
  length_test(129);
  length_test(511);
  length_test(512);
  length_test(513);
  
  
  
  printf ("\n Session \"server\": %ld bytes sent, %ld bytes received.\n",
	  server->dks_bytes_sent, server->dks_bytes_received);
  fflush(stdout); 
  
  printf ("\n Test ok.\n");
  fflush(stdout); 
  
  PrpcDisconnect (listen_session);
  PrpcDisconnect (self);
  PrpcDisconnect (server2);
  
  /* Kill server if more than two arguments */
  
  
  if (
#if defined(WINDOWS_APP) 
      1
#else
      (argc > 3)
#endif
      ) {
    future_t * kill  = PrpcFuture (server, &drop_dead_service, 0);
    PrpcFutureSetTimeout (kill, 3000);
    PrpcValueOrWait (kill);
    printf ("Killer future timed out"); fflush (stdout);
    /* The server is dead now and the connection should be disconnected 
       automatically */
    PrpcFuture (server, &drop_dead_service, 0);
    /* Write to a disconnected session. What happens? */
  } else {
    PrpcDisconnect (server);
  };
  
  
  PrpcDisconnectAll();
  /* Disconnect the other end of the session to self in case the 
     server thread did not notice self being disconnected. 
     The process may exit before this condition is detected, so it is
     best to make sure. */
  
  
  printf ("\nAll disconnected.\n"); fflush (stdout);
  /* Run the polling thread without any sessions to poll. See what happens. */
  process_allow_schedule ();
  process_allow_schedule ();
  
  printf ("Client Test Exiting.\n"); fflush (stdout);
  dk_exit (1);
}




