; ctags.cmd,	for MicroEmacs (c) Daniel Lawrence
;
; 22 Apr 89 S.D. Maley		Add %ctags_exact.  If user sets it TRUE
;			before invoking the to-tag macro, the search
;			will be case-sensitive.  Useful in C, but not
;			for most Fortran implementations.
; 19 Apr 89 S.D. Maley		Initial implementation.
;
;    Macros to make use of Unix-like ctags tag file.
; Assumes the existence of a file called "tags"
; (or whatever %tags_file is currently set to)
; which can be created using the "ctags" utility.
; If this file doesn't exist, the tags file is reported "Not found".
;    The format of each line of the tags file must be:
;
; tagword  filename  ?^pattern$?
;
; This is the normal output produced by the Unix utility "ctags".
;
;    For those unfamiliar with tagging, it is a form of hypertext.
; If you create a tags file for all your C source files you can locate
; the file and line where a given function is declared when you're in
; the editor, even if that function is declared in a different
; source file from the one you are editing.  Tagging will move you
; to that line in that file.
;
;    To use these macros, you can call "to-tag" directly as a command,
; or invoke it with the key combination M-<PgDn>.  Invoked directly,
; it will ask you to enter a tag.  Give the name of the function you
; are searching for (case sensitivity will depend on the current emacs
; setting).  If invoked with M-<PgDn>, the word where the cursor is
; positioned will be used as the tag to search for.

; to-tag
;
; Looks in tags file for given tag, gets the filename and search
; pattern to use in locating the tag, opens the given file and
; searches for the pattern.  If the tag isn't found, reports an
; error.  There is no error checking in this macro for a valid
; function name (tag).

write-message "[Loading ctags macros]"
store-procedure to-tag
   ;-- push down a level
   set %tag_lvl &add %tag_lvl 1
   set &ind &cat "%tag_b" %tag_lvl $cbufname
   set &ind &cat "%tag_c" %tag_lvl $curcol
   set &ind &cat "%tag_l" %tag_lvl $curline
   ;-- find out what we're looking for
   !if &equ &len %ctag 0
      set %prompt "tag: "
      set %ctag @%prompt
   !else
      ;-- align cursor to beginning of closest tag
      !if &equ 0 &sindex " ~t~n" &chr $curchar
         !force backward-character	;-- back up till we see whitespace
         !while &equ 0 &sindex " ~t~n" &chr $curchar
	    !if &not $status
               !break
            !endif
            !force backward-character
         !endwhile
      !endif
      !if &not &and &equ $curcol 0 &equ $curline 1 ;-- beginning-of-file
         !force forward-character	;-- !break apparently cleared $status
      !endif
      ;-- isolate the tag (trailed by whitespace or left-paren "(")
      set %ctag &chr $curchar	;-- necessary because macro 25
      !force forward-character	;-- may have set it non-null
      !while &equ 0 &sindex " ~t~n(" &chr $curchar
         !if &not $status
            !break ; a while
         !endif
         set %ctag &cat %ctag &chr $curchar
         !force forward-character
      !endwhile
   !endif

   ;-- See if it's in the tags file

   !force view-file %tags_file
   !if &not $status
      execute-procedure tag-out
      write-message &cat "File Not found: " %tags_file
      !return
   !endif
   beginning-of-file	;-- in case we've been this way before
   !if %ctags_exact
      add-mode exact	; case-sensitive search
   !endif
   add-mode magic		;-- enable regular expression searching
                                ;-- v match @ beginning of line
   !force search-forward &cat &cat "^" %ctag " "
   !if &not $status
      write-message &cat "Tag Not found: " %ctag
      execute-procedure tag-out
      !return
   !endif
   ;-- isolate the filename (trailed by whitespace)
   set %tagged_file &chr $curchar
   forward-character
   !while &equ 0 &sindex " ~t" &chr $curchar
      set %tagged_file &cat %tagged_file &chr $curchar
      forward-character
   !endwhile
   ;-- isolate the regular expression
   next-word	; skips leading "?"
   set %tagged_regex #$cbufname
   set %tagged_regex &left %tagged_regex &sub &len %tagged_regex 2

   ;-- get on down

   !force view-file %tagged_file
   !if &not $status
      execute-procedure tag-out
      write-message &cat "File Not found: " %tagged_file
      !return
   !endif
   beginning-of-file	;-- in case we've been this way before
   !force search-forward %tagged_regex
   !if $status
      write-message "<Esc><PgUp> to pop back"
   !else
      execute-procedure tag-out
      write-message &cat "Tag Declaration Not found:~n" %tagged_regex
      write-message "~ntags file should be re-generated with ctags"
      !return
   !endif
   set %ctag ""
!endm

; tag-out
;
; puts us back where we were before invoking tag
;
store-procedure tag-out
   !if &less 0 %tag_lvl		;-- "&gre %tag_lvl 1" doesn't work
      ;-- pop out a tag level

      select-buffer &ind &cat "%tag_b" %tag_lvl
      goto-line &ind &cat "%tag_l" %tag_lvl
      set $curcol &ind &cat "%tag_c" %tag_lvl
      set %tag_lvl &sub %tag_lvl 1
   !endif
   set %ctag ""
!endm

set %ctag ""
set %ctags_exact FALSE	;-- set TRUE for case-sensitive searches
set %tag_lvl 0
set %tags_file "tags"	;-- this can be reset from within Emacs

25 store-macro
   set %ctag find
   execute-procedure to-tag
!endm

26 store-macro
   clear-message-line
   execute-procedure tag-out
!endm

bind-to-key execute-macro-25 M-FNQ	;-- arbitrary macro number, <Esc><PgDn>
bind-to-key execute-macro-26 M-FNI	;-- arbitrary macro number, <Esc><PgUp>
