/***************************************************************
*
*
*   ShareCom 1.0   
*    Copyright (C)  1994 Identity Software Inc
*
* dkses.h
* 
* Portable Session API
*
*/


#ifndef DKSES_H
#define DKSES_H


/* Interface to a certain communication device */
typedef struct devicestruct device_t;

/* Whenever needed, specify as you like */
typedef struct semaphorestruct  semaphore_t;

/* General type for specifying timeout values for select, read and write */
typedef struct {
	long to_sec;    /* seconds */
	long to_usec;   /* and microseconds */
} timeout_t;

typedef struct {
	int         ctrl_blocking;      /* 1 if read/write blocking */     
	timeout_t*  ctrl_timeout;       /* s+us value for read/write */
	int         ctrl_msg_length;    /* max length of single message */
} control_t;


#define SESSION_IN_BUFFER_LENGTH 1024
#define SESSION_OUT_BUFFER_LENGTH 1024


/* General session object */

typedef struct sessionstruct {

	int         ses_class;          /* internal info, TCPIP, NMP, ... */

	/* Properties set by session_set_control() */
	control_t*  ses_control;

	/* Return fields set by session_ functions */
	int         ses_bytes_read;         /* updated by read */
	int         ses_bytes_written;      /* updated by write */
	int         ses_status;             /* single bit SST_ flags */

	/* Device dependent part */
	device_t*   ses_device;

	/* This is reserved for two way link to struct of type dk_session_t. */
	void*       ses_client_data;

} session_t;



/* Session "methods" */

#ifdef NO_ANSI

session_t*  session_allocate();
int         session_free();

int         session_set_address();
int         session_listen();
int         session_accept();
int         session_connect();
int         session_disconnect();
int         session_write();
int         session_read();
int         session_select();

int         session_set_control();
int         session_get_control();

int         session_set_default_control();
int         session_get_default_control();

#else /* NO_ANSI */

session_t*  session_allocate(int seslass);
int         session_free(session_t* ses);

int         session_set_address(session_t* ses, char* addrinfo);
int         session_listen(session_t* ses);
int         session_accept(session_t* ses, session_t* new_ses);
int         session_connect(session_t* ses);
int         session_disconnect(session_t* ses);
int         session_write(session_t* ses, char* buffer, int n_bytes);
int         session_read(session_t* ses, char* buffer, int n_bytes);
int         session_select(int ses_count,
			    session_t** reads,
			    session_t** writes,
			    timeout_t*  timeout);

int         session_set_control(session_t* ses, int field, char* p_value, int size);
int         session_get_control(session_t* ses, int field, char* p_value, int size);

int         session_set_default_control(int field, char* p_value, int size);
int         session_get_default_control(int field, char* p_value, int size);

#endif /* NO_ANSI */


/* Macros for specifying wanted action to session_control() */

#define SC_BLOCKING 1
#define SC_TIMEOUT  2
#define SC_MSGLEN   3

/* Macros defining default values for session's control fields */

#define SDC_BLOCKING    1           /* 1=yes, 0=no */
#define SDC_TIMEOUT     {0, 0}      /* sec, usec   */
#define SDC_MSGLEN      512


/* Session status flags */

/* DANGER! sizeof(int) is processor dependent, so we just reserve 16 flags */
#define N_SSTBITS   16  
#define SSTBIT(n)   (1 << ((n) % N_SSTBITS)) 

#define SST_OK                  SSTBIT(0)  /* Previous operation succesfull */
#define SST_BLOCK_ON_WRITE      SSTBIT(1)  /* Write would have blocked */
#define SST_BLOCK_ON_READ       SSTBIT(2)  /* Read would have blocked */
#define SST_BROKEN_CONNECTION   SSTBIT(3)  /* Remote party down or unreachable */
#define SST_TIMED_OUT           SSTBIT(4)  /* Blocking oper timed out */
#define SST_NO_PARTNER          SSTBIT(5)  /* No process listening in given addr */
#define SST_ADDR_USED           SSTBIT(6)  /* Address used by another process */
#define SST_CONNECT_PENDING     SSTBIT(7)  /* Client issued connect */
#define SST_INTERRUPTED         SSTBIT(8)  /* oper interrupted by a signal */
#define SST_LISTENING           SSTBIT(9)  /* this is a listening session */

/* Macros for setting, clearing and testing session status flags */

#define SESSTAT_SET(ses, stat)     ((ses)->ses_status |= stat)     
#define SESSTAT_CLR(ses, stat)     ((ses)->ses_status &= ~stat)
#define SESSTAT_ISSET(ses, stat)   ((ses)->ses_status & stat)


/* Return codes of session functions */

#define SER_SUCC     0  /* Operation succeeded */
#define SER_FAIL    -1  /* Operation failed, reason unspecified */
#define SER_ILLPRM  -2  /* illegal parameter */
#define SER_ILLSESP -3  /* illegal session pointer */
#define SER_SYSCALL -4  /* System call failed */
#define SER_NOREC   -5  /* no recources available */
#define SER_ILLADDR -6  /* illegal address specified */
#define SER_MSGSIZE -7  /* message was too long to be sent all at once */
#define SER_CNTRL   -8  /* mode controlling failed */
#define SER_ILLCL   -9  /* unrecognized session class */
#define SER_INTR    -10 /* external interrupt or signal */
#define SER_NOSUP   -11 /* operation is not supported */

/* Macros for distincting communication devices */

#define SESCLASS_TCPIP      0
#define SESCLASS_NMPIPE     1
#define SESCLASS_NETBIOS    2
#define SESCLASS_STRING     4
#define SESCLASS_UDPIP      7

#endif /* DKSES_H */



