/***********************************************************
*
*   ShareCom 1.0
*    Copyright (C)  1994 Identity Software Inc
*
* semaphor.h
* 
* Portable Semaphores
*
*/




#ifndef THREAD_INC
#define THREAD_INC


#include <sysincs.h>
#include <dksetjmp.h>
#include <dkses.h>



#include <sys/types.h>


#ifdef OS2
/* mty added 10.6.92 */
typedef char * caddr_t;
#endif


typedef void (* init_func)(void * arg);
typedef void (* interrupt_func)(void * arg);
typedef int (* wait_func)(void * arg);


#define BREAK_STACK_LIMIT 0xffffffff
#define SCHEDULE_STACK_LIMIT 0xfffffffe


/* 
  struct interrupt_record

  This record represents a pendinf interrupt on a thread
  The interrupt is represented by a function and an argument
*/


typedef struct interrupt_record {
  interrupt_func func;
  void * arg;
  struct interrupt_record *next;
} interrupt_record;



/* 
  struct STackDesc

  Represents the low level data on a thread
*/



#define MAX_NESTED_FUTURES 20
#define MAX_INTERRUPTS 20
#define MAX_MASKED 10




typedef struct _StackStruct
{
  int                   thr_IsActive;
  int                   thr_IsAllocated;
  void *                thr_Bottom;
  void *                thr_Top;
  long                  thr_StackLimit;
  jmp_buf_splice        thr_Context;
  jmp_buf_splice        thr_init_context;
#ifdef SUN_LWP
  thread_t              thr_tid;
  cv_t                  thr_sv;
#endif
  int                   thr_to_preset;
  init_func             thr_initial_function;
  void *                thr_initial_argument;
  int                   thr_status;
  wait_func             thr_wait;
  void *                thr_wait_arg;
  interrupt_func        thr_interrupt_mask[MAX_MASKED];
  interrupt_record *    thr_interrupts;
  void *                thr_client_data;
  struct semaphorestruct * thr_sem;
  struct _StackStruct * thr_next_waiting;
#ifdef OS2
  TID                   thr_tid ;
  ULONG                 thr_hsem ; /* = 0L */
  ULONG                 thr_initsem ; /* = 0L */
#endif
#ifdef WINNT
  HANDLE thr_hthread;
  long  thr_id_thread;
#endif

  struct dkhash * thr_attributes;
} du_thread_t;


#define THR_ATTR(th,a) gethash((void *)a, (dk_hash_t *) ( th->thr_attributes))
#define SET_THR_ATTR(th,a,v) sethash((void *)a,  th->thr_attributes,(void *)v)


#define RUNNABLE 1
#define STOPPED 0


#ifdef SCO
#define MAX_THREADS 12
#else
#define MAX_THREADS 20
#endif

extern du_thread_t  threads [MAX_THREADS];

#ifndef OS2
extern du_thread_t * current_process;
#endif

extern long tic_counter;
extern int time_slice;

extern long StackLimit;

/* Function Prototypes */


extern int process_allow_schedule (void ); /* mty MAALIS */
extern du_thread_t *next_runnable ();
extern void process_sleep(timeout_t * timeout);           
extern void process_enable (du_thread_t * thread);
extern void process_disable (du_thread_t * thread);
extern int process_reset (du_thread_t * thread);
extern void process_interrupt (du_thread_t * thread, interrupt_func func, void * arg);
extern void process_wait (du_thread_t* proc, wait_func func, void * arg);
extern void du_thread_init();
extern du_thread_t * process_allocate (long sz);
extern void process_set_init_function (du_thread_t* thread,
				      init_func in_fun, void *in_arg);

extern void stack_overflow();


/* Macros */


#define DK_RETURN_FROM_CRITICAL(arg)     \
{ dk_exit_critical(); return(arg); }

#define DK_RETURN_FROM_CRITICAL_2(dtp,arg)     \
{dtp v_; v_=arg; dk_exit_critical(); return(v_); }




/* Use check_switch when task switching is allowable
   this will switch if there has been a scheduling signal
   since last switch.  This will also detect stack overflows
   on systems that do not check it in hardware.
*/

extern int atomic_ctr;


#ifndef PREEMPT

#define dk_enter_critical() \
  atomic_ctr ++;

#define dk_exit_critical()     \
  atomic_ctr --;             \
  if (atomic_ctr <= 0) {     \
    atomic_ctr = 0;          \
    check_switch();          \
  }



#endif /* no PREEMPT */

#ifdef WINNT
#define dk_enter_critical()
#define dk_exit_critical()
#endif


/* MARRAS 9.11.92 mty */
#ifndef PREEMPT

#ifdef WINDOWS


#define DLL_ENTRY __export

#define check_switch() process_allow_schedule();
/* Windows is non preemptive but does not have timer signals. 
   Therefore switch always */
#else

#define check_switch()              \
{                                   \
  int dum__;                        \
  if ((unsigned long)&dum__ < (unsigned long)StackLimit)      \
    stack_overflow();              \
}
#endif

#else

#define check_switch() ;

#endif  /*PREEMPT*/
#endif

#ifdef WINDOWS
extern du_thread_t * get_current_process();
#define THREAD_CURRENT_THREAD get_current_process()
#else
#define THREAD_CURRENT_THREAD current_process
#endif

#define THREAD_ALLOW_SWITCH() check_switch()
#define PROCESS_ALLOW_SCHEDULE() process_allow_schedule()


#if defined(AIX) || defined(SUNOS)
#define USING_TIKS
#endif



/* mty HUHTI */
#if defined (DEFOS2) || defined (WINDOWS)


#define without_scheduling_tic() 
#define restore_scheduling_tic() 
/* oui 21.4.93 */


#elif defined (USING_TIKS)


#define without_scheduling_tic() \
  {  \
    int oldsig = sigblock(sigmask(SIGALRM));

#define restore_scheduling_tic() \
    sigsetmask(oldsig); \
  };

#else


#define without_scheduling_tic()  

#define restore_scheduling_tic()

#endif


#ifdef WINNT

du_thread_t * get_current_process ();

#define current_process get_current_process()

#define THREAD_CURRENT_THREAD  current_process

#define THREAD_ALLOW_SWITCH()

#define PROCESS_ALLOW_SCHEDULE() process_allow_schedule();


#endif


#ifdef PREEMPT
#define IF_PREEMPT(x) x;
#define dk_enter_critical_if() dk_enter_critical()
#define dk_exit_critical_if() dk_exit_critical()

#define DK_RETURN_FROM_CRITICAL_IF(arg)     \
  DK_RETURN_FROM_CRITICAL (arg)

#define DK_RETURN_FROM_CRITICAL_IF_2(dtp,arg)     \
    DK_RETURN_FROM_CRITICAL_2(dtp,arg)



#else
#define IF_PREEMPT(x)

#define dk_enter_critical_if() 1
#define dk_exit_critical_if() 1

#define DK_RETURN_FROM_CRITICAL_IF(arg)     \
  return(arg)

#define DK_RETURN_FROM_CRITICAL_IF_2(dtp,arg)     \
  return (arg)



#endif

#ifdef PREEMPT
#define IF_PREEMPT(x) x;
#define dk_enter_critical_if() dk_enter_critical()
#define dk_exit_critical_if() dk_exit_critical()

#define semaphore_enter_if(s) semaphore_enter(s)
#define semaphore_leave_if(s) semaphore_leave(s)

#define DK_RETURN_FROM_CRITICAL_IF(arg)     \
  DK_RETURN_FROM_CRITICAL (arg)

#define DK_RETURN_FROM_CRITICAL_IF_2(dtp,arg)     \
    DK_RETURN_FROM_CRITICAL_2(dtp,arg)



#else
#define IF_PREEMPT(x)

#define dk_enter_critical_if() 1
#define dk_exit_critical_if() 1

#define semaphore_enter_if(s) 1
#define semaphore_leave_if(s) 1

#define DK_RETURN_FROM_CRITICAL_IF(arg)     \
  return(arg)

#define DK_RETURN_FROM_CRITICAL_IF_2(dtp,arg)     \
  return (arg)



#endif

#ifndef DLL_ENTRY

#define DLL_ENTRY

#endif
