//  FPMATH.H
//
//  Macros to provide easier interface to Win87Em's __fpmath() function.
//
//  __fpmath is Win87Em's "multiplexed" entry point: it provides multiple
//  services to control the emulator DLL.
// 
//  __fpmath is really intended to be called by assembly language code, 
//  and consequently isn't well-suited to being called from C because it 
//  expects its parameter to be passed in the BX register.  However, it 
//  does return its result in AX, so except for function 3 we can make it
//  *look* like a normal C function with a little inline _asm massaging.
//  Function 3 is a problem because it expects the 32-bit address of a
//  signal handler in DS:AX registers. 
//
//  These macros assume Microsoft C 6/7 inline _asm capability.
//  
//  See SDK documentation for _fpmath().

void FAR _cdecl _fpmath(void);
typedef void (FAR *LPFNSIGNALPROC)(int, int);
typedef void (*PFNSIGNALPROC)(int, int);
LPFNSIGNALPROC FAR PASCAL _FPInit(void);
VOID FAR PASCAL _FPTerm(LPFNSIGNALPROC);

// Win87Em's equivalent of FSTCW -  Puts control word in wFCW paramater.
#define FPMATH_FSTCW(wFCW)  \
{                           \
    { _asm mov bx, 5 };     \
    _fpmath();              \
    { _asm mov wFCW, ax };  \
}

// Win87Em's equivalent of FLDCW -  Puts wFCW parameter in control word.
#define FPMATH_FLDCW(wFCW)  \
{                           \
    { _asm mov ax, wFCW };  \
    { _asm mov bx, 4 };     \
    _fpmath();              \
}

#define FPMATH_SETSIGNAL(lpfnHandler)       \
{                                           \
    { _asm push bx };                       \
    { _asm push dx };                       \
                                            \
    { _asm mov dx, SEG lpfnHandler };       \
    { _asm mov ax, OFFSET lpfnHandler };    \
    { _asm mov bx, 3 };                     \
    _fpmath();                              \
                                            \
    { _asm pop dx };                        \
    { _asm pop bx };                        \
}
