/***************************************************************
*
*
*   ShareCom 1.0   
*    Copyright (C)  1994 Identity Software Inc
*
*
*  Futures RPC Data Structures
*
*/



#ifndef FUTURE_H
#define FUTURE_H

#include <sysincs.h>


#if defined(WINDOWS) || defined(WINNT)
typedef char* caddr_t;
typedef unsigned long u_iong;
#define TRUE 1
#define FALSE 0
#endif

/* for dbg_printf().
#define PRINT_DEBUG
#define LEVEL_VAR 4

*/

/* The future and process structures */


#include <dkses.h>


#include <signal.h>
#include <sys/types.h>


#ifdef OS2
/* mty added 10.6.92 */
typedef char * caddr_t;
#endif

#define MAX_SESSIONS 100


#include <dkthread.h>

#include <semaphor.h>
#include <basket.h>

#include <dk-util.h>
#include <messages.h>

#define SESCLASS_STRING 4

#define DKSES_IN_BUFFER_LENGTH  1024
#define DKSES_OUT_BUFFER_LENGTH 1024


typedef struct bufelt {
  char *data;
  int fill;
  int read;
  struct bufelt * next;
} buffer_elt_t;



   
typedef struct dk_sessionstruct {

   session_t *     dks_session;

   semaphore_t*    dks_semaphore;   /* Used by tasking system */
   
   char *          dks_in_buffer;
   int             dks_in_length;
   int             dks_in_fill;
   int             dks_in_read;

   buffer_elt_t *          dks_buffer_chain;

   char *          dks_out_buffer;
   int             dks_out_length;
   int             dks_out_fill;
   int             dks_out_written;  /* Used by single-thread router */

   void*           dks_client_data;  /* Used by scheduler */
   void*           dks_object_data;  /* Used by Distributed Objects */
   void*           dks_object_temp;  /* Used by Distributed Objects */
   unsigned long   dks_bytes_sent;   /* Used by Administration server */
   unsigned long   dks_bytes_received;/* Used by Administration server */

                  /* Used by router */
   struct dk_sessionstruct * dks_connected_to;
   int            dks_to_be_closed;
   char *         dks_route_address;  /* Used by router for a listening 
                     session Otherwise unused */
   int            dks_route_protocol; 

   char *         dks_peer_name;
   char *         dks_own_name;

   void *        dks_dbs_data;
} dk_session_t;

#define SESSION_DK_SESSION(session) (*((dk_session_t **) (&((session)->ses_client_data))))

#define DKSESSTAT_ISSET(x,y) SESSTAT_ISSET(x->dks_session, y)


/* 
  struct dk_thread_t

  This represents the high level data on a thread.
  The action of the thread is represented by the initiak_funcion and
  initial_argument.

  The status is either RUNNABLE or STOPPED. These tell the scheduler 
  whether to resume the thread when it comes to the execution turn.

  The wait_func and wait_arg allow temporarily setting the thread in busy wait. 
  The scheduler will call this function if it is supplied before
  resuming the thread. If the wait_func returns true the thread is resumable.

  The array of future requests represents the future computations in progress.
  Computations may be nested if an outgoing service request 
  causes a follow up request on the server. If no parallel threads are 
  available the nested request is scheduled onto the ancestor thread.
*/


typedef struct _thread {
  du_thread_t *  dkt_process;
  int            dkt_request_count;
  struct _srv  * dkt_requests [MAX_NESTED_FUTURES];

} dk_thread_t;


#define DK_THREAD_PROCESS(x) (x->dkt_process)
#define DK_THREAD_SEM(x) (x->dkt_process->thr_sem)



/*  During the execution of a future service function this return
    the session through which the request came. This would belong to 
    the future.h file 
*/




/* 
  struct future_request_t

  This represents a request received by a dis kit server.

  The condition is the number of the request.

*/

#define MAX_FUTURE_ARGUMENTS 10
 
#ifdef __cplusplus
typedef char * (* server_func)(char *, ...);
#else
typedef char * (* server_func)(caddr_t x, ...);
#endif

#ifndef WINDOWS
#define __loadds
#define DK_CALLBACK

#else

#define DK_CALLBACK __loadds

#endif



#ifndef __cplusplus
typedef struct _srv
#else
struct _srv
#endif

{
   struct service_t *      rq_service;
   jmp_buf_splice          rq_start_context;
   long **                 rq_arguments;
   dk_session_t *          rq_client;
   long                    rq_condition;
   int                     rq_ancestor_count;
   struct _srv **          rq_ancestors;
   dk_thread_t *           rq_thread;
   struct _srv *           rq_next_waiting;
}
#ifndef __cplusplus
future_request_t;
#else
;
typedef _srv future_request_t ;
#endif

/* TAMMI mty needed for C++ strong type checking */
typedef void  (* post_func)(caddr_t b, future_request_t *f) ;


typedef unsigned char dtp_t;
/* dtp_t   added oui 11.6.93 */

/* 
  struct service_desc_t

  This structure represents a sevice that a dis kit client uses.
  This structure exists on the client. The arg_types is an array
  that holds values from the DV<xxx> group. See messages.h.
*/

typedef struct _sd {
  char *  sd_name;
  int     sd_arg_count;
  long *  sd_arg_types;
  int     sd_type;
  dtp_t   sd_return_type;
} service_desc_t;



#define SERVICE_0(name1, str, send, ret_dtp)    \
  service_desc_t name1 = {str, 0, (long *)NULL, send, ret_dtp};


#define SERVICE_1(name1,name2, str, send, ret_dtp, a1)    \
  long  name2 [1] = {a1};  \
  service_desc_t name1 = {str, 1, name2, send, ret_dtp};


#define SERVICE_2(name1,name2, str, send, ret_dtp, a1,a2)    \
  long  name2 [2] = {a1,a2};  \
  service_desc_t name1 = {str, 2, name2, send, ret_dtp};


#define SERVICE_3(name1,name2, str, send, ret_dtp, a1,a2,a3)    \
  long  name2 [3] = {a1,a2,a3};  \
  service_desc_t name1 = {str, 3, name2, send, ret_dtp};


#define SERVICE_4(name1,name2, str, send, ret_dtp, a1,a2,a3,a4)    \
  long  name2 [4] = {a1,a2,a3,a4};  \
  service_desc_t name1 = {str, 4, name2, send, ret_dtp};

#define SERVICE_5(name1,name2, str, send, ret_dtp, a1,a2,a3,a4,a5)    \
  long  name2 [5] = {a1,a2,a3,a4,a5};  \
  service_desc_t name1 = {str, 5, name2, send, ret_dtp};

#define SERVICE_6(name1,name2, str, send, ret_dtp, a1,a2,a3,a4,a5,a6)    \
  long  name2 [6] = {a1,a2,a3,a4,a5,a6};  \
  service_desc_t name1 = {str, 6, name2, send, ret_dtp};

#define SERVICE_7(name1,name2, str, send, ret_dtp, a1,a2,a3,a4,a5,a6,a7)    \
  long  name2 [7] = {a1,a2,a3,a4,a5,a6,a7};  \
  service_desc_t name1 = {str, 7, name2, send, ret_dtp};



/*
  struct future_t 

  This represents a pending or complete future request on the client.
  The request_no matches the condition member in the server's 
  future_request_t struct.

*/

#define FE_TIMED_OUT 1
/* Value stored in error member of future_t struct
   upon time out */

typedef struct _frq {
  dk_session_t*         ft_server;
  long                  ft_request_no;
  service_desc_t*       ft_service;
  caddr_t *             ft_arguments;
  caddr_t               ft_result;
  caddr_t               ft_error;
  int                   ft_is_ready;
  timeout_t             ft_timeout;
  timeout_t             ft_time_issued;
  timeout_t             ft_time_received;
  future_request_t *    ft_waiting_requests;
} future_t;


/* The client representation of a future */


/* Scheduler Data structures */




#ifdef OS2  /* MARRAS 4.11.92 mty below 4 lines */
#define MAX_FUTURE_THREADS 12
#define FUTURE_THREAD_SIZE 12000 /* max 64 K under OS/2 */
#else

#ifdef SCO
#define MAX_FUTURE_THREADS 7
#define FUTURE_THREAD_SIZE 30000

#else
#define MAX_FUTURE_THREADS 12
#define FUTURE_THREAD_SIZE 120000
#endif
#endif

extern dk_thread_t * PrpcThreadAllocate (long sz);

extern du_thread_t * server_process;


extern int max_future_threads;




#define DV_SEND_NO_ANSWER 1
/* This is used as the return type of the 
   service if the service is not to send ANY automatic 
   message to the client. NO DA_FUTURE_ANSWER MESSAGE WILL BE
   SENT IF THIS IS THE RETURN_TYPE OF THE SERVICE. The service still
   may itself  compose and send  a DA_FUTURE_ANSWER message.

   Otherwise this may be one of the DV_<xxx> constants whereupon 
   the appropriate return message is built and sent as part of
   the DA_FUTURE_ANSWER message.
*/

#define DV_MULTIPLE_VALUES 2
/* 
  This indicates that the future returns an array
  of pointers that is considered as multiple values by an Identity
  client.  This array is deallocated after sending.
*/






/* 
  struct service_t

  This represents a service offered by a server on the server.

  The return type is a value in the DV<xxx> group.
*/

typedef struct service_t {
  char *                sr_name;
  void*                 sr_client_data;
  server_func           sr_func;
  post_func           sr_postprocess;
  int                   sr_return_type;
  struct service_t*     sr_next;
} service_t;





/* 
  struct scheduler_io_data_t

  This is an extension to the dk_session_t structure used to hold scheduling data.

  The default_read_ready_action is invoked when the session is ready for input and
  there is no random_read_ready_action. If there is a random_write_ready_action
  it is invoked when the session is ready for output.

  The random actions are mostly used when the session blocks. The scheduler 
  The thread then stops and the scheduler will wake it up when the blocking condition
  is part. Hence the reading and writing threads that are passed to the above functions.

  See dk-io.c unfreeze_<xx>().

*/


typedef int (* io_action_func)(dk_session_t *ses);

typedef struct {
  io_action_func        sio_default_read_ready_action;
  io_action_func        sio_random_read_ready_action;
  io_action_func        sio_random_write_ready_action;
  du_thread_t *         sio_writing_thread;
  du_thread_t *         sio_reading_thread;
  int                   sio_is_served;
   /* Index in served sessions table, if this is there */
  int                   sio_is_regular_input;
   /* true if regularly checked for client input */
  io_action_func        sio_partner_dead_action;
#ifdef DEFOS2 /* TAMMI mty */
  jmp_buf       sio_read_broken_context;
  jmp_buf       sio_write_broken_context;
#else
  jmp_buf_splice        sio_read_broken_context;
  jmp_buf_splice        sio_write_broken_context;
#endif
  void *                sio_client_data;     /* For application use */
} scheduler_io_data_t;




#define SESSION_SCH_DATA(ses) \
  (*((scheduler_io_data_t **) (&((ses)->dks_client_data))))




#define box_tag(box) \
  (*((dtp_t *) &(((unsigned char *)box)[-1])))

#define BOX_ELEMENTS(b) (box_length ((caddr_t) (b)) / sizeof (caddr_t))

#define TAG_FREE 0
#define TAG_BEING_FREED 1
#define TAG_BOX 2

#define TAG_PRESENT 10
#define TAG_NON_PRESENT 11


#define SMALLEST_POSSIBLE_POINTER 10000
/* We allow pointer arrays to contain small integers 
   for the sake of convenience. Even is a segmented 
   architecture this would be an invalid pointer.

   If the configuration has distributed objects they do
   not get deleted in dk_delete_tree.
*/

/*
  Test if the item should be bozed when appearing in an array
  of pointer. Null is a pointer (= nil). Small positive and 
  negative number are not pointers
*/

#define IS_POINTER(n) \
  (((long) n) >= SMALLEST_POSSIBLE_POINTER ||    \
   ((long) n) <= -SMALLEST_POSSIBLE_POINTER ||   \
   ((long) n) == NULL)


/* 
  IS_BOX_POINTER.  Test of the item is a non null pointer
*/
#define IS_BOX_POINTER(n) \
  (((long) n) >= SMALLEST_POSSIBLE_POINTER ||    \
   ((long) n) <= -SMALLEST_POSSIBLE_POINTER)



#ifndef true
#define true 1
#define false 0
#endif


#ifndef MIN 
#define MIN(x,y) \
  ((x < y) ? x : y)
#endif



typedef void (* background_action_func)();


#define PUT_BOX(to, data, dtp, doubleac)   \
      if (dtp == DV_LONG_INT)              \
   to = (caddr_t)box_num ((long) data);               \
      else if (dtp == DV_C_STRING)         \
  to = (caddr_t) box_string ((char *)  data);            \
      else if (dtp == DV_SINGLE_FLOAT)     \
   to = (caddr_t) box_float (*(float *)(&data)); \
      else if (dtp == DV_DOUBLE_FLOAT) {       \
   to = (caddr_t)box_double (*(double *)(&data)); doubleac; }   \
      else to = (caddr_t)data;



#ifndef DKDO
#define IS_PERSISTENT(x) 0
#endif


/* #define SRV_DEBUG */

#ifdef SRV_DEBUG
extern int srv_debug_level;
extern int errno;
#endif


#define PROCESS_TO_DK_THREAD(p) \
  ((dk_thread_t * )((p)->thr_client_data))



#define DK_CURRENT_THREAD ((dk_thread_t *)(THREAD_CURRENT_THREAD->thr_client_data))

#define IMMEDIATE_CLIENT \
  (DK_CURRENT_THREAD -> dkt_requests [DK_CURRENT_THREAD -> dkt_request_count -1] \
  -> rq_client)

#ifdef DEFOS2 /* TAMMI mty */
//#define current_process get_current_thread()
#define current_process find_thread( get_tid() )
#endif



/*  possible values of ft_is_ready flag */

#define FS_SINGLE_COMPLETE 1
#define FS_RESULT_LIST 2
#define FS_RESULT_LIST_COMPLETE 3
#define FS_FALSE 0

#define FUTURE_IS_READY(future) (future->ft_is_ready)

#define FUTURE_IS_NEXT_RESULT(future) (future->ft_result)

#define FUTURE_IS_EXHAUSTED(future) \
  (!(future->ft_result) && (future->ft_is_ready == FS_RESULT_LIST_COMPLETE))

#define FUTURE_IS_TIMED_OUT(f) (f->ft_error == FE_TIMED_OUT)


/* Future results are of type DV_ARRAY_OF_POINTER.
   usualy we are only in the first one. */

#define FUTURE_RESULT_FIRST(future_result) \
  ((caddr_t) unbox (((caddr_t *)(future_result))[0]))


#define SESSION_CHECK_OUT(session) \
  SESSION_SCH_DATA (session) -> sio_default_read_ready_action = NULL
/* The session will not be listened to by check_inputs() */


#define SESSION_CHECK_IN(session) \
  SESSION_SCH_DATA(session) ->sio_default_read_ready_action \
  = read_service_request
/* Re-enable services reading */



#define TIMEOUT_TO_MILLISECONDS(timeout) \
  (timeout.to_sec * 1000 + timeout.to_usec / 1000);

#define SET_TIMEOUT_TO_MILLISECONDS(timeout, milliseconds) \
  (timeout.to_sec = (milliseconds / 1000), \
   timeout.to_usec =  (milliseconds % 1000));



typedef caddr_t (* srv_req_hook_func)(dk_session_t* session, caddr_t request);

typedef void (* sch_hook_func)();


typedef caddr_t(* macro_char_func)(dk_session_t * ses, char macro);


typedef int (* printer_ext_func) (caddr_t obj, dk_session_t *ses, void * ext, 
              caddr_t ea);

extern int srv_write_in_session (caddr_t thing, dk_session_t * session,
          printer_ext_func extension, void* xx);




typedef int ( * ses_write_func)(caddr_t thing, dk_session_t * session,
            printer_ext_func extension, void * ea);







#include <rpcapi.h>

#endif /* future.h */



