; HIDE87.ASM
;
; "Hide the 80x87 from software sight ... sometimes."
;
; Terminate and Stay Resident code that hooks Int 11h (BIOS equipment check)
; to fool some programs (notably Windows' Kernel) into thinking there isn't 
; a math coprocessor.  It does this by masking bit 1 of the equipment list 
; code word.  Once installed, it cannot be removed from memory ... you must 
; reboot to unhook it.  However, it can be deactivated and reactivated with 
; by issuing Int 11h with:
;
;       AX = 444Ch      turn off mask (let coprocessor be seen)      
;       AX = 646Ch      turn on mask  (hide coprocessor)
;
; HIDE87.COM is a dual-purpose program:
;       1.  If the TSR isn't installed, install it and begin masking.
;       2.  If TSR is installed, [re-]enable masking.
; 
; SHOW87.EXE is the companion program.  It disables masking so the 80x87
; can be seen.
;
; There is no harm in using this program (along with SHOW87) on a machine 
; that does not have a coprocessor, but there will be no change in 
; behavior because the BIOS already returns 0 for the coprocessor bit.
;
; To affect Windows' awareness of the presence|absence of a coprocessor,
; Windows must be [re-]started after running HIDE87 or SHOW87.

; COPYRIGHT:
;
;   (C) Copyright Microsoft Corp. 1993.  All rights reserved.
;
;   You have a royalty-free right to use, modify, reproduce and
;   distribute the Sample Files (and/or any modified version) in
;   any way you find useful, provided that you agree that
;   Microsoft has no warranty obligations or liability for any
;   Sample Application Files which are modified.
;        
        
cr      equ     0dh
lf	equ	0ah

        .MODEL tiny, pascal, os_dos
        .STACK
        .CODE
        ORG     5Dh

        .STARTUP
        jmp short Install               ; Jump over ISR code and data

; Place data in the code segment so it won't be discarded with install code.
OldInt11Proc    DWORD   ?               ; Address of original Int 11h ISR.
fHide87         WORD    1               ; Keeps track of current "hide
                                        ; state"; initially hidden.

;*************************************************************************
; NewInt11Proc simply calls the original Int 11h handler, then tweaks the
; bits of its return value.  The BIOS returns an "equipment list control
; word" in the AX register.  Bit 1 has a 1 if a coprocessor is present.
; By AND-ing AL with 11111101, bit 1 gets cleared and software that
; pays attention to this will think there is no coprocessor.

NewInt11Proc    PROC    FAR
        jmp short skip_marker
MagicMarker     DWORD   444C5748h       ; Location is important;
skip_marker:                            ; don't move it!
        ; First, see if we're being asked to Show or Hide the 87...
        .IF     ax == 444Ch             ; This is our signal.
        mov     cs:fHide87, 0           ; Stop hiding the coprocessor.
        .ELSEIF ax == 646Ch             ; This is also our signal.
        mov     cs:fHide87, 1           ; Resume hiding the coprocessor.
        .ENDIF

        ; Next, call the original ISR...
        pushf                           ; Simulate interrupt by pushing flags
        call    cs:OldInt11Proc         ; and far-calling original routine.
        sti                             ; Enable interrupts.

        ; Finally, if fHide87 is TRUE, tweak the bits in AX...
        .IF     cs:fHide87 == 1         ; Do we want to hide the 87?                
        and     al, 11111101y           ; Clear bit 1 of control word
        .ENDIF
        iret                            ; Return to sender.
NewInt11Proc ENDP

;*************************************************************************
; Install - This procedure marks the end of the TSR's resident section
; and the beginning of the installation section.  Install performs 3
; operations:
;       1. Retrieve the address of the previous INT 11h ISR.
;       2. Installs its own INT 11h ISR.
;       3. Prints a message saying it's installed.
;       4. Issues the MS-DOS TSR call.

Install PROC
        ; Retrieve the address of the old Int 11 ISR...
        mov     ax, 3511h       ; DOS GetVect for Int 11
        int     21h
        mov     WORD PTR OldInt11Proc[0], bx    ; save old proc address
        mov     WORD PTR OldInt11Proc[2], es

        ; Find out if we're already installed by scanning current ISR
        ; for our magic marker...
        mov     di, MagicMarker-NewInt11Proc    ;offset from ISR start
        mov     si, OFFSET MagicMarker
        mov     ax, [si]
        cmp     WORD PTR es:[bx+di], ax
        jne     NotInstalledYet
        mov     ax, [si+2]
        cmp     WORD PTR es:[bx+di+2], ax
        jne     NotInstalledYet
        
        ; If we've made it here, then we know we're already installed.
        ; All we need to do now is activate the "hiding" feature and exit.
        mov     ax,     646Ch   ; Request ISR to hide coprocessor
        pushf
        int     11h
        
        ; Notify user that it's activated...
        mov     dx, offset msg3         ; Address of message
	mov	ah, 09h                 ; Display String
        int     21h                    
        mov     dx, offset msg2         ; Address of message
	mov	ah, 09h                 ; Display String
        int     21h                    

        jmp NormalExit                  ; Time to leave


NotInstalledYet:
        ; Set the interrupt vector to point to us...
        mov     ax, 2511h       ;DOS SetVect for Int 11
        mov     dx, OFFSET NewInt11Proc         ; DS:DX points to new ISR
        int     21h

        ; Output message saying it's installed...
        mov     dx, offset msg1         ; Address of message
	mov	ah, 09h                 ; Display String
        int     21h                    
        mov     dx, offset msg2         ; Address of message
	mov	ah, 09h                 ; Display String
        int     21h                    
   
        ; Put ourselves to sleep...
        mov     dx, OFFSET Install      ; DX = bytes in resident section
        mov     cl, 4                   
        shr     dx, cl                  ; Convert to # of paragraphs
        inc     dx                      ; plus one.
        mov     ax, 3100h               ; Issue Terminate and Stay
        int     21h                     ; Resident call to DOS.
        ; The Int21h func 31h was our last instruction ... we don't fall thru.
NormalExit:
        .EXIT
msg1    db 'HIDE87: Int 11h hook installed (must reboot to remove).', cr,lf,'$'
msg2    db '        Use SHOW87 to deactivate.',cr,lf,'$'
msg3    db 'HIDE87 activated - coprocessor masking on.',cr,lf,'$'
Install ENDP
        END

