/*
 *  ISOLATE.C - Routine to isolate one card
 *
 *  Notes:
 *
 *      This file contain the routine to isolate cards
 */

#include "parse.h"

CM_LOCAL_DATA   WORD    CM_BASED        ReadPortToTry[]={       0x203,
                                0x2e3,
                                0x3e3,
                                0x303,
                                0       // Terminator
                            };


CM_LOCAL_DATA   char    CM_BASED        CurrentID[ENUM_NAME_LEN+3+5+1+8+1]=ENUM_NAME;


//WARNING: ExtraBit must be 0 for the isolation case

VOID
Out(WORD wAddress, BYTE bValue)
{
    _asm {
        mov     dx, wAddress
        mov     al, bValue
        out     dx, al
    }
    return;
}



#pragma warning (disable:4785)  // turn off near call intersegment problem
#pragma warning (disable:4033)  // turn off must return value warning
#pragma warning (disable:4035)  // turn off no return value warning

BYTE VXDINLINE
Xor(BYTE bValue1, BYTE bValue2)
{
    _asm {
        mov     al, bValue1
        xor     al, bValue2
    }
    return;
}

BYTE
InData()
{
    WORD Port;

    Port=ReadPort;
    _asm{
        mov     dx, Port
        in      al, dx
    }
    return;
}


BYTE
In(WORD wPort)
{
    _asm {
        mov     dx, wPort
        in      al, dx
    }
    return;
}


WORD CM_LOCAL
ReadTimer()
{
    _asm {  
        pushf
        cli

        xor     al, al
        out     43h, al

        jmp     next1
next1:
        in      al, 40h
        mov     ah, al

        jmp     next2
next2:

        in      al, 40h
        xchg    ah, al

        popf
    }
}

LONG 
TimeToTicks(LONG);

#pragma warning (default:4035)  // turn on no return value warning
#pragma warning (default:4033)  // turn on must return value warning

BYTE CM_LOCAL
LFSR(BYTE bPrevious, BYTE bExtraBit)
{
    BYTE    bHighBit=Xor(Xor((bPrevious&2)>>1, bPrevious&1), bExtraBit);

    return((bPrevious>>1)|(bHighBit<<7));

}

VOID CM_LOCAL
SendKey()
{
    BYTE    i;
    BYTE    current=LFSR_SEED;

    Out(ADR_PORT, 0);
    Out(ADR_PORT, 0);

    for (i=0; i<32; i++) {
        Out(ADR_PORT, current);
        current=LFSR(current, 0);
    }
}

VOID CM_LOCAL
Delay(LONG DelayTime)
{
    WORD    old, new;

    DelayTime=TimeToTicks(DelayTime);

    old=ReadTimer();

    while (DelayTime>0) {
        new=ReadTimer();
        DelayTime-=((old-new)>>1);
        old=new;
    }
}

CONFIGRET CM_LOCAL
SetupCard(CSN csn)
{
    UCHAR           Buffer[255];

    char far *      name=&(CurrentID[ENUM_NAME_LEN]);
    BYTE            i, tmp;
    ULONG           serial;
    WORD            idn;

    Out(ADR_PORT, ADR_WAKE);
    Out(CMD_PORT, csn);

    for (i=0; i<255; i++) {

        Out(ADR_PORT, ADR_RD_CFG_STATUS);

        while ((InData() & 0x01)!=1);

        Out(ADR_PORT, ADR_RD_CFG_DATA);

        Buffer[i]=InData();
    }

    *name=(Buffer[0]>>2)+'A'-1;
    name++;

    *name=((Buffer[0]&0x3)<<3)+(Buffer[1]>>5)+'A'-1;
    name++;

    *name=(Buffer[1]&0x1f)+'A'-1;
    name++;

    idn=*((PWORD)&(Buffer[2]));

    for (i=0;i<5; i++) {
        name[4-i]=(char)('0'+idn%10);
        idn/=10;
    }
    name+=5;

    *name='\\';
    name++;

    serial=*((PULONG)&(Buffer[4]));

    for (i=0;i<8; i++) {
        tmp=(BYTE)(serial&0xF);
        name[7-i]=(char)((tmp>9) ? ('A'+tmp-10) : ('0'+tmp));
        serial>>=4;
    }

    name+=8;

    return(CR_SUCCESS);
}

BOOL CM_LOCAL
IsolateOneCard(PCARD_ID pCardID, CSN csn)
{
    BYTE            bit, one_bit, i, b1, b2, valid;
    PBYTE           pCardData;
    BYTE            checksum=LFSR_SEED;

    pCardData=(PBYTE)pCardID;

    Out(ADR_PORT, ADR_WAKE);
    Out(CMD_PORT, 0);

    Out(ADR_PORT, ADR_READ_ID);

    Delay(DELAY_1MS-DELAY_250US);

    pCardData=(PBYTE)pCardID;

    for (i=0;i<SIZE_CARD_ID; i++) {
        *pCardData=0;
        pCardData++;
    }

    one_bit=0;

    pCardData=(PBYTE)pCardID;

    pCardData--;

    for (i=0; i<8*SIZE_CARD_ID; i++) {

        Delay(DELAY_250US);

        b1=InData();
        b2=InData();

        bit=(b1==BIT_SET_1)&&(b2==BIT_SET_2);

        one_bit|=bit;

        if (!(i%8)) {
            pCardData++;
        }

        *pCardData|=bit<<((i%8));

        if (i<CHECKSUMED_BITS) {
            checksum=LFSR(checksum, bit);
        }

    }

    valid=one_bit && ((checksum==pCardID->CheckSum) || !pCardID->CheckSum);

    if (valid) {
        Out(ADR_PORT, ADR_SET_CARD_ID);
        Out(CMD_PORT, csn);
        if (csn!=InData())
            valid=FALSE;
    }

    return(valid);

}

DWORD CM_EXTERNAL
IsolateRM(DWORD Services)
{
    BYTE            ReadPortNumber=0;
    BYTE            nISACards=0;
    BYTE            i;
    CARD_ID         CardID;
    DOS386Services=Services;

    for (;;) {

        ReadPort=ReadPortToTry[ReadPortNumber];
        if (!ReadPort) {
            if (fVerbose) {
                printf("Failed to find any cards!!\n");
            }
            return(0);
        }

        if (fVerbose) {
            LogPrintf("Read Port: %hx\n",ReadPort);
        }

        SendKey();

        Out(ADR_PORT, ADR_CONFIG_CNTL);
        Out(CMD_PORT, 6);

        Delay(DELAY_2MS);

        SendKey();

        Out(ADR_PORT, ADR_WAKE);
        Out(CMD_PORT, 0);

        Delay(DELAY_1MS);

        Out(ADR_PORT, ADR_SET_RD_PORT);

        Out(CMD_PORT, (BYTE)(ReadPort>>2));

        if (IsolateOneCard(&CardID, 1))
            break;

        ReadPortNumber++;
    }

    do {
        nISACards++;
    } while (IsolateOneCard(&CardID, nISACards+1));

    for (i=1; i<=nISACards; i++)
        SetupCard(i);


    return((DWORD)(nISACards+(ReadPort<<16)));
}

void
main(int argc,char *argv[])
{

    long int nCards;
    int  i;

    nScreenHeight=GetScreenHeight()-3;

    LogPrintf("PNP Card Real Mode Isolation Program V2.1\n(C) Microsoft Corp, 1993\n\n");

    //Set up globals;
    if (argc > 1) {
        for (i=1;i<argc;i++) {
            if (strchr (argv[i],'v') || strchr (argv[i],'V')) {
                fVerbose=TRUE;
            }
            else {
                fVerbose=FALSE;
            }
        }
    }

    nStartDF=0;
    fEndDF=FALSE;

    //Isolate all PNP cards
    nCards=IsolateRM(0);

    if (fVerbose) {
        LogPrintf("%d PNP Card(s) Seen and Isolated\n",nCards);
    }

    
    // Goto menu
    if (nCards ) {
        ConfigureCards((int)nCards);
    }
    else {
        LogPrintf("No PNP cards found in the system\n");
    }

    reset_cards();

}


/*
 * ReadResData. This routine is passed the CSN of a card found in the
 * isolation process, and then reads the card's resource information.
 *
 * NEEDS a check for unprogrammed cards -BUGBUG
 */
BYTE *
ReadResData (short int nCard)
{
    ULONG nBytes=0,i;
    BYTE *pBase, *pData;


    LogPrintf("\n\n");
    pData=NULL;
    nBlocks=1;

    //Allocate 32K -1 
    pBase=AllocateData(0,READ_BLOCK_SIZE,pData);
    pData=pBase;

    Out(ADR_PORT, ADR_WAKE);
    Out(CMD_PORT, (BYTE)nCard);

    do {

        //Wait for the status bit to be set, data ready
        Out(ADR_PORT, ADR_RD_CFG_STATUS);
        while (!(InData() & 0x01)) ;

        *pData=ReadDataWord();
        pData++;
        nBytes++;

    //Read in 32K-1
    } while (nBytes < READ_BLOCK_SIZE);

    pData=pBase;

    if (fVerbose) {
        LogPrintf("Data: \n");

        for (i=0;i<nBytes;i++) 
            LogPrintf("%02x ",pData[i]);
        LogPrintf("\n");
    }

    return (pData);
}

/*
 * Mostly PSS code from here on in, bug fixes, goto removal done
 * by PNPTest/PNPDev
 *    
 */
    
/*
 * ConfigureCards.
 */

void
ConfigureCards(int nCards)
{
    BYTE * pData;
    int nTargetCard=0;
    int nConfigChoice;
    int bDone=FALSE;

    LogPrintf("\n   %d PNP Card(s) found\n", nCards);
    // Get target card from user if more than 1 card in system
    if (nCards > 1) {

        do {
            LogPrintf("\nEnter Card Selection Number - ");
            scanf("%d", &nTargetCard);
            
            if ((nTargetCard > nCards) || (!nTargetCard)) {
            
                LogPrintf("\nERROR: Invalid card number\n\n");

                //Flush the buffer to avoid infinite loop, RAID bug from intel
                fflush (stdin);

            }
        } while ((nTargetCard > nCards) || (!nTargetCard));
    }
    else
        nTargetCard = 1;

    // Read card's resource data
    LogPrintf("\nReading.....\n");
    pData = ReadResData(nTargetCard);

    // Display memu


    while (nConfigChoice != 5) {

        //Reset the Globals...
        nStartDF=0;
        fEndDF=FALSE;
        nConfigChoice=0;
        nLines=0;


        LogPrintf("\n");
        LogPrintf("1) - View card resource information\n");
        LogPrintf("2) - View current configuration for logical device\n");
        LogPrintf("3) - Configure logical device\n");
        LogPrintf("4) - Choose new card\n");
        LogPrintf("5) - Exit\n");
        
        LogPrintf("\nEnter choice - ");
        scanf("%d", &nConfigChoice);
        fflush (stdin);
    
        switch (nConfigChoice) {
            // exit
            case 5:
                break;

            // Display card's resource data
            case 1:
                parse_data(pData, READ_BLOCK_SIZE);
                break;
                        
            // Read a logical device's configuration registers
            case 2: 
                ReadConfig(nTargetCard, pData, (int)READ_BLOCK_SIZE);
                break;
    
            // Set a logical device's configuration registers
            case 3:
                SetConfig(nTargetCard, pData, (int)READ_BLOCK_SIZE);
                break;
    
            // Choose a different Card...
            case 4:
                if (pData) {
                    free (pData);
                }
                ConfigureCards(nCards);
                return;

            default:
                break;
        }

    }
    free(pData);
}       

/*
 * ReadConfig.  This routine selects a logical device on the card
 * and then reads it's configuration registers.
 */

void
ReadConfig(int nTargetCard, BYTE *pData, int nBytes)
{
    ULONG nTargetDevice, nNumDev;
    DEVICE_CFG DevCfg;
    
    // Find out how many logical devices are supported on the card.
    if (nNumDev = EnumLogDev(pData, nBytes))
    {
        // Choose one of the devices
        nTargetDevice = SelectDevice(nNumDev);

        // Read the logical device's configuration registers
        GetDevConfig(nTargetCard, nTargetDevice, &DevCfg);

        // Display the device's current configuration
        ParseDevConfig(&DevCfg);
    }
}


/*
 * SelectDevice.        This routine inputs the logical device number from
 * the user.  If the number of devices on the card is 1, it just returns 0.
 */


ULONG
SelectDevice(ULONG nNumDev)
{
    ULONG nTargetDevice=INT_MAX;

    if (nNumDev != 1) {

        do {
            LogPrintf("\nEnter Logical Device number to read - ");
            scanf("%lx", &nTargetDevice);
  
            // Make sure it's a valid device number
            if (nTargetDevice >= nNumDev) {
                LogPrintf("\nERROR: Invalid Logical Device!\n");
                fflush(stdin);
            }
        } while (nTargetDevice >= nNumDev);

        return (nTargetDevice);
    }
    else {
        //Only one device on card, so return 0
        return (0);
    }
}


/*
 * EnumLogDev.  Counts the number of logical device's on the card
 * and displays each logical ID.
 */
ULONG
EnumLogDev(BYTE * pResData, int nSizeRes)
{
    int Tag, nCur=DEV_CODE_OFFSET+1, nTagLen;
    ULONG LogDevCnt=0, DevID=0;

    LogPrintf("\n");
    do
    {
        Tag=GetTag(pResData,nCur,&nTagLen);
        
        
        // Check if tag is a logical device
        if (Tag==PNP_LOG_DEV_ID)
        {
            // Get the device's ID
            DevID=*((ULONG *)&(pResData[nCur+1]));

            LogPrintf("    Logical Device %ld - ID 0x%08lX\n", LogDevCnt, DevID);
            
            LogDevCnt++;
        }
    
        if (Tag >> 7)
            Add_Large(nCur);
        else
            Add_Small(nCur);

        nCur += nTagLen;
            
    } while (Tag != PNP_END);

    LogPrintf("\n");
    return LogDevCnt;
}
    

/*
 * getDevConfig. Read all the device's configuration registers and stuff
 * them into a DEVICE_CFG structure.
 */
void
GetDevConfig(int nTargetCard, ULONG nLogDev, DEVICE_CFG * DevCfg)
{
    int nDesc, IOPort;
    
    //Wake target card
    Out(ADR_PORT, ADR_WAKE);
    Out(CMD_PORT, nTargetCard);

    //Select logical device
    Out(ADR_PORT, ADR_LOGICAL_DEV);
    Out(CMD_PORT, (BYTE)nLogDev);


    // Read all the memory configuration registers
    IOPort = ADR_MEMORY_BASE;
    for (nDesc=0; nDesc<NUM_MEMORY_DESC; nDesc++) {

        Out(ADR_PORT, IOPort++);
        DevCfg->MemConfig[nDesc].nHiBase = InData();

        Out(ADR_PORT, IOPort++);
        DevCfg->MemConfig[nDesc].nLoBase = InData();
        
        Out(ADR_PORT, IOPort++);
        DevCfg->MemConfig[nDesc].nControl = InData();
        
        Out(ADR_PORT, IOPort++);
        DevCfg->MemConfig[nDesc].nHiRange = InData();
        
        Out(ADR_PORT, IOPort++);
        DevCfg->MemConfig[nDesc].nLoRange = InData();
        
        IOPort += 3;
    }

    
    // Read all the I/O configuration registers
    IOPort = ADR_IO_BASE;
    for (nDesc=0; nDesc<NUM_IO_DESC; nDesc++) {
        Out(ADR_PORT, IOPort++);
        DevCfg->IOConfig[nDesc].nHiIOBase = InData();
    
        Out(ADR_PORT, IOPort++);
        DevCfg->IOConfig[nDesc].nLoIOBase = InData();
    }
    
    
    // Read all the IRQ configuration registers
    IOPort = ADR_IRQ_BASE;
    for (nDesc=0; nDesc<NUM_IRQ_DESC; nDesc++) {
        Out(ADR_PORT, IOPort++);
        DevCfg->IRQConfig[nDesc].nIRQLevel = InData();
        Out(ADR_PORT, IOPort++);
        DevCfg->IRQConfig[nDesc].nIRQType = InData();
    }


    // Read all the DMA configuration registers
    IOPort = ADR_DMA_BASE;
    for (nDesc=0; nDesc<NUM_DMA_DESC; nDesc++) {
        Out(ADR_PORT, IOPort++);
        DevCfg->DMAConfig[nDesc].nDMAMask = InData();
    }

}
    
void
ParseDevConfig(DEVICE_CFG * DevCfg)
{
    int nDesc;
    WORD nMemCtrl,IOBase;
    ULONG nBase, nRange;
    BYTE nType,nDMA;
    
    for (nDesc=0; nDesc<NUM_MEMORY_DESC; nDesc++) {
        
        nBase = MAKELONG( ((WORD)(DevCfg->MemConfig[nDesc].nLoBase) << 8),
                    (WORD)DevCfg->MemConfig[nDesc].nHiBase );
        LogPrintf("Memory descriptor %d\n", nDesc);


        nRange = MAKELONG( ((WORD)DevCfg->MemConfig[nDesc].nLoRange << 8),
                     (WORD)DevCfg->MemConfig[nDesc].nHiRange );
        LogPrintf("    Base address  = 0x%08lX\n", nBase);

        if ((nMemCtrl = DevCfg->MemConfig[nDesc].nControl) & 0x01) {
            LogPrintf("    Upper address limit = 0x%08lX\n", nRange);
        }

        else {
            LogPrintf("    Address range = 0x%08lX\n", nRange);
        }

        nMemCtrl = (nMemCtrl >> 1) & 0x03;

        switch (nMemCtrl) {
            case (0x00):
                LogPrintf("8-bit memory\n");
                break;

            case (0x01):
                LogPrintf("16-bit memory\n");
                break;
        
            case (0x02):
                LogPrintf("8 & 16-bit memory\n");
                break;

            default:
                LogPrintf("    Invalid memory control type\n", nMemCtrl);
                break;
        }
    }
        
    LogPrintf("\n");
    for (nDesc=0; nDesc<NUM_IO_DESC; nDesc++) {

        IOBase = (WORD)(DevCfg->IOConfig[nDesc].nHiIOBase << 8) +
                     DevCfg->IOConfig[nDesc].nLoIOBase;
        LogPrintf("I/O Port Descriptor %d\n", nDesc);
        LogPrintf("    I/O port base address = 0x%X\n", IOBase);    
    }

    LogPrintf("\n");
    for (nDesc=0; nDesc<NUM_IRQ_DESC; nDesc++) {
    
        LogPrintf("IRQ Descriptor %d\n", nDesc);
        LogPrintf("    IRQ level = 0x%X\n", DevCfg->IRQConfig[nDesc].nIRQLevel);

        nType = DevCfg->IRQConfig[nDesc].nIRQType;

        if (nType & 0x02)
            LogPrintf("    High ");
        else
            LogPrintf("    Low ");
        
        if (nType & 0x01)
            LogPrintf("Level Triggered\n");
        else
            LogPrintf("Edge Triggered\n");
    }
    LogPrintf("\n");

    for (nDesc=0; nDesc<NUM_DMA_DESC; nDesc++) {
    
        LogPrintf("DMA Descriptor %d\n", nDesc); 
        nDMA = DevCfg->DMAConfig[nDesc].nDMAMask & 0x07;
        if (nDMA == 0x04)
            LogPrintf("    No DMA Channel Selected\n");
        else
            LogPrintf("    DMA Channel %d\n\n", nDMA);
    }
}




void
SetConfig(int nTargetCard, BYTE * pData, int nResourceSize)
{
    int   nIRQDesc=INT_MAX, nDMADesc=INT_MAX, nIODesc=INT_MAX, nChoice,nIRQLevel=INT_MAX, nIRQType=INT_MAX;
    int   nMemDesc=INT_MAX,nRangeType,nRetVal,nIOChoice=0;
    WORD  nDMAChannel=INT_MAX,nIOBase=INT_MAX;
    ULONG nDeviceCnt, nTargetDevice,uBase=0,uRange=0;

    if (nDeviceCnt = EnumLogDev(pData, nResourceSize))
    {
        nTargetDevice = SelectDevice(nDeviceCnt);

        //Wake target card
        Out(ADR_PORT, ADR_WAKE);
        Out(CMD_PORT, nTargetCard);

        //Select logical device
        Out(ADR_PORT, ADR_LOGICAL_DEV);
        Out(CMD_PORT, (BYTE)nTargetDevice);
        
        do
        {
            LogPrintf("\n1) Configure Memory Resource\n");
            LogPrintf("2) Configure Interrupt Resource\n");
            LogPrintf("3) Configure DMA Resource\n");
            LogPrintf("4) Configure I/O Resource\n");
            LogPrintf("5) Previous Menu\n\n");
            LogPrintf("Enter Choice - ");
            scanf("%d", &nChoice);

            switch (nChoice) {
                case 2: {
                    
                    do {
                        LogPrintf("\nSelect IRQ Descriptor (0 - %d) - ", NUM_IRQ_DESC-1);
                    
                        scanf("%d", &nIRQDesc);
                    
                        if (nIRQDesc >= NUM_IRQ_DESC) {
                
                            LogPrintf("ERROR: Invalid IRQ descriptor\n");
                            fflush (stdin);
                        }
                    } while (nIRQDesc >=NUM_IRQ_DESC);

                    do {
                        LogPrintf("Select IRQ Line (0x0 - 0xF) - ");
                        scanf("%x", &nIRQLevel);

                        if (nIRQLevel <= 0x0F) {
                            if (TestRegister(ADR_IRQ_BASE+nIRQDesc*ADR_IRQ_INCR, nIRQLevel))
                                LogPrintf("\nSuccessfully set IRQ line!\n\n");
                            else
                                LogPrintf("\nERROR: Failed to configure IRQ line!\n\n");
                        
                        }
                        else {
                            LogPrintf("\nERROR: Invalid IRQ line\n");
                             fflush (stdin);
                        }
                    } while (nIRQLevel > 0x0F);


                    do {
                        LogPrintf("    IRQ Types:\n");
    
                        // Display possible interrupt types
                        LogPrintf("    0 - High True Edge Sensitive\n");
                        LogPrintf("    1 - Low True Edge Sensitive\n");
                        LogPrintf("    2 - High True Level Sensitive\n");
                        LogPrintf("    3 - Low True Level Sensitive\n\n");
                        LogPrintf("    4 - Don't Set IRQ Type\n\n");
                                                                  
                        LogPrintf("\n    Select IRQ Type - ");
                        scanf("%d", &nIRQType);
                                
    
                        if (nIRQType < 4)
                        {
                            int nTypeMask=0;
                        
                            if ((!nIRQType) || (nIRQType == 2))
                                nTypeMask |= 0x02;
                                
                            if (nIRQType > 1)
                                nTypeMask |= 0x01;
                                
                            if (TestRegister(ADR_IRQ_BASE+nIRQDesc*ADR_IRQ_INCR+1, (BYTE)nTypeMask))
                                LogPrintf("Successfully set IRQ type!\n\n");
                            else
                                LogPrintf("ERROR: Failed to configure IRQ type!\n\n");
                        }
                        else if (nIRQType > 4) {
                            LogPrintf("ERROR: Invalid IRQ Type!\n");
                            fflush (stdin);
                        }
                    } while (nIRQType >= 4);
                    break;
                }
                
                case 3:
                {

                    do {
                        LogPrintf("\nSelect DMA Descriptor (0 - %d) - ", NUM_DMA_DESC-1);
                    
                        scanf("%d", &nDMADesc);
                        if (nDMADesc >= NUM_DMA_DESC) {
                            LogPrintf("ERROR: Invalid DMA descriptor\n");
                            fflush (stdin);
                            break;
                        }
                    } while (nDMADesc >= NUM_DMA_DESC);
                    
                    do {
                        LogPrintf("Select DMA channel (0 - 7) - ");

                        scanf("%d", &nDMAChannel);
                        LogPrintf("\n\n");

                        if (nDMAChannel <= 7) {

                            if (TestRegister(ADR_DMA_BASE+nDMADesc*ADR_DMA_INCR, (BYTE)nDMAChannel))
                                LogPrintf("\nSuccessfully set DMA channel!\n\n");
                            else
                                LogPrintf("\nERROR: Failed to configure DMA channel!\n\n");
                        }                                                         
                        else {
                            LogPrintf("\nERROR: Invalid DMA channel\n");
                             fflush (stdin);
                           }
                    } while (nDMAChannel > 7);
                    break;
                }

                case 4:
                {

                    do {
                        LogPrintf("\nSelect IO Descriptor (0 - %d) - ", NUM_IO_DESC-1);
                    
                        scanf("%d", &nIODesc);
                        if (nIODesc >= NUM_IO_DESC) {
                            LogPrintf("ERROR: Invalid IO descriptor\n");
                            fflush (stdin);
                        }
                    } while (nIODesc >= NUM_IO_DESC);

                    LogPrintf("\n    Enter I/O base address - ");
                    nRetVal=scanf("%x", &nIOBase);

                    if (TestRegister(ADR_IO_BASE+nIODesc*ADR_IO_INCR, HIBYTE(nIOBase)) &&
                         TestRegister(ADR_IO_BASE+nIODesc*ADR_IO_INCR+1,LOBYTE(nIOBase)) && nRetVal)
                    {
                        
                        LogPrintf("\nSuccessfully set I/O base address!\n\n");
                        LogPrintf("1) Perform I/O Range Check\n");
                        LogPrintf("2) Return to previous menu\n");
                        LogPrintf("\nEnter Choice - ");
                        
                        scanf("%d", &nIOChoice);

                        if (nIOChoice == 1)
                            IORangeCheck(nIOBase);
                    }
                    else {
                        LogPrintf("\nERROR: Failed to configure I/O base address!\n\n");
                        fflush (stdin);
                    }
                    break;
                }

                case 1:
                {

                    do {
                        LogPrintf("\nSelect Memory Descriptor (0 - %d) - ", NUM_MEMORY_DESC-1);
                    
                        scanf("%d", &nMemDesc);
                        if (nMemDesc >= NUM_MEMORY_DESC) {
                            LogPrintf("ERROR: Invalid memory descriptor\n");
                            fflush (stdin);
                        }
                    } while (nMemDesc >= NUM_MEMORY_DESC);

                    LogPrintf("\n    Enter memory base address - ");
                    nRetVal=scanf("%lx", &uBase);

                    if (TestRegister(ADR_MEMORY_BASE+nMemDesc*ADR_MEMORY_INCR, LOBYTE(HIWORD(uBase))) &&
                        TestRegister(ADR_MEMORY_BASE+nMemDesc*ADR_MEMORY_INCR+1, HIBYTE(LOWORD(uBase))) 
                        && nRetVal) {
                        LogPrintf("\nSuccessfully set base memory address!\n\n");
                    }
                    else {
                        LogPrintf("\nERROR: Failed to configure base memory address!\n\n");
                        fflush (stdin);
                        break;
                    }
                    fflush (stdin);

                    //Get range type
                    Out(ADR_PORT, ADR_MEMORY_BASE + nMemDesc * ADR_MEMORY_INCR +2);
                    nRangeType = InData();

                    if (nRangeType & 0x01)
                    {
                        //This card expects an upper memory limit
                
                        LogPrintf("\n    Enter upper memory address limit - ");
                        nRetVal=scanf("%lx", &uRange);

                        if ( TestRegister(ADR_MEMORY_BASE+nMemDesc*ADR_MEMORY_INCR+3,
                                      LOBYTE(HIWORD(uRange))) &&
                                TestRegister(ADR_MEMORY_BASE+nMemDesc*ADR_MEMORY_INCR+4,
                                      HIBYTE(LOWORD(uRange)))  && nRetVal)
                            LogPrintf("\nSuccessfully set upper limit address!\n\n");
                        else
                            LogPrintf("\nERROR: Failed to configure upper limit address!\n\n");
                            fflush (stdin);
                            break;
                    }
                    else
                    {
                        //This card expects a range limit
                        //Note: This code assumes the given range is a power of 2

                        LogPrintf("\n    Enter memory range limit - ");
                        nRetVal=scanf("%lx", &uRange);
                        
                        if ( TestRegister(ADR_MEMORY_BASE+nMemDesc*ADR_MEMORY_INCR+3,
                                      LOBYTE(HIWORD(uRange))) &&
                              TestRegister(ADR_MEMORY_BASE+nMemDesc*ADR_MEMORY_INCR+4,
                                      HIBYTE(LOWORD(uRange)))  && nRetVal)
                            LogPrintf("\nSuccessfully set range length!\n\n");
                        else
                            LogPrintf("\nERROR: Failed to configure range length!\n\n");
                    }
                    fflush (stdin);
                    break;
                }
            }

        } while (nChoice != 5);
    }
}

/*
 * TestRegister. Tries to write a specified value out to a configuration
 * port, and then read it back for comparison.
 */
int TestRegister(int nConfigPort, BYTE nValue)
{
    BYTE nRet;
    
    Out(ADR_PORT, nConfigPort);
    Out(CMD_PORT, nValue);
    Out(ADR_PORT, nConfigPort);
    nRet = InData();

    if (fVerbose)
        LogPrintf("Wrote %x to port %x, read back %x\n", nValue, nConfigPort, nRet);
    
    if (nRet == nValue)
        return (TRUE);
    else
        return (FALSE);
}

void IORangeCheck(WORD nIOBase)
{
    WORD nIORange, i, nConflict=FALSE, nIOPort;
    BYTE nInData;
    BYTE nrActive;

    //Test to ensure not active
    Out (ADR_PORT,ADR_IO_ACTIVE);
    nrActive=InData();

    if (nrActive & 0x01) {
        LogPrintf("\n Card is active, must be inactive to perform I/O Range Checking\n");
        LogPrintf("Setting card to inactive, card will be restored after test\n");
    }
    

    LogPrintf("\nBase I/O address = 0x%X\n", nIOBase);
    LogPrintf("\nEnter size of range to check (hex) - ");
    scanf("%x", &nIORange);
    fflush (stdin);


    //Enable I/O Range Check, using I/O reads of 0x55
    Out(ADR_PORT, ADR_IO_PORT_CONFL);
    Out(CMD_PORT, 0x03);

    nIOPort = nIOBase;

    LogPrintf("\n");
    for (i=0; i<nIORange; i++) {
        _asm {
 
            mov    dx, nIOPort
            in     al, dx
            mov    BYTE PTR nInData, al
        }
        if (nInData != 0x55) {
            LogPrintf("ERROR: Port 0x%X, I/O read = 0x%X, should be 0x55\n",
                    nIOPort, nInData);
            nConflict = TRUE;
        }
        else {
            if (fVerbose) {
                LogPrintf("Port 0x%X, I/O read = 0x%X\n", nIOPort, nInData);
            }
        }

        nIOPort++;
    }

    //Enable I/O Range Check, using I/O reads of 0xAA
    Out(ADR_PORT, ADR_IO_PORT_CONFL);
    Out(CMD_PORT, 0x02);

    nIOPort = nIOBase;

    LogPrintf("\n");
    for (i=0; i<nIORange; i++) {
        _asm {
            mov    dx, nIOPort
            in     al, dx
            mov    BYTE PTR nInData, al
        }

        if (nInData != 0xAA) {
            LogPrintf("ERROR: Port 0x%X, I/O read = 0x%X, should be 0xAA\n",
                    nIOPort, nInData);
            nConflict = TRUE;
        }
        else {
            if (fVerbose) {
                LogPrintf("Port 0x%X, I/O read = 0x%X\n", nIOPort, nInData);
            }
        }
        nIOPort++;
    }

    //Disable I/O Range Check
    Out(ADR_PORT, ADR_IO_PORT_CONFL);
    Out(CMD_PORT, 0x00);

    if (nConflict)
        LogPrintf("\nERROR: I/O range conflict detected!\n\n");
    else
        LogPrintf("\nPort I/O range check successful!  No I/O conflicts detected.\n\n");

    //Test to reset active
    if (nrActive & 0x01) {
        Out (ADR_PORT,ADR_IO_ACTIVE);
        Out (ADR_PORT,nrActive);
    }


}
