/***************************************************************
*
*
*   ShareCom 1.0   
*    Copyright (C)  1994 Identity Software Inc
*
* messages.h
* 
*  External Data Representation
*
*/






/*
********************************

Codes for Communicating scalars and structures 

DV = Dis Kit Value

*********************************
*/


#define FIRST_DV_DTP 180
#define LAST_DV_DTP 220

/* Data types */

#define DV_NULL 180
/* NIL. false, the NULL pointer */

#define DV_SHORT_STRING 181
/* Binary string with 1 byte length */
#define DV_LONG_STRING 182
/* Binary string with 4 byte length */


#define DV_C_STRING 183
#define DV_C_SHORT 184
/* DV_C_STRING and DV_C_SHORT specifically refer to the C data types
   char * and short. The char * is treated as a string that has no header
   and that ends in a NULL character. It is variously transmitted as a 
   DV_SHORT_STRING or DV_LONG_STRING.

   The short is transmitted as a DV_SHORT_INT or DV_LONG_INTm depending on 
   its value
*/

#define DV_STRING_SESSION 185
/* Internally used in DB client to send string output  sessions
   as DV_<xx>_CONT_STRING's. */

#define DV_SHORT_CONT_STRING 186
#define DV_LONG_CONT_STRING 187




#define DV_SHORT_INT 188
/* signed 8 bit byte */
#define DV_LONG_INT 189
/* signed 4 byte integer. Sent low order first */

#define DV_SINGLE_FLOAT 190
/* 4 byte floar */
#define DV_DOUBLE_FLOAT 191
/* 8 byte float */
#define DV_CHARACTER 192
/* 1 byte character */

#define DV_ARRAY_OF_POINTER  193
/* record with 4 byte element count. The said number
   of items follow (mostly OBJECT_AND_CLASS records.
   
   The size of the object constructed from this is 4 times the length 
   at the lead of the message
*/

#define DV_ARRAY_OF_LONG  194
#define DV_ARRAY_OF_FLOAT  202
#define DV_ARRAY_OF_DOUBLE  195



#define DV_LIST_OF_POINTER  196
/* Interpreted as ARRAY_OF_POINTER by C or C++ client */


#define DV_OBJECT_AND_CLASS 197
/* The message contain an object id and a class id, both 4 byte ints.
   This is interpreted as a reference to the object with the id in the 
   session. If no object with that id is found in the receiving party a
   non present object is constructed. The class allows knowing the length
   of the object so that a memory address can be reserved for eventually
   bringing in the object.
*/

#define DV_OBJECT_REFERENCE 198
/* 4 bytes of object id follow. This may appear in slot values inside
   clusters or in other places where the message sender knows that he receiver
   has previously been informed of the class of the referenced object.
   When this is not certain the sender uses DV_OBJECT_AND_CLASS.
*/


#define DV_DELETED 199
/* Appears in a cluster delta as predicate value if the predicate has
   been deleted. Does not really apply to C based Dis Kit 
   Interpreted as a NULL. 
*/

#define DV_MEMBER_POINTER 200
/* pointer into a structure. object id, class id, member id, count */

#define DV_C_INT 201
/* C integer, whateber ;ength it be. 16/32 */


/***********************************

Distributed Objects Codes 

DO = Distributed Object

************************************/

#define DO_FOR_STORAGE 220
/* Prefix used when sending data to a database server for storage
   as received */


#define DO_CLUSTER  221
#define DO_OBJECT 222

#define DO_SET_CLASS 223
/* 4 bytes of object id and 4 bytes of instance id follow. */

#define DO_SET_OBJECT 224
/* 4 byte object id follows */

#define DO_SET_SLOT 225
/*4 bytes of member (slot) id follow */

#define DO_SET_VERSION 226
/* 4 bytes of version id follow. Ignored by C/C++ client */

/* obsolate  pkr 17.12.1992
#define DO_SET_VALUE 227
   A value follows.  preficed with data type */

#define DO_SET_SAME_VALUE 228
/* This slot has the same value as the previous slot */


/* TAMMI mty */
/* #define DO_SET_FINGE_CLASS  256 */


#define DO_DELETED 229
/* 
  Object id follows.
  Appers alone or  in a cluster delta  header part after the classes to indicate 
  that a certain object is no more
*/




/* META */

#define DO_SET_BACKING_SESSION 230
#define DO_SET_RESIDENCE_SESSION 231

/* These are followed by an object id in the current session, then a host
   specification and a session id in the session between the receiving host
   and the host specified.
   RESERVED FOR FUTURE USE. NOT INPLEMENTED NOW.
*/

#define DO_TRANSACTION_ADD_UPDATE 232

#define DO_EMBEDDED_ARRAY 233
/* Followed by 4 byte count and 1 byte dtp. Count
  successive elements */

/* This trailer appears on records beginning with DO_OBJECT or DO_CLUSTER
   or corresponding deltas */
#define DO_PACKET_END 252


#define DO_CLUSTER_DELTA 253
#define DO_OBJECT_DELTA 254
#define DO_BLOB_DELTA 255









/************************************

Codes for Remote Actions

DA = Dis Kit Action

*************************************/

#define DA_FUTURE_REQUEST 1

#define DA_FUTURE_ANSWER 2

#define DA_FUTURE_PARTIAL_ANSWER 3

#define DA_DIRECT_IO_FUTURE_REQUEST 4

#define DA_CALLER_IDENTIFICATION 5

/* FRQ = futue request
   RRC = remote realize condition = future answer.

   The below constants are the offsets of message oarts
   inside an ARRAY_OF_POINTER format.
*/

#define DA_MESSAGE_TYPE 0  /* futur, answer or something else */

#define FRQ_COND_NUMBER 1
#define FRQ_ANCESTRY 2
#define FRQ_SERVICE_NAME 3
#define FRQ_ARGUMENTS 4

#define DA_FRQ_LENGTH 5


#define RRC_COND_NUMBER 1
#define RRC_VALUE 2
#define RRC_ERROR 3

#define DA_ANSWER_LENGTH 4
