/****************************************************************************

    PROGRAM: MOVE

    PURPOSE: Microsoft Sample Application

    FUNCTIONS:

        WinMain() - calls initialization function, processes message loop
        InitApplication() - initializes window data and registers window
        InitInstance() - saves instance handle and creates main window
        MainWndProc() - processes messages
        ChildWndProc () - Custom Control Window Procedure
        About() - processes messages for "About" dialog box
        TestProc() - Test procedure.

****************************************************************************/

#include "windows.h"
#include "stdio.h"
#include "MOVE.h"

/*
 *  This is the size of a custom control child window.
 */

#define  CHILD_X   10
#define  CHILD_Y   10
#define  CHILD_CX  150
#define  CHILD_CY  150

HANDLE hInst;
HWND   hwnd;            /* handle to main window */
int    cbIDCount;
BOOL   bMove;           /* boolean stating if a move is in progress */
HWND   ghwndChild;      /* handle of the currently moving window */
unsigned int dx,dy;     /* dx,dy value of the mouse relative to the window */

/****************************************************************************

    FUNCTION: WinMain(HANDLE, HANDLE, LPSTR, int)

    PURPOSE: calls initialization function, processes message loop

****************************************************************************/

int PASCAL WinMain(hInstance, hPrevInstance, lpCmdLine, nCmdShow)
HANDLE hInstance;
HANDLE hPrevInstance;
LPSTR lpCmdLine;
int nCmdShow;
{
    MSG msg;

    if (!hPrevInstance)
        if (!InitApplication(hInstance))
            return (FALSE);

    if (!InitInstance(hInstance, nCmdShow))
        return (FALSE);

    while (GetMessage(&msg, NULL, NULL, NULL))
        {
        TranslateMessage(&msg);
        DispatchMessage(&msg);
        }

    return (msg.wParam);
}


/****************************************************************************

    FUNCTION: InitApplication(HANDLE)

    PURPOSE: Initializes window data and registers window class

****************************************************************************/

BOOL InitApplication(hInstance)
HANDLE hInstance;
{
    WNDCLASS  wc;

    wc.style          = NULL;
    wc.lpfnWndProc    = MainWndProc;
    wc.cbClsExtra     = 0;
    wc.cbWndExtra     = 0;
    wc.hInstance      = hInstance;
    wc.hIcon          = LoadIcon(NULL, IDI_APPLICATION);
    wc.hCursor        = LoadCursor(NULL, IDC_ARROW);
    wc.hbrBackground  = GetStockObject(WHITE_BRUSH);
    wc.lpszMenuName   = "SampleMenu";
    wc.lpszClassName  = "SampleClass";

    if (!RegisterClass(&wc))
        return 0;

    cbIDCount = 1;

    wc.style          = NULL;
    wc.lpfnWndProc    = ChildWndProc;
    wc.cbClsExtra     = 0;
    wc.cbWndExtra     = 0;
    wc.hInstance      = hInstance;
    wc.hIcon          = LoadIcon(NULL, IDI_APPLICATION);
    wc.hCursor        = LoadCursor(NULL, IDC_ARROW);
    wc.hbrBackground  = GetStockObject(WHITE_BRUSH);
    wc.lpszMenuName   = NULL;
    wc.lpszClassName  = "CustomClass";

    return (RegisterClass(&wc));

}


/****************************************************************************

    FUNCTION:  InitInstance(HANDLE, int)

    PURPOSE:  Saves instance handle and creates main window

****************************************************************************/

BOOL InitInstance(hInstance, nCmdShow)
HANDLE          hInstance;
int             nCmdShow;
{

    hInst = hInstance;

    hwnd = CreateWindow(
             "SampleClass",
             "Microsoft Windows Sample Application",
             WS_OVERLAPPEDWINDOW,
             CW_USEDEFAULT,
             CW_USEDEFAULT,
             CW_USEDEFAULT,
             CW_USEDEFAULT,
             NULL,
             NULL,
             hInstance,
             NULL );

    if ( !hwnd )
        return ( FALSE );

    ShowWindow(hwnd, nCmdShow);
    UpdateWindow(hwnd);
    return (TRUE);
}


/****************************************************************************

    FUNCTION: MainWndProc(HWND, unsigned, WORD, LONG)

    PURPOSE:  Processes messages

    MESSAGES:

        WM_COMMAND    - application menu (About dialog box)
        WM_DESTROY    - destroy window

    COMMENTS:

        WM_COMMAND processing:

            IDM_ABOUT - display "About" box.
            IDM_TEST  - Test Procedure.

****************************************************************************/

long FAR PASCAL MainWndProc(hWnd, message, wParam, lParam)
HWND hWnd;
unsigned message;
WORD wParam;
LONG lParam;
{

    FARPROC  lpProcAbout;
    switch ( message )
        {
        case WM_COMMAND:
            switch ( wParam )
                {
                case IDM_ABOUT:
                    lpProcAbout = MakeProcInstance( About, hInst );
                    DialogBox(hInst, "AboutBox", hWnd, lpProcAbout);
                    FreeProcInstance( lpProcAbout );
                    break;

                case IDM_TEST:
                    TestProc( hWnd );
                    break;

                }
            break;

        case WM_DESTROY:
            PostQuitMessage(0);
            break;

        default:
            return (DefWindowProc(hWnd, message, wParam, lParam));
     }

     return (NULL);
}


/****************************************************************************

    FUNCTION: About(HWND, unsigned, WORD, LONG)

    PURPOSE:  Processes messages for "About" dialog box

    MESSAGES:

        WM_INITDIALOG - initialize dialog box
        WM_COMMAND    - Input received

****************************************************************************/

BOOL FAR PASCAL About(hDlg, message, wParam, lParam)
HWND hDlg;
unsigned message;
WORD wParam;
LONG lParam;
{
    switch (message)
        {
        case WM_INITDIALOG:
            return (TRUE);

        case WM_COMMAND:
            if (wParam == IDOK || wParam == IDCANCEL)
                {
                EndDialog(hDlg, TRUE);
                return (TRUE);
                }
            break;
        }
    return (FALSE);
}

/***********************************************************************/


int PASCAL TestProc( HWND hWnd )
{
    HWND  hwndChild;

    /*
     *  We want to create a child window of our own custom class.  Note
     *  the use of WS_CLIPSIBLINGS.  This is necessary to prevent drawing
     *  on another window's client area.  Necessary in this example when
     *  moving the windows.
     */

    hwndChild = CreateWindow ("CustomClass",
                  NULL,
                  WS_BORDER | WS_VISIBLE | WS_CHILD | WS_CLIPSIBLINGS,
                  CHILD_X,
                  CHILD_Y,
                  CHILD_CX,
                  CHILD_CY,
                  hWnd,
                  cbIDCount,
                  hInst,
                  NULL);

    ShowWindow (hwndChild, SW_SHOWNORMAL);
    UpdateWindow (hwndChild);

    /*
     *  By calling SetWindowPos, we now place the new window at the top
     *  of the internal window list.  Hence, if an existing child window
     *  was already in the upper left hand corner, this causes the new
     *  window to appear on top of the existing window.
     */

    SetWindowPos (hwndChild, NULL, 0,0,0,0,
                  SWP_NOMOVE | SWP_NOSIZE | SWP_SHOWWINDOW);

    /*
     *  increment the child ID count.
     */

    cbIDCount++;

    return 1;
}

/************************************************************************/

long FAR PASCAL ChildWndProc(hWnd, message, wParam, lParam)
HWND hWnd;
unsigned message;
WORD wParam;
LONG lParam;
{
    int  ID;
    HDC  hDC;
    PAINTSTRUCT lpPaint;
    char  szBuffer[25];
    RECT  rect;
    POINT point;

    switch ( message )
        {
        case WM_LBUTTONDOWN:
           /*
            *  We want to move a control.  First, grab the mouse capture.
            *  Then find the hwnd of the window in which the mouse was
            *  clicked.  This can be found via the WindowFromPoint function.
            *  Note that the mouse coordiates stored in the lParam are
            *  client coordinates, and need to be converted to screen
            *  coordinates for WindowFromPoint.
            *
            *  dx, dy are the delta values of the mouse from the upper
            *  left hand corner of the window.
            */

           SetCapture (hWnd);
           dx = point.x = LOWORD (lParam);
           dy = point.y = HIWORD (lParam);
           ClientToScreen (hWnd, &point);
           ghwndChild = WindowFromPoint (point);

           /*
            *  This solves a problem with windows appearing to
            *  go "underneath" other windows.  By calling SetWindowPos,
            *  the window is bumped to the top of the window list thus
            *  causing it to go over the top of all other controls.
            */

           SetWindowPos (ghwndChild, NULL, 0,0,0,0,
                         SWP_NOMOVE | SWP_NOSIZE | SWP_SHOWWINDOW);

           /*
            *  Set the move flag
            */

           bMove = TRUE;

           return 1;

        case WM_MOUSEMOVE:
           /*
            *  This were the window is actually moved.  If the move
            *  flag is set and the left mouse button is down when the
            *  WM_MOUSEMOVE message is received, the window will be
            *  moved.
            *
            *  Note:  need to take care of coordinate conversion.  The
            *  coordinates stored in the lParam are based off of the
            *  upper left hand corner of the *custom control*.  We need
            *  to convert them to screen coordinates, and then back
            *  to client coordinates of the parent window for the
            *  MoveWindow call.
            */

           if (bMove)
             {
             if (wParam & MK_LBUTTON)
               {

               point.x = LOWORD(lParam);
               point.y = HIWORD(lParam);

               ClientToScreen (hWnd, &point);
               ScreenToClient (hwnd, &point);

               /*
                *  The dx, dy values reposition the window relative
                *  to the mouse when the initial click occured.  Otherwise
                *  the window's upper left hand corner would jump to were
                *  the mouse is.
                */

               MoveWindow (ghwndChild, point.x - dx, point.y - dy,
                           CHILD_CX, CHILD_CY, TRUE);
               }
             }
           return 1;

        case WM_LBUTTONUP:
           /*
            *  We are done moving the window.  Set the move flag to
            *  FALSE, set the global window handle to NULL, and release
            *  the mouse capture.
            */

           if (bMove)
             {
             bMove = FALSE;
             ghwndChild = NULL;
             }
           ReleaseCapture ();
           return 1;


        case WM_PAINT:
           /*
            *  Paint routine for the child windows.
            */

           hDC = BeginPaint (hWnd, &lpPaint);

           /*
            *  Get the control's id and insert it into the text string
            *  to be displayed.
            */

           ID = GetDlgCtrlID (hWnd);
           sprintf (szBuffer, "Child Window #%d", ID);

           GetClientRect (hWnd, &rect);

           DrawText (hDC, szBuffer, -1, &rect,
                     DT_SINGLELINE | DT_CENTER | DT_VCENTER);

           EndPaint (hWnd, &lpPaint);
           return 1;

        default:
            return (DefWindowProc(hWnd, message, wParam, lParam));
        }

     return (NULL);
}



/* END OF FILE */
