#include "stdafx.h"

#include "minmax.h"
#include "multidlg.h"

#include <malloc.h>

IMPLEMENT_DYNAMIC(CMultiDialog, CDialog);

#ifdef _DEBUG
	// Define this to get debugging trace output
	#define DEBUG
	#undef THIS_FILE
	static char BASED_CODE THIS_FILE[] = __FILE__;
	#define new DEBUG_NEW
#endif


// ---------- CSubTemplateMap ----------
//	
// Mapping between text and subtemplate controls


class CSubTemplateMap : public CObject
	{
public:
	CSubTemplateMap(LPCSTR szLabel, int iFirstItem, int nItems);
	~CSubTemplateMap(void);

	CString m_szLabel;		// includes char * and 2 ints = 8 bytes
	int m_iFirstItem;
	int m_nItems;
	};


CSubTemplateMap::CSubTemplateMap(LPCSTR szLabel, int iFirstItem, int nItems)
{
#ifdef DEBUG
	TRACE("CSubTemplateMap::CSubTemplateMap(\"%s\", %d, %d) entered [this=%04X:%04X]\n",
		szLabel, iFirstItem, nItems, SELECTOROF(this), OFFSETOF(this));
#endif

	m_szLabel = szLabel;
	m_iFirstItem = iFirstItem;
	m_nItems = nItems;
}


CSubTemplateMap::~CSubTemplateMap(void)
{
}


// ---------- CMultiDialog ----------


CMultiDialog::CMultiDialog(LPCSTR szBaseTemplate, UINT uIDGroupBox, CWnd *pParent)
	: CDialog(), m_sTemplate(pParent)
{
	init(szBaseTemplate, uIDGroupBox, pParent);
}


CMultiDialog::CMultiDialog(UINT nIDBaseTemplate, UINT uIDGroupBox, CWnd *pParent)
	: CDialog(), m_sTemplate(pParent)
{
	init(MAKEINTRESOURCE(nIDBaseTemplate), uIDGroupBox, pParent);
}


CMultiDialog::~CMultiDialog(void)
{
	CSubTemplateMap *pMap;

#ifdef DEBUG
	TRACE0("CMultiDialog::~CMultiDialog() entered\n");
#endif

	while (m_aMaps.GetSize())
		{
		pMap = (CSubTemplateMap *)m_aMaps[0];
		m_aMaps.RemoveAt(0);
		delete pMap;
		}

	m_aStatus.RemoveAll();
}


BOOL CMultiDialog::init(LPCSTR szBaseTemplate, UINT uIDGroupBox, CWnd *pParent)
{
	CDialogItem *pGroupBox;
	int iIndex;

#ifdef DEBUG
	TRACE("CMultiDialog::init(%08lX, %04X, %04X:%04X) entered at %08ld\n",
		szBaseTemplate, uIDGroupBox, SELECTOROF(pParent), OFFSETOF(pParent), GetTickCount());
#endif

	m_pParentWnd = pParent;		// In CDialog class

	// Load base template definition
	if (!m_sTemplate.load(szBaseTemplate))
		{
		if (_AFX_FP_SEG(szBaseTemplate) == 0)
			TRACE("CMultiDialog failure : unable to load template %04X\n", _AFX_FP_OFF(szBaseTemplate));
		else
			TRACE("CMultiDialog failure : unable to load template \"%s\"\n", szBaseTemplate);
		return FALSE;
		}

	// Into what rectangle shall we load the subdialogs?
	iIndex = m_sTemplate.find_item_index(uIDGroupBox);
	if (iIndex < 0)
		{
		TRACE("CMultiDialog failure : group box with ID %04X not found\n", uIDGroupBox);
		return FALSE;
		}
	pGroupBox = m_sTemplate.get_item(iIndex);
	ASSERT(pGroupBox != NULL);
	m_rectSub = pGroupBox->get_rect();		// In dialog units
#ifdef DEBUG
	TRACE("   Group box rectangle is (%d,%d),(%d,%d)\n",
		m_rectSub.left, m_rectSub.top, m_rectSub.right, m_rectSub.bottom);
#endif

	// Now that we have the rectangle, remove the group box from the template
	m_sTemplate.remove_item(uIDGroupBox);
	m_nMainTemplateItems = m_sTemplate.num_items();

	// The index where the group box control was is now the index of the first control
	// which should come after the subtemplates (if any).
	m_iInsertIndex = iIndex;
#ifdef DEBUG
	TRACE("   m_nMainTemplateItems = %d ; m_iInsertIndex = %d\n", m_nMainTemplateItems, m_iInsertIndex);
#endif

	// No subtemplate enabled yet
	m_iFirstEnabledItem = -1;
	m_nEnabledItems = 0;

	return TRUE;
}


void CMultiDialog::AddSelection(LPCSTR szSubTemplate, LPCSTR szText)
{
	CDialogTemplate subTemplate(m_pParentWnd);
	CSize subSize;
	CDialogItem *pItem;
	int i;
	long lStyle;
	CRect rect;
	CSubTemplateMap *pMap;

#ifdef DEBUG
	if (szText)
		TRACE("CMultiDialog::AddSelection(%04X:%04X, \"%s\") entered at %08ld\n",
			SELECTOROF(szSubTemplate), OFFSETOF(szSubTemplate), szText, GetTickCount());
	else
		TRACE("CMultiDialog::AddSelection(%04X:%04X, NULL) entered at %08ld\n",
			SELECTOROF(szSubTemplate), OFFSETOF(szSubTemplate), GetTickCount());
#endif

	// Can't add any more selections once the dialog box is running
	if (m_hWnd)
		{
		TRACE("CMultiDialog::AddSelection() failure : m_hWnd(%04X) != 0!\n", (WORD)m_hWnd);
		AfxThrowResourceException();
		return;
		}

	if (!subTemplate.load(szSubTemplate))
		{
		if (_AFX_FP_SEG(szSubTemplate) == 0)
			TRACE("CMultiDialog::AddSelection() failure : unable to load sub template %04X\n", _AFX_FP_OFF(szSubTemplate));
		else
			TRACE("CMultiDialog::AddSelection() failure : unable to load sub template \"%s\"\n", szSubTemplate);
		AfxThrowResourceException();
		return;
		}

	// Is this template too big?
	subSize = subTemplate.get_size();
	if ( (subSize.cx > m_rectSub.Width()) || (subSize.cy > m_rectSub.Height()) )
		{
		TRACE("CMultiDialog::AddSelection() failure : sub dialog [%d X %d] too large for area [%d X %d]!\n",
			subSize.cx, subSize.cy, m_rectSub.Width(), m_rectSub.Height() );
		AfxThrowResourceException();
		return;
		}

	// Do we know the name of this subtemplate?
	if (!szText)
		szText = subTemplate.get_caption();

	// Remember text, array index, and # of items for use during OnInitDialog()
	pMap = new CSubTemplateMap(szText, m_iInsertIndex, subTemplate.num_items());
	m_aMaps.Add(pMap);
#ifdef DEBUG
	TRACE("   Sub dialog \"%s\" : start item %d, number of items %d [map at %04X:%04X]\n",
		szText, m_iInsertIndex, subTemplate.num_items(),
		SELECTOROF(pMap), OFFSETOF(pMap));
#endif

	// Copy items to main dialog template, making them invisible and
	// moving them to the proper location as we go.
	for (i = 0 ; i < subTemplate.num_items() ; i++)
		{
		pItem = subTemplate.get_item(i);
		ASSERT(pItem != NULL);
		lStyle = pItem->get_style();
		rect = pItem->get_rect();
		// Store this item's visible status bit in our array
		m_aStatus.SetAt(pItem->get_id(), lStyle & WS_VISIBLE);
#ifdef DEBUG
		TRACE("   Subtemplate item [%d] = main template item [%d]\n",
			i, m_iInsertIndex);
		TRACE("      ID %04X, style %08lX, position (%d,%d),(%d,%d)\n",
			pItem->get_id(), lStyle, rect.left, rect.top, rect.right, rect.bottom);
#endif
		lStyle = lStyle & ~WS_VISIBLE;
		rect.OffsetRect(m_rectSub.left, m_rectSub.top);
#ifdef DEBUG
		TRACE("      Setting to style %08lX, position (%d,%d),(%d,%d)\n",
			lStyle, rect.left, rect.top, rect.right, rect.bottom);
#endif
		pItem->set_style(lStyle);
		pItem->set_rect(rect);
		// Add copy of this item to the main dialog box
		m_sTemplate.add_item(pItem, m_iInsertIndex++);
		}

	return;
}


BOOL CMultiDialog::AddDlgItem(HWND hWnd, int iWhichSubTemplate)
{
	CSubTemplateMap *pMap;
	int iIndex, i;
	CDialogItem *pItem;

#ifdef DEBUG
	TRACE("CMultiDialog::AddDlgItem(%04X, %d) entered\n", (WORD)hWnd, iWhichSubTemplate);
#endif
	ASSERT(hWnd != 0);

	if ( (iWhichSubTemplate < 0) || (iWhichSubTemplate >= m_aMaps.GetSize()) )
		return FALSE;

	// Create a temporary CDialogItem object for the new item
	long lStyle = ::GetWindowLong(hWnd, GWL_STYLE);
	UINT uID = ::GetWindowWord(hWnd, GWW_ID);
	char szClass[80];
	::GetClassName(hWnd, szClass, sizeof(szClass));
	CDialogItem sItem(&m_sTemplate, szClass, lStyle);

	// Where shall we insert this new item?
	pMap = (CSubTemplateMap *)m_aMaps[iWhichSubTemplate];
	ASSERT(pMap != NULL);
	iIndex = pMap->m_iFirstItem + pMap->m_nItems;
	ASSERT(iIndex >= m_nMainTemplateItems);
	ASSERT(iIndex <= m_iInsertIndex);

	// Insert item into template
#ifdef DEBUG
	TRACE("   Inserting new item (ID %04X) into template at index [%d]\n", uID, iIndex);
#endif
	sItem.set_id(uID);
	pItem = m_sTemplate.add_item(&sItem, iIndex);
	pMap->m_nItems++;
	m_aStatus.SetAt(uID, lStyle & WS_VISIBLE);

	// Update all index pointers as needed
	if (pMap->m_iFirstItem == m_iFirstEnabledItem)
		m_nEnabledItems++;
	else
		{
		// Item should be invisible, it's not in the active subtemplate!
		::ShowWindow(hWnd, SW_HIDE);
		}
	for (i = 0 ; i < m_aMaps.GetSize() ; i++)
		{
		pMap = (CSubTemplateMap *)m_aMaps[i];
		if (pMap->m_iFirstItem >= iIndex)
			pMap->m_iFirstItem++;
		}
	m_iInsertIndex++;
	if (m_iFirstEnabledItem >= iIndex)		// New item is before the enabled sub
		m_iFirstEnabledItem++;

	return TRUE;
}


int CMultiDialog::DoModal()
{
#ifdef DEBUG
	TRACE0("CMultiDialog::DoModal() entered\n");
#endif

	// Have we not yet constructed the template?
	if ( (!m_hDialogTemplate) && (!ConstructTemplate()) )
		return -1;
#ifdef DEBUG
	TRACE0("   Template constructed successfully!\n");
#endif

	m_iFirstEnabledItem = -1;
	m_nEnabledItems = 0;

	return CDialog::DoModal();
}


BOOL CMultiDialog::UpdateData(BOOL bSaveAndValidate)
{
	BOOL bRet;
	HWND hFocus;
	unsigned int uID;
	int iIndex;
	CSubTemplateMap *pMap;
	int i;

#ifdef DEBUG
	TRACE("CMultiDialog::UpdateData(%d) entered\n", bSaveAndValidate);
#endif

	bRet = CDialog::UpdateData(bSaveAndValidate);
#ifdef DEBUG
	TRACE("   CDialog::UpdateData(%d) returned %d\n", bSaveAndValidate, bRet);
#endif

	// Where is the focus now?
	hFocus = ::GetFocus();
	uID = ::GetWindowWord(hFocus, GWW_ID);
#ifdef DEBUG
	TRACE("   ::GetFocus() returns handle %04X, ID %04X\n", (WORD)hFocus, uID);
#endif

	// Look through the template for this item
	iIndex = m_sTemplate.find_item_index(uID);
#ifdef DEBUG
	TRACE("   m_sTemplate.find_item_index(%04X) returned %d\n", uID, iIndex);
#endif

	// Decide where in the template it is - do we have to switch subtemplates?
	if (iIndex < 0)
		{
#ifdef DEBUG
		TRACE0("   Item is not in this template at all!\n");
#endif
		}
	else if (iIndex < m_nMainTemplateItems)
		{
#ifdef DEBUG
		TRACE0("   Item is in main template, before subtemplates\n");
#endif
		}
	else if (iIndex >= m_iInsertIndex)
		{
#ifdef DEBUG
		TRACE0("   Item is in main template, after subtemplates\n");
#endif
		}
	else if ( (iIndex >= m_iFirstEnabledItem) && (iIndex < m_iFirstEnabledItem + m_nEnabledItems) )
		{
#ifdef DEBUG
		TRACE("   Item is in enabled subtemplate [%d,%d]\n",
			m_iFirstEnabledItem, m_iFirstEnabledItem + m_nEnabledItems - 1);
#endif
		}
	else
		{
		// Must enable this subtemplate, but which one is it?
		for (i = 0 ; i < m_aMaps.GetSize() ; i++)
			{
			pMap = (CSubTemplateMap *)m_aMaps[i];
#ifdef DEBUG
			TRACE("   Subtemplate [%d] \"%s\" = items [%d,%d]\n",
				i, (LPCSTR)pMap->m_szLabel, pMap->m_iFirstItem, pMap->m_iFirstItem + pMap->m_nItems - 1);
#endif
			if ( (iIndex >= pMap->m_iFirstItem) && (iIndex < pMap->m_iFirstItem + pMap->m_nItems) )
				{
				// Select this subtemplate!
				SetSelection(i, FALSE, MDN_UPDATEDATA_FAIL);
				break;
				}
			}
		}

	return bRet;
}


BOOL CMultiDialog::ConstructTemplate(void)
{
	void *pTemplate;
	UINT uTemplateSize;
	HGLOBAL hData;
	void *pData;
	int i, j;
	CDialogItem *pItem1, *pItem2;

#ifdef DEBUG
	TRACE("CMultiDialog::ConstructTemplate() entered at %08ld\n", GetTickCount());
#endif

	// Check for duplicate IDs in the subtemplates.  An ID may occur twice in the main
	// template but may not appear twice or more in one subtemplate, once in each of two
	// or more subtemplates, or in the main template and in one or more subtemplate.
#ifdef DEBUG
	TRACE0("   Checking for duplicate ID values\n");
#endif
	for (i = 0 ; i < m_sTemplate.num_items() - 1 ; i++)
		{
		pItem1 = m_sTemplate.get_item(i);
#ifdef DEBUG
		TRACE("      Item [%d] : ID %04X\n", i, pItem1->get_id());
#endif
		for (j = max(i + 1, m_nMainTemplateItems) ; j < m_sTemplate.num_items() ; j++)
			{
			pItem2 = m_sTemplate.get_item(j);
			if (pItem1->get_id() == pItem2->get_id())
				{
#ifdef DEBUG
				TRACE("         ID duplicated in item [%d]!\n", j);
#else
				TRACE("CMultiDialog failure : ID value %04X used twice!\n", pItem1->get_id());
#endif
				return FALSE;
				}
			}
		}

	pTemplate = m_sTemplate.generate(&uTemplateSize);
	if (!pTemplate)
		{
		TRACE0("CMultiDialog failure : unable to generate template!\n");
		return FALSE;
		}

	// Unfortunately, InitModalIndirect() wants an HGLOBAL!
	hData = GlobalAlloc(GMEM_MOVEABLE, uTemplateSize);
	if (!hData)
		{
		_ffree(pTemplate);
		return FALSE;
		}
	pData = GlobalLock(hData);

	ASSERT(pData != NULL);
	memcpy(pData, pTemplate, uTemplateSize);
#ifdef DEBUG
	TRACE("   Data copied from buffer at %04X:%04X to %04X:%04X\n",
		SELECTOROF(pTemplate), OFFSETOF(pTemplate), SELECTOROF(pData), OFFSETOF(pData));
#endif

	GlobalUnlock(hData);
	_ffree(pTemplate);

	return CDialog::InitModalIndirect(hData);
}


BOOL CMultiDialog::FillListBox(CListBox *pListBox, int iSelectIndex)
{
	CString szText;
	int i, iControlIndex;
	CSubTemplateMap *pMap;

#ifdef DEBUG
	TRACE("CMultiDialog::FillListBox(%04X:%04X, %d) entered at %08ld\n",
		SELECTOROF(pListBox), OFFSETOF(pListBox), iSelectIndex, GetTickCount());
#endif

	ASSERT(pListBox != NULL);

	// Add items to listbox
	for (i = 0 ; i < m_aMaps.GetSize() ; i++)
		{
		pMap = (CSubTemplateMap *)m_aMaps[i];
#ifdef DEBUG
		TRACE("   Adding label \"%s\" for subdialog %d\n",
			(LPCSTR)pMap->m_szLabel, i);
#endif
		iControlIndex = pListBox->AddString((LPCSTR)pMap->m_szLabel);
		pListBox->SetItemData(iControlIndex, (DWORD)i);
		}

	if (iSelectIndex >= 0)
		pListBox->SetCurSel(iSelectIndex);

	return TRUE;
}


BOOL CMultiDialog::ShowDlgItem(int nID, int nCmdShow)
{
	int iIndex;
	HWND hControl;
	long lStyle;

#ifdef DEBUG
	TRACE("CMultiDialog::ShowDlgItem(%d, %d) entered\n", nID, nCmdShow);
#endif

	// Look through the template for this item
	iIndex = m_sTemplate.find_item_index(nID);
#ifdef DEBUG
	TRACE("   m_sTemplate.find_item_index(%04X) returned %d\n", nID, iIndex);
#endif

	// Decide where in the template it is
	if (iIndex < 0)
		{
#ifdef DEBUG
		TRACE0("   Item is not in this template at all!\n");
#endif
		return FALSE;
		}

	hControl = ::GetDlgItem(m_hWnd, nID);
	ASSERT(hControl != 0);

	if (iIndex < m_nMainTemplateItems)
		{
#ifdef DEBUG
		TRACE0("   Item is in main template, before subtemplates\n");
#endif
		return ::ShowWindow(hControl, nCmdShow);
		}
	else if (iIndex >= m_iInsertIndex)
		{
#ifdef DEBUG
		TRACE0("   Item is in main template, after subtemplates\n");
#endif
		return ::ShowWindow(hControl, nCmdShow);
		}

	// Update the WS_VISIBLE flag in the map
	VERIFY(m_aStatus.Lookup(nID, lStyle));
	if (nCmdShow == SW_HIDE)
		lStyle &= ~WS_VISIBLE;
	else
		lStyle |= WS_VISIBLE;
	m_aStatus.SetAt(nID, lStyle & WS_VISIBLE);

	if ( (iIndex >= m_iFirstEnabledItem) && (iIndex < m_iFirstEnabledItem + m_nEnabledItems) )
		{
#ifdef DEBUG
		TRACE("   Item is in enabled subtemplate [%d,%d]\n",
			m_iFirstEnabledItem, m_iFirstEnabledItem + m_nEnabledItems - 1);
#endif
		return ::ShowWindow(hControl, nCmdShow);
		}

#ifdef DEBUG
	TRACE0("   Item is in disabled subtemplate\n");
#endif

	return TRUE;
}


BOOL CMultiDialog::SetSelection(LPCSTR szText, BOOL bEraseBkgnd)
{
	int i;
	CSubTemplateMap *pMap;

#ifdef DEBUG
	TRACE("CMultiDialog::SetSelection(\"%s\", %d) entered\n", szText, bEraseBkgnd);
#endif

	for (i = 0 ; i < m_aMaps.GetSize() ; i++)
		{
		pMap = (CSubTemplateMap *)m_aMaps[i];
		if (pMap->m_szLabel == szText)
			return SetSelection(i, bEraseBkgnd);
		}

	return FALSE;
}


BOOL CMultiDialog::SetSelection(int iWhich, BOOL bEraseBkgnd, int iReason)
{
	CSubTemplateMap *pMap;
	int i;
	CDialogItem *pItem;
	CWnd *pControl;
	CRect rect, rInvalid;
	long lStyle;
	UINT uID;

#ifdef DEBUG
	TRACE("CMultiDialog::SetSelection(%d, %d, %04X) entered\n",
		iWhich, bEraseBkgnd, iReason);
#endif

	if ( (iWhich < 0) || (iWhich >= m_aMaps.GetSize()) )
		return FALSE;

	pMap = (CSubTemplateMap *)m_aMaps[iWhich];
	ASSERT(pMap != NULL);

	if ( (pMap->m_iFirstItem == m_iFirstEnabledItem) && (pMap->m_nItems == m_nEnabledItems) )
		return TRUE;

	// Wait and do all window updates at once
	VERIFY(LockWindowUpdate());

	// Calculate rectangle to invalidate
	rInvalid.SetRect(INT_MAX, INT_MAX, INT_MIN, INT_MIN);

	if (m_nEnabledItems > 0)
		{
		for (i = m_iFirstEnabledItem ; i < m_iFirstEnabledItem + m_nEnabledItems ; i++)
			{
			pItem = m_sTemplate.get_item(i);
#ifdef DEBUG
			TRACE("   Item [%d] = %04X:%04X\n", i, SELECTOROF(pItem), OFFSETOF(pItem));
#endif
			ASSERT(pItem != NULL);
			pControl = GetDlgItem(pItem->get_id());
			if (!pControl)
				continue;
			// Get control's location in dialog client coords
			pControl->GetWindowRect(&rect);
			ScreenToClient(&rect);
#ifdef DEBUG
			TRACE("   Disabling and hiding control %04X (%04X:%04X) at (%d,%d),(%d,%d)\n",
				pItem->get_id(), SELECTOROF(pControl), OFFSETOF(pControl),
				rect.left, rect.top, rect.right, rect.bottom);
#endif
			if (rInvalid.left == INT_MAX)
				rInvalid = rect;
			else
				rInvalid |= rect;
			pControl->ShowWindow(SW_HIDE);
			}
		}
	
	for (i = pMap->m_iFirstItem ; i < pMap->m_iFirstItem + pMap->m_nItems ; i++)
		{
		pItem = m_sTemplate.get_item(i);
#ifdef DEBUG
		TRACE("   Item [%d] = %04X:%04X\n", i, SELECTOROF(pItem), OFFSETOF(pItem));
#endif
		ASSERT(pItem != NULL);
		uID = pItem->get_id();
		pControl = GetDlgItem(uID);
		if (!pControl)
			continue;
		VERIFY(m_aStatus.Lookup(uID, lStyle));
		// Get control rectangle in dialog client units
		pControl->GetWindowRect(&rect);
		ScreenToClient(&rect);
#ifdef DEBUG
		TRACE("   Setting control %04X (%04X:%04X) at (%d,%d),(%d,%d) to %08lX\n",
			pItem->get_id(), SELECTOROF(pControl), OFFSETOF(pControl),
			rect.left, rect.top, rect.right, rect.bottom, lStyle);
#endif
		if (lStyle & WS_VISIBLE)
			{
			pControl->ShowWindow(SW_SHOW);
			if (rInvalid.left == INT_MAX)
				rInvalid = rect;
			else
				rInvalid |= rect;
			}
		}

	m_iFirstEnabledItem = pMap->m_iFirstItem;
	m_nEnabledItems = pMap->m_nItems;

	// OK, update everything!
	VERIFY(::LockWindowUpdate(NULL));
#ifdef DEBUG
	GetClientRect(&rect);
	TRACE("   Dialog client rectangle : (%d,%d),(%d,%d)\n",
		rect.left, rect.top, rect.right, rect.bottom);
#endif

	if (rInvalid.left != INT_MAX)
		{
		if (!bEraseBkgnd)
			{
			// Validate the rectangle so we can re-invalidate it without background painting
			ValidateRect(&rInvalid);
			InvalidateRect(&rInvalid, FALSE);
			}
		else
			InvalidateRect(&rInvalid, TRUE);
#ifdef DEBUG
		TRACE("   Invalidated rectangle : (%d,%d),(%d,%d)\n",
			rInvalid.left, rInvalid.top, rInvalid.right, rInvalid.bottom);
#endif
		}

	// Send message to dialog to tell him what happened
	SendMessage(MDN_SELECTCHANGE, iWhich, MAKELPARAM(iReason, 0));

	return TRUE;
}


