#include "stdafx.h"
#include "myfile.h"

#include "minmax.h"
#include "tresourc.h"


IMPLEMENT_DYNAMIC(CMyFile, CFile)
IMPLEMENT_DYNAMIC(CMyMemFile, CMemFile)

#ifdef _DEBUG
	// Define this to get debugging trace output
	// #define DEBUG
	#undef THIS_FILE
	static char BASED_CODE THIS_FILE[] = __FILE__;
	#define new DEBUG_NEW
#endif


// ---------- CMyFile  ----------

CMyFile::CMyFile()
	: CFile()
{ m_iError = 0; }
CMyFile::CMyFile(int hFile)
	: CFile(hFile)
{ m_iError = 0; }
CMyFile::CMyFile(const char *pszFileName, UINT nOpenFlags)
	: CFile(pszFileName, nOpenFlags)
{ m_iError = 0; }


BOOL CMyFile::Open(const char *pszFileName, UINT nOpenFlags, CFileException *pException)
{
	CFileException sException;
	BOOL bRet;
#define MODES	(CFile::modeRead | CFile::modeReadWrite | CFile::modeWrite)
#define SHARES	(CFile::shareDenyNone | CFile::shareDenyRead | CFile::shareDenyWrite | CFile::shareExclusive | CFile::shareCompat)

	// Make sure we have sharing modes set reasonably
	if (nOpenFlags & CFile::modeCreate)
		{
		// Creating a file
		if (!(nOpenFlags & SHARES))
			nOpenFlags |= CFile::shareExclusive;
		if (!(nOpenFlags & MODES))
			nOpenFlags |= CFile::modeReadWrite;
		}
	else if ((nOpenFlags & MODES) == CFile::modeRead)
		{
		// Reading a file
		if (!(nOpenFlags & SHARES))
			nOpenFlags |= CFile::shareDenyWrite;
		}
	else
		{
		// Writing (and maybe reading) a file
		if (!(nOpenFlags & SHARES))
			nOpenFlags |= CFile::shareExclusive;
		}

	if (!pException)
		pException = &sException;
	bRet = CFile::Open(pszFileName, nOpenFlags, pException);
	if (!bRet)
		m_iError = pException->m_cause;

	return bRet;
}


BOOL CMyFile::Insert(long nBytes)
{
	long lPos, lSize, lMove;
	char *pData = NULL;

	if (nBytes)
		{
		TRY
			{
			// Save current position and size.
			lPos = GetPosition();
			lSize = GetLength();
			lMove = lSize - lPos;
			// If we're at EOF, don't need to do anything.
			if (lMove > 0)
				{
				// Allocate enough space to read in the file.
				pData = new char[lMove];
				// Read data, seek to new position, write data, return to original position.
				BigRead(pData, lMove);
				Seek(lPos + nBytes, CFile::begin);
				BigWrite(pData, lMove);
				Seek(lPos, CFile::begin);
	    		}
	    	}
	    CATCH(CFileException, theFileException)
	    	{
	    	m_iError = theFileException->m_cause;
	    	}
	    AND_CATCH(CMemoryException, theMemoryException)
	    	{
	    	m_iError = noMemory;
	    	}
	    END_CATCH
	    }

	if (pData)
		delete [] pData;
		
	return (BOOL)(m_iError == 0);
}


BOOL CMyFile::Remove(long nBytes)
{
	long lPos, lSize, lMove;
	char *pData = NULL;

#ifdef DEBUG
	TRACE("CMyFile::Remove(%ld) entered\n", nBytes);
#endif

	if (nBytes)
		{
		TRY
			{
			// Save current position and size.
			lPos = GetPosition();
			lSize = GetLength();
			lMove = lSize - (lPos + nBytes);
#ifdef DEBUG
			TRACE("   lPos = %ld ; lSize = %ld ; lMove = %ld\n", lPos, lSize, lMove);
#endif
			// If we're at EOF, truncate file.
			if (lMove <= 0)
				{
				if (lPos != lSize)
					SetLength(lPos);
				}
			else
				{
				// Allocate enough space to read in the file.
				pData = new char[lMove];
				// Read data, seek to new position, write data, truncate file, return
				// to original position.
				Seek(lPos + nBytes, CFile::begin);
				BigRead(pData, lMove);
				Seek(lPos, CFile::begin);
				BigWrite(pData, lMove);
				SetLength(lSize - nBytes);
				Seek(lPos, CFile::begin);
				}
			}
	    CATCH(CFileException, theFileException)
	    	{
	    	m_iError = theFileException->m_cause;
	    	}
	    AND_CATCH(CMemoryException, theMemoryException)
	    	{
	    	m_iError = noMemory;
	    	}
	    END_CATCH
		}

	if (pData)
		delete [] pData;
		
	return (BOOL)(m_iError == 0);
}


void CMyFile::BigWrite(const void huge *lpBuf, DWORD lCount)
{
	const char huge *lpcBuf = (const char huge *)lpBuf;
	
	while (lCount > 0x8000)
		{
		Write(lpcBuf, 0x8000);
		lpcBuf += 0x8000;
		lCount -= 0x8000;
		}
	if (lCount)
		Write(lpcBuf, (UINT)lCount);
}


DWORD CMyFile::BigRead(void huge *lpBuf, DWORD lCount)
{
	DWORD lRead;	// Pronounced "L-REDD", not "L-REED"
	UINT uRead;
	char huge *lpcBuf = (char huge *)lpBuf;

	lRead = 0L;
	
	while (lCount > 0x8000)
		{
		uRead = Read(lpcBuf, 0x8000);
		lRead += (DWORD)uRead;
		if (uRead != 0x8000)
			return lRead;
		lpcBuf += 0x8000;
		lCount -= 0x8000;
		}
	if (lCount)
		{
		uRead = Read(lpcBuf, (UINT)lCount);
		lRead += (DWORD)uRead;
		}
	return lRead;
}


int CMyFile::GetError(void)
{
	int iError = m_iError;
	
	m_iError = 0;
	
	return iError;
}


// ---------- CMyMemFile ----------

CMyMemFile::CMyMemFile(UINT nGrowBytes)
	: CMemFile(nGrowBytes)
{
#ifdef DEBUG
	TRACE("CMyMemFile::CMyMemFile(%u) entered\n", nGrowBytes);
#endif

	cInitMask = 0x80;
	cBitReadMask = cInitMask;
	cBitWriteMask = cInitMask;
	cBitReadRack = 0;
	cBitWriteRack = 0;
}


BOOL CMyMemFile::Abort(BOOL bSaveGrowBytes)
{
	int nGrowBytes;

#ifdef DEBUG
	TRACE("CMyMemFile::Abort(%d) entered\n", bSaveGrowBytes);
#endif

	ASSERT_VALID(this);

	// FlushBits();		Might throw an exception

	nGrowBytes = m_nGrowBytes;
	CMemFile::Close();			// Doesn't throw exceptions
	if (bSaveGrowBytes)
		m_nGrowBytes = nGrowBytes;

	return TRUE;
}


void CMyMemFile::Close(BOOL bSaveGrowBytes)
{
	int nGrowBytes;
	
#ifdef DEBUG
	TRACE("CMyMemFile::Close(%d) entered\n", bSaveGrowBytes);
#endif
	
	ASSERT_VALID(this);
	
	FlushBits();
	
	nGrowBytes = m_nGrowBytes;
	CMemFile::Close();			// Doesn't throw exceptions
	if (bSaveGrowBytes)
		m_nGrowBytes = nGrowBytes;

	return;
}


void CMyMemFile::Attach(BYTE *pBuffer, UINT nFileSize)
{
#ifdef DEBUG
	TRACE("CMyMemFile::Attach(%04X:%04X, %u) entered\n",
		SELECTOROF(pBuffer), OFFSETOF(pBuffer), nFileSize);
#endif
	
    // Clear everything except m_nGrowBytes
	Close(TRUE);

	if (!pBuffer)
		AfxThrowFileException(nullBuffer);
		
	m_lpBuffer = pBuffer;

	m_nBufferSize = (UINT)nFileSize;
	m_nFileSize = nFileSize;

#ifdef DEBUG
	TRACE("   m_nBufferSize = %u ; m_nFileSize = %u\n", m_nBufferSize, m_nFileSize);
#endif
	
	ASSERT_VALID(this);

	return;
}


BYTE *CMyMemFile::Detach(void)
{
	BYTE *pBuffer;

#ifdef DEBUG
	TRACE0("CMyMemFile::Detach() entered\n");
#endif
	
	ASSERT_VALID(this);

	pBuffer = m_lpBuffer;
	
	m_nPosition = 0;
	m_nBufferSize = 0;
	m_nFileSize = 0;
	m_lpBuffer = NULL;
	m_nBufferSize = 0;
	
	return pBuffer;
}


BOOL CMyMemFile::ReadFrom(HINSTANCE hInst, LPCSTR szResource)
{
	tResource resource;
	void *pData;
	DWORD lSize;

	if ( (resource.find(hInst, szResource, RT_RCDATA))
		&& (resource.load())
		&& (lSize = resource.size())
		&& (pData = resource.lock()) )
		{
		if (lSize <= UINT_MAX)
			{
			Write(pData, (UINT)lSize);
			return TRUE;
			}
		}

  return FALSE;
}


BOOL CMyMemFile::WriteBits(unsigned long lCode, int iBitCount)
{
	unsigned long uMask;

	uMask = 1L << (iBitCount - 1);
	while (uMask != 0)
		{
		if (uMask & lCode)
			cBitWriteRack |= cBitWriteMask;
		cBitWriteMask >>= 1;
		if (cBitWriteMask == 0)
			{
			Write(&cBitWriteRack, 1);
			cBitWriteRack = 0;
			cBitWriteMask = cInitMask;
			}
		uMask >>= 1;
		}

  return TRUE;
}


BOOL CMyMemFile::ReadBits(unsigned long *pCode, int iBitCount)
{
	unsigned long uMask;
	unsigned long uRet;

	uMask = 1L << (iBitCount - 1);
	uRet = 0;

	while (uMask != 0)
		{
		if (cBitReadMask == cInitMask)
			Read(&cBitReadRack, 1);
		if (cBitReadRack & cBitReadMask)
			uRet |= uMask;
		uMask >>= 1;
		cBitReadMask >>= 1;
		if (cBitReadMask == 0)
			cBitReadMask = cInitMask;
		}

	*pCode = uRet;

	return 1;
}


BOOL CMyMemFile::FlushBits(void)
{
	if (cBitWriteMask != cInitMask)
		{
		cBitWriteMask = cInitMask;
		Write(&cBitWriteRack, 1);
		}
		
	return TRUE;
}


BOOL CMyMemFile::_UseBits(int nBits)
{
	if ( (nBits > 8) || (nBits < 1) )
		return FALSE;

	cInitMask = (1 << (nBits - 1));		// 0x80 for 8 bits
	cBitReadMask = cInitMask;
	cBitWriteMask = cInitMask;
	
	return TRUE;
}


