// ------------------------------------------------------------------------
//
//   Create a class that extends the standard MFC CFile classes
//
//   Copyright 1993, Chris Shearer Cooper  CompuServe 76666,3026
//   This file may be copied and distributed free of charge as long as
//   this copyright notice remains intact and any changes by other
//   authors are clearly indicated.
//   This code may be used in your application without any monetary
//   royalty, however payment is required in the form of feedback and
//   bug reports to the author.  Although every effort has been made
//   to make this code bug-free, if you use this code in your application
//   the author takes no responsibility for any damages which may occur
//   from the operation of this code.
//
// ------------------------------------------------------------------------

#ifndef __MYFILE_H

#include <limits.h>

	
class CMyFile : public CFile
	{
	DECLARE_DYNAMIC(CMyFile)
public:
	__export CMyFile();
	__export CMyFile(int hFile);
	__export CMyFile(const char *pszFileName, UINT nOpenFlags);

	virtual BOOL __export Open(const char *pszFileName, UINT nOpenFlags, CFileException *pException = NULL);
	    
    BOOL __export IsOpen(void)
    	{ return (BOOL)(m_hFile != CFile::hFileNull); }
    	
	// Make it easier to read & write basic types
	virtual void Write(const void FAR *lpBuf, UINT nCount)
		{ CFile::Write(lpBuf, nCount); return; }
	void Write(int &iValue)		{ CFile::Write(&iValue, sizeof(int)); return; }
	void Write(long &lValue)	{ CFile::Write(&lValue, sizeof(long)); return; }
	virtual UINT Read(void FAR *lpBuf, UINT nCount)
		{ return CFile::Read(lpBuf, nCount); }
	UINT Read(int &iValue)		{ return CFile::Read(&iValue, sizeof(int)); }
	UINT Read(long &lValue)		{ return CFile::Read(&lValue, sizeof(long)); }

	// Read and write lots of data
	void __export BigWrite(const void huge *lpBuf, DWORD lCount);
	DWORD __export BigRead(void huge *lpBuf, DWORD lCount);
	
	// These functions allocate memory and thus may throw CMemoryExceptions
	BOOL __export Insert(long nBytes);
	BOOL __export Remove(long nBytes);

	int __export GetError(void);

	// Add another enum to the list of possible errors
	enum {
		noMemory = 0x4100
		};
	
protected:
	int m_iError;
	};
	
	
class CMyMemFile : public CMemFile
	{
	DECLARE_DYNAMIC(CMyMemFile)

public:
	__export CMyMemFile(UINT nGrowBytes = 1024);

	const BYTE *GetBuffer(void)
		{ return m_lpBuffer; }

	void __export Attach(BYTE *pBuffer, UINT m_nFileSize = UINT_MAX);
	// The pointer returned from Detach() has been _fmalloc()ed so make sure to
	// _ffree() it when you're done with it!
	BYTE * __export Detach(void);

	BOOL __export ReadFrom(HINSTANCE hInst, LPCSTR szResource);
	
	// Bit access
	BOOL __export WriteBits(unsigned long lCode, int iBitCount);
	BOOL __export ReadBits(unsigned long *pCode, int iBitCount);
	// Make sure that there are no bits still waiting to be written
	BOOL __export FlushBits(void);

	// By setting a value with this function that is less than 8, only the lower
	// nBits of each byte will be used - the upper unused bits will be 0.
	BOOL __export _UseBits(int nBits = 8);

	virtual void __export Close(BOOL bSaveGrowBytes = TRUE);
	BOOL __export Abort(BOOL bSaveGrowBytes = TRUE);

	// Add another enum to the list of possible errors
	enum {
		nullBuffer = 0x4200
		};
	
protected:
	// These values are used to support bitwise reads and writes.  Code 
	// adapted from "The Data Compression Book", Mark Nelson, M&T Books.
	unsigned char cBitReadMask, cBitWriteMask;
	unsigned char cBitReadRack, cBitWriteRack;
	unsigned char cInitMask;
	};


#define __MYFILE_H
#endif


