//     (C) Copyright Microsoft Corp. 1991.  All rights reserved.
//
//     You have a royalty-free right to use, modify, reproduce and 
//     distribute the Sample Files (and/or any modified version) in 
//     any way you find useful, provided that you agree that 
//     Microsoft has no warranty obligations or liability for any 
//     Sample Application Files which are modified. 

/****************************************************************************

    MODULE    : plyvideo.C

    PURPOSE   : Demonstrates the use of MCI functions to control a VideoDisc
		drive.

    FUNCTIONS :  About
		 PlayVDInit
		 PlayVDWinProc
		 WinMain
		 OpenVDDevice
		 PlayVD
		 PlayVDVariableSpeed
		 CloseVDDevice
		 PauseVD
		 StopVD
		 GoToHMS
		 GoToNextFrame
		 GoToPrevFrame
		 GetPositionHMS
		 GetPositionFrames
		 VDEject
		 ErrorProc
		 IsVDFormatCAV



    COMMENTS  : This program implements playing a Videodisc using the MCI command
		interface.

    HISTORY   :

****************************************************************************/


#include "windows.h"
#include "plyvideo.h"
#include "mmsystem.h"

char szAppName[10];
char szAbout[10];
char szMessage[15];
int MessageLength;


FARPROC lpprocAbout;
HANDLE hLibrary;

WORD wGlobalDeviceID=0;
HMENU hMyMenu;

FARPROC lpFrameDlgProc;
FARPROC lpHMSDlgProc;
FARPROC lpSpeedDlgProc;





static HANDLE hInst;
FARPROC lpprocAbout;

// forward declarations.

WORD FAR PASCAL OpenVDDevice(HWND);
void FAR PASCAL PlayVD(HWND,WORD,DWORD);
void FAR PASCAL PlayVDVariableSpeed(HWND,WORD,DWORD,int);
WORD FAR PASCAL CloseVDDevice(HWND,WORD);
BOOL FAR PASCAL PauseVD(HWND,WORD);
void FAR PASCAL StopVD(HWND,WORD);
DWORD FAR PASCAL GoToHMS(HWND,WORD,int,int,int);
BOOL FAR PASCAL GoToNextFrame(HWND,WORD,int);
BOOL FAR PASCAL GoToPrevFrame(HWND,WORD,int);
DWORD FAR PASCAL GetPositionHMS(HWND,WORD);
DWORD FAR PASCAL GetPositionFrames(HWND,WORD);
void FAR PASCAL VDEject(HWND,WORD);
void FAR PASCAL ErrorProc(DWORD);
BOOL FAR PASCAL IsVDFormatCAV(HWND,WORD);

long FAR PASCAL PlayVDWndProc(HWND, unsigned, WORD, LONG);
BOOL FAR PASCAL HMSDlgProc(HWND, unsigned, WORD, LONG);
BOOL FAR PASCAL FrameDlgProc(HWND, unsigned, WORD, LONG);
BOOL FAR PASCAL SpeedDlgProc(HWND, unsigned, WORD, LONG);


 /****************************************************************************

    FUNCTION  :  About( HWND, unsigned, WORD, LONG )

    PURPOSE   :  This function processes Dialog Box messages associated with
		 the about dialog box.

    COMMENTS  :


    HISTORY   :

****************************************************************************/

BOOL FAR PASCAL About( hDlg, message, wParam, lParam )
HWND hDlg;
unsigned message;
WORD wParam;
LONG lParam;
{
    if (message == WM_COMMAND) {
        EndDialog( hDlg, TRUE );
        return TRUE;
        }
    else if (message == WM_INITDIALOG)
        return TRUE;
    else return FALSE;
}



 /****************************************************************************

    FUNCTION  :  HMSDlgProc( HWND, unsigned, WORD, LONG )

    PURPOSE   :  This function processes Dialog Box messages associated with
		 the about dialog box.

    COMMENTS  :


    HISTORY   :

****************************************************************************/

BOOL FAR PASCAL HMSDlgProc(hDlg, message, wParam, lParam)
HWND hDlg;				  /* window handle of the dialog box */
unsigned message;                         /* type of message                 */
WORD wParam;                              /* message-specific information    */
LONG lParam;
{

    BOOL bTranslated;
    int minute;
    int second;
    int hour;

    switch (message)
    {

	case WM_INITDIALOG:		   /* message: initialize dialog box */
	    SetDlgItemInt(hDlg, HMS_EDIT_HOUR,0,0);
	    SetDlgItemInt(hDlg, HMS_EDIT_MINUTE,0,0);
	    SetDlgItemInt(hDlg, HMS_EDIT_SECOND,0,0);

	    return (TRUE);

	case WM_COMMAND:		      /* message: received a command */
	    switch (wParam)
	    {

		case IDOK:

		   hour = GetDlgItemInt(hDlg, HMS_EDIT_HOUR, &bTranslated,0);
		   if (!bTranslated)
		       {
			MessageBox(NULL,"Please enter a correct hour","ERROR",MB_OK);
			return FALSE;
			}

		   minute = GetDlgItemInt(hDlg, HMS_EDIT_MINUTE, &bTranslated,0);
		   if (!bTranslated)
			{
			MessageBox(NULL,"Please enter a correct minute","ERROR",MB_OK);
			return FALSE;
			}

		   second = GetDlgItemInt(hDlg, HMS_EDIT_SECOND, &bTranslated,0);
		   if (!bTranslated)
			{
			MessageBox(NULL,"Please enter a correct second","ERROR",MB_OK);
			return FALSE;
			}

		    GoToHMS(hDlg,wGlobalDeviceID,hour,minute,second);

		    EndDialog(hDlg, TRUE);

		return(TRUE);



	    }
	    break;
    }
    return (FALSE);			      /* Didn't process a message    */
}


 /****************************************************************************

    FUNCTION  :  FrameDlgProc( HWND, unsigned, WORD, LONG )

    PURPOSE   :  This function processes Dialog Box messages associated with
		 the frame dialog box.

    COMMENTS  :


    HISTORY   :

****************************************************************************/

BOOL FAR PASCAL FrameDlgProc(hDlg, message, wParam, lParam)
HWND hDlg;				  /* window handle of the dialog box */
unsigned message;                         /* type of message                 */
WORD wParam;                              /* message-specific information    */
LONG lParam;
{

    BOOL bTranslated;
    int numofframes;

    switch (message)
    {

	case WM_INITDIALOG:		   /* message: initialize dialog box */

	    SetDlgItemInt(hDlg, FRAME_EDIT,0,0);

	    return (TRUE);

	case WM_COMMAND:		      /* message: received a command */
	    switch (wParam)
	    {

		case FRAME_PREV:

		     numofframes = GetDlgItemInt(hDlg, FRAME_EDIT, &bTranslated,0);
		     if (!bTranslated)
			 {
			 MessageBox(NULL,"Entering Frame","ERROR",MB_OK);
			 return (FALSE);
			 }

		    GoToPrevFrame(hDlg,wGlobalDeviceID,numofframes);

		return(TRUE);


		case FRAME_NEXT:

		     numofframes = GetDlgItemInt(hDlg, FRAME_EDIT, &bTranslated,0);
		     if (!bTranslated)
			 {
			 MessageBox(NULL,"Entering Frame","ERROR",MB_OK);
			 return (FALSE);
			 }

		     GoToNextFrame(hDlg,wGlobalDeviceID,numofframes);

		return(TRUE);

		case FRAME_EXIT:

		    EndDialog(hDlg, TRUE);

		return(TRUE);


	    }
	    break;
    }
    return (FALSE);			      /* Didn't process a message    */
}


 /****************************************************************************

    FUNCTION  :  SpeedDlgProc( HWND, unsigned, WORD, LONG )

    PURPOSE   :  This function processes Dialog Box messages associated with
		 the speed dialog box.

    COMMENTS  :


    HISTORY   :

****************************************************************************/

BOOL FAR PASCAL SpeedDlgProc(hDlg, message, wParam, lParam)
HWND hDlg;				  /* window handle of the dialog box */
unsigned message;                         /* type of message                 */
WORD wParam;                              /* message-specific information    */
LONG lParam;
{

    BOOL bTranslated;
    int speed;

    switch (message)
    {

	case WM_INITDIALOG:		   /* message: initialize dialog box */

	    SetDlgItemInt(hDlg, SPEED_EDIT,0,0);

	    return (TRUE);

	case WM_COMMAND:		      /* message: received a command */
	    switch (wParam)
	    {

		case IDOK:

		     speed = GetDlgItemInt(hDlg, SPEED_EDIT, &bTranslated,0);
		     if (!bTranslated)
			 {
			 MessageBox(NULL,"Entering Speed","ERROR",MB_OK);
			 return (FALSE);
			 }

		    PlayVDVariableSpeed(hDlg,wGlobalDeviceID,MCI_VD_PLAY_SPEED,speed);

		return(TRUE);


		case IDCANCEL:

		    EndDialog(hDlg, TRUE);
		return(TRUE);


	    }
	    break;
    }
    return (FALSE);			      /* Didn't process a message    */
}

void PlayVDPaint( hDC )
HDC hDC;
{
    TextOut( hDC,
             (short)10,
             (short)10,
	     (LPSTR)"VD output",
	     (short)9);
}


/* Procedure called when the application is loaded for the first time */
BOOL PlayVDInit( hInstance )
HANDLE hInstance;
{
    PWNDCLASS	pplayVDClass;

    /* Load strings from resource */
    LoadString( hInstance, IDSNAME, (LPSTR)szAppName, 10 );
    LoadString( hInstance, IDSABOUT, (LPSTR)szAbout, 10 );
    MessageLength = LoadString( hInstance, IDSTITLE, (LPSTR)szMessage, 15 );

    pplayVDClass = (PWNDCLASS)LocalAlloc( LPTR, sizeof(WNDCLASS) );

    pplayVDClass->hCursor	  = LoadCursor( NULL, IDC_ARROW );
    pplayVDClass->hIcon	  = LoadIcon( hInstance, MAKEINTRESOURCE(PLAYVDICON) );
    pplayVDClass->lpszMenuName	= (LPSTR)"TheMenu";
    pplayVDClass->lpszClassName  = (LPSTR)szAppName;
    pplayVDClass->hbrBackground  = (HBRUSH)GetStockObject( WHITE_BRUSH );
    pplayVDClass->hInstance	  = hInstance;
    pplayVDClass->style	  = CS_HREDRAW | CS_VREDRAW;
    pplayVDClass->lpfnWndProc	  = PlayVDWndProc;

    if (!RegisterClass( (LPWNDCLASS)pplayVDClass ) )
        /* Initialization failed.
         * Windows will automatically deallocate all allocated memory.
         */
        return FALSE;

    LocalFree( (HANDLE)pplayVDClass );
    return TRUE;        /* Initialization succeeded */
}


int PASCAL WinMain( hInstance, hPrevInstance, lpszCmdLine, cmdShow )
HANDLE hInstance, hPrevInstance;
LPSTR lpszCmdLine;
int cmdShow;
{
    MSG   msg;
    HWND  hWnd;
    HMENU hMenu;
    HMENU hMyMenu;

    if (!hPrevInstance) {
        /* Call initialization procedure if this is the first instance */
	if (!PlayVDInit( hInstance ))
            return FALSE;
        }
    else {
        /* Copy data from previous instance */
        GetInstanceData( hPrevInstance, (PSTR)szAppName, 10 );
        GetInstanceData( hPrevInstance, (PSTR)szAbout, 10 );
        GetInstanceData( hPrevInstance, (PSTR)szMessage, 15 );
        GetInstanceData( hPrevInstance, (PSTR)&MessageLength, sizeof(int) );
        }

    hWnd = CreateWindow((LPSTR)szAppName,
                        (LPSTR)szMessage,
                        WS_TILEDWINDOW,
                        0,    /*  x - ignored for tiled windows */
                        0,    /*  y - ignored for tiled windows */
			200,	/* cx - ignored for tiled windows */
			200,	/* cy - ignored for tiled windows */
                        (HWND)NULL,        /* no parent */
                        (HMENU)NULL,       /* use class menu */
                        (HANDLE)hInstance, /* handle to window instance */
                        (LPSTR)NULL        /* no params to pass on */
                        );

    /* Save instance handle for DialogBox */
    hInst = hInstance;

    /* Bind callback function with module instance */
    lpprocAbout = MakeProcInstance( (FARPROC)About, hInstance );

    /* Insert "About..." into system menu */
    hMenu = GetSystemMenu(hWnd, FALSE);
    ChangeMenu(hMenu, 0, NULL, 999, MF_APPEND | MF_SEPARATOR);
    ChangeMenu(hMenu, 0, (LPSTR)szAbout, IDSABOUT, MF_APPEND | MF_STRING);


       /****************************************************************
	This is where the Videodisc device is initially opened through	the
	function OpenVDDevice which in turn calls mciSendCommand.
	The device is opened at this point so that the user can immediately
	start playing the Videodisc.  Also the loading of the driver takes some
	time and it would be better for the user if the waiting happened
	at init time vs waiting additional time when they are in the app.
	The function returns a Device ID which is used to identify the
	Videodisc driver.  Please see the actual function declaration header
	for complete details.
	****************************************************************/



	  wGlobalDeviceID = OpenVDDevice(hWnd);

	 /* grey menu item first */
	 /* enable second */

	 if (wGlobalDeviceID)
	      {
	       hMyMenu=GetMenu(hWnd);

	       ModifyMenu(hMyMenu, IDM_OPEN_DEVICE, MF_BYCOMMAND | MF_GRAYED,IDM_OPEN_DEVICE,"&Open VD Device");
	       ModifyMenu(hMyMenu, IDM_CLOSE_DEVICE, MF_BYCOMMAND | MF_ENABLED,IDM_CLOSE_DEVICE,"&Close Videodisc Device");
	      }

	 if(!IsVDFormatCAV(hWnd,wGlobalDeviceID))
	      {
	      MessageBox(NULL,"This disc is not in CAV format therefore many options may not be supported","Caution",MB_OK);
	      ModifyMenu(hMyMenu, IDM_GET_FRAMES, MF_BYCOMMAND | MF_GRAYED,IDM_GET_FRAMES,"Get Position &Frames");
	      ModifyMenu(hMyMenu, IDM_GO_TO_NEXT_FRAME, MF_BYCOMMAND | MF_GRAYED, IDM_GO_TO_NEXT_FRAME,"Go To &Next Frame");
	      ModifyMenu(hMyMenu, IDM_GO_TO_PREV_FRAME, MF_BYCOMMAND | MF_GRAYED,IDM_GO_TO_PREV_FRAME,"Go To &Previous Frame");
	      ModifyMenu(hMyMenu, IDM_GO_TO_ANY_FRAME, MF_BYCOMMAND | MF_GRAYED,IDM_GO_TO_ANY_FRAME,"Go To &Any Frame");
	      ModifyMenu(hMyMenu, IDM_PLAY_REVERSE, MF_BYCOMMAND | MF_GRAYED,IDM_PLAY_REVERSE,"Play Videodisc in Reverse");
	      ModifyMenu(hMyMenu, IDM_PLAY_SLOW, MF_BYCOMMAND | MF_GRAYED,IDM_PLAY_SLOW,"play Videodisc slow");
	      ModifyMenu(hMyMenu, IDM_PLAY_FAST, MF_BYCOMMAND | MF_GRAYED,IDM_PLAY_FAST,"play Videodisc fast");
	      ModifyMenu(hMyMenu, IDM_PLAY_SCAN, MF_BYCOMMAND | MF_GRAYED,IDM_PLAY_SCAN,"Play Videodisc scan mode");
	      ModifyMenu(hMyMenu, IDM_PICK_SPEED, MF_BYCOMMAND | MF_GRAYED,IDM_PICK_SPEED,"play Videodisc slow");

	      }


    /* Make window visible according to the way the app is activated */
    ShowWindow( hWnd, cmdShow );
    UpdateWindow( hWnd );

    /* Polling messages from event queue */
    while (GetMessage((LPMSG)&msg, NULL, 0, 0)) {
        TranslateMessage((LPMSG)&msg);
        DispatchMessage((LPMSG)&msg);
        }

    return (int)msg.wParam;
}


 /****************************************************************************

    FUNCTION  :  PlayVDWndProc( HWND, unsigned, WORD, LONG)

    PURPOSE   :  This is the main PlayVDWnd procedure.	This function
		 calls into several functions which in turn call the
		 MCI command interface.

    COMMENTS  :


    HISTORY   :

****************************************************************************/

long FAR PASCAL PlayVDWndProc( hWnd, message, wParam, lParam )
HWND hWnd;
unsigned message;
WORD wParam;
LONG lParam;
{
    PAINTSTRUCT ps;

    BOOL bReturn;
    DWORD dwCurrentTrack;
    char szCurrentTrack[80];
    DWORD dwPosition;
    int hour;
    int minute;
    int second;
    int frame;
    int track;
    char szPosition[80];


    switch (message)
    {
    case WM_SYSCOMMAND:
        switch (wParam)
	{

        case IDSABOUT:
            DialogBox( hInst, MAKEINTRESOURCE(ABOUTBOX), hWnd, lpprocAbout );
	    break;

        default:
	    return DefWindowProc( hWnd, message, wParam, lParam );

        }
	break;

    case WM_COMMAND:
	switch(wParam)
	     {
	     case IDM_OPEN_DEVICE:

	/****************************************************************
	When the user selects open device the device is opened through the
	function OpenVDDevice which calls mciSendCommand.
	The function returns a Device ID which is used to identify the
	Videodisc driver.  Please see the actual function declaration header
	for complete details.
	****************************************************************/



		 wGlobalDeviceID = OpenVDDevice(hWnd);

		 if(!IsVDFormatCAV(hWnd,wGlobalDeviceID))
		      {
		      MessageBox(NULL,"This disc is not in CAV format therefore many options may not be supported","Caution",MB_OK);
		      ModifyMenu(hMyMenu, IDM_GET_FRAMES, MF_BYCOMMAND | MF_GRAYED,IDM_GET_FRAMES,"Get Position &Frames");
		      ModifyMenu(hMyMenu, IDM_GO_TO_NEXT_FRAME, MF_BYCOMMAND | MF_GRAYED, IDM_GO_TO_NEXT_FRAME,"Go To &Next Frame");
		      ModifyMenu(hMyMenu, IDM_GO_TO_PREV_FRAME, MF_BYCOMMAND | MF_GRAYED,IDM_GO_TO_PREV_FRAME,"Go To &Previous Frame");
		      ModifyMenu(hMyMenu, IDM_GO_TO_ANY_FRAME, MF_BYCOMMAND | MF_GRAYED,IDM_GO_TO_ANY_FRAME,"Go To &Any Frame");
		      ModifyMenu(hMyMenu, IDM_PLAY_REVERSE, MF_BYCOMMAND | MF_GRAYED,IDM_PLAY_REVERSE,"Play Videodisc in Reverse");
		      ModifyMenu(hMyMenu, IDM_PLAY_SLOW, MF_BYCOMMAND | MF_GRAYED,IDM_PLAY_SLOW,"play Videodisc slow");
		      ModifyMenu(hMyMenu, IDM_PLAY_FAST, MF_BYCOMMAND | MF_GRAYED,IDM_PLAY_FAST,"play Videodisc fast");
		      ModifyMenu(hMyMenu, IDM_PLAY_SCAN, MF_BYCOMMAND | MF_GRAYED,IDM_PLAY_SCAN,"Play Videodisc scan mode");
		      ModifyMenu(hMyMenu, IDM_PICK_SPEED, MF_BYCOMMAND | MF_GRAYED,IDM_PICK_SPEED,"play Videodisc slow");
		      }

		 /* grey menu item first */
		 /* enable second */

		 if (wGlobalDeviceID)
		      {
		       hMyMenu=GetMenu(hWnd);

		       ModifyMenu(hMyMenu, IDM_OPEN_DEVICE, MF_BYCOMMAND | MF_GRAYED,IDM_OPEN_DEVICE,"&Open Videodisc Device");
		       ModifyMenu(hMyMenu, IDM_CLOSE_DEVICE, MF_BYCOMMAND | MF_ENABLED,IDM_CLOSE_DEVICE,"&Close Videodisc Device");
		      }

	     break;

	     case IDM_PLAY:


	/****************************************************************
	When the user selects play, the Videodisc is played through
	the function PlayVD which calls mciSendCommand.
	The function is passed a Device ID which is used to identify the
	global "Videodisc" driver.  Please see the actual function declaration
	header for complete details.
	****************************************************************/


			PlayVD(hWnd,wGlobalDeviceID,0);
	     break;

	     case IDM_CLOSE_DEVICE:


       /****************************************************************
	When the user selects Close, the Videodisc device is Closed through
	the function CloseVDDevice which calls mciSendCommand.
	The function is passed a Device ID which is used to identify the
	global "Videodisc" driver.  Please see the actual function declaration
	header for complete details.
	****************************************************************/

		      wGlobalDeviceID = CloseVDDevice(hWnd,wGlobalDeviceID);

		       /* grey menu item first */
		       /* enable second */

		      if (!wGlobalDeviceID)
			    {
			     hMyMenu=GetMenu(hWnd);

			     ModifyMenu(hMyMenu, IDM_OPEN_DEVICE, MF_BYCOMMAND | MF_ENABLED,IDM_OPEN_DEVICE,"&Open Videodisc Audio Device");
			     ModifyMenu(hMyMenu, IDM_CLOSE_DEVICE, MF_BYCOMMAND | MF_GRAYED,IDM_CLOSE_DEVICE,"&Close Videodisc Audio Device");


			     ModifyMenu(hMyMenu, IDM_GET_FRAMES, MF_BYCOMMAND | MF_GRAYED,IDM_GET_FRAMES,"Get Position &Frames");
			     ModifyMenu(hMyMenu, IDM_GO_TO_NEXT_FRAME, MF_BYCOMMAND | MF_GRAYED, IDM_GO_TO_NEXT_FRAME,"Go To &Next Frame");
			     ModifyMenu(hMyMenu, IDM_GO_TO_PREV_FRAME, MF_BYCOMMAND | MF_GRAYED,IDM_GO_TO_PREV_FRAME,"Go To &Previous Frame");
			     ModifyMenu(hMyMenu, IDM_GO_TO_ANY_FRAME, MF_BYCOMMAND | MF_GRAYED,IDM_GO_TO_ANY_FRAME,"Go To &Any Frame");
			     ModifyMenu(hMyMenu, IDM_PLAY_REVERSE, MF_BYCOMMAND | MF_GRAYED,IDM_PLAY_REVERSE,"Play Videodisc in Reverse");
			     ModifyMenu(hMyMenu, IDM_PLAY_SLOW, MF_BYCOMMAND | MF_GRAYED,IDM_PLAY_SLOW,"play Videodisc slow");
			     ModifyMenu(hMyMenu, IDM_PLAY_FAST, MF_BYCOMMAND | MF_GRAYED,IDM_PLAY_FAST,"play Videodisc fast");
			     ModifyMenu(hMyMenu, IDM_PLAY_SCAN, MF_BYCOMMAND | MF_GRAYED,IDM_PLAY_SCAN,"Play Videodisc scan mode");
			     ModifyMenu(hMyMenu, IDM_PICK_SPEED, MF_BYCOMMAND | MF_GRAYED,IDM_PICK_SPEED,"play Videodisc slow");

			    }


	     break;


	     case IDM_STOP:

	/****************************************************************
	When the user selects stop, the Videodisc device is stopped through
	the function StopVideodisc which in turn calls mciSendCommand.
	The function is passed a Device ID which is used to identify the
	global "Videodisc" driver.  Please see the actual function declaration
	header for complete details.
	****************************************************************/



		       StopVD(hWnd,wGlobalDeviceID);
	     break;

	     case IDM_PAUSE:

	/****************************************************************
	When the user selects pause, the Videodisc device is paused through
	the function PauseVD which in turn calls mciSendCommand.
	The function is passed a Device ID which is used to identify the
	global "Videodisc" driver.  Please see the actual function declaration
	header for complete details.
	****************************************************************/


		       PauseVD(hWnd,wGlobalDeviceID);

	     break;



	     case IDM_GO_TO_HMS:

	/****************************************************************
	When the user selects go to HMS, the Videodisc device seeks to the
	specified position through the dialog box hms.	The function is passed
	a Device ID which is used to identify the global "Videodisc" driver.
	Please see the actual function declaration header for complete details.
	****************************************************************/


		    lpHMSDlgProc = MakeProcInstance(HMSDlgProc, hInst);
		    DialogBox(hInst,
			      "hms",
			      hWnd,
			      lpHMSDlgProc);
		    FreeProcInstance(lpHMSDlgProc);


	     break;

	     case IDM_GO_TO_PREV_FRAME:

	/****************************************************************
	When the user selects go to previous frame, the Videodisc device seeks
	to the next frame through the function GoToPrevFrame which in turn
	calls mciSendCommand.	The function is passed a Device ID which is
	used to identify the global "Videodisc" driver.	Please see the actual
	function declaration header for complete details.
	****************************************************************/


		  if(!GoToPrevFrame(hWnd,wGlobalDeviceID,36))
		       MessageBox(NULL,"Failed to decrement frame","ERROR",MB_OK);


	     break;

	     case IDM_GO_TO_NEXT_FRAME:

	/****************************************************************
	When the user selects go to next frame, the Videodisc device seeks to
	the next frame through the function GoToNextFrame which in turn calls
	mciSendCommand.	The function is passed a Device ID which is used to
	identify the global "Videodisc" driver.	Please see the actual function
	declaration header for complete details.
	****************************************************************/


		  if(!GoToNextFrame(hWnd,wGlobalDeviceID,36))
		       MessageBox(NULL,"Failed to increment frame","ERROR",MB_OK);


	     break;

	/****************************************************************
	When the user selects go to any frame, the Videodisc device seeks to
	the next frame/prev frame or a specified frame through the dialog box
	frameadv.  Please see the actual dialog box declaration header for
	complete details.
	****************************************************************/

	     case IDM_GO_TO_ANY_FRAME:
		    lpFrameDlgProc = MakeProcInstance(FrameDlgProc, hInst);
		    DialogBox(hInst,
			      "FRAMEADV",
			      hWnd,
			      lpFrameDlgProc);
		    FreeProcInstance(lpFrameDlgProc);





	     case IDM_GET_HMS:

	/****************************************************************
	When the user selects Get HMS, the Videodisc device returns the
	current position in HMS format (Hour Minute Second).	This is
	done through the function GetPositionHMS which calls mciSendCommand.
	The function is passed a Device ID which is used to identify
	the global "Videodisc" driver.	Please see the actual function
	declaration header for complete details.
	****************************************************************/



		  dwPosition = GetPositionHMS(hWnd,wGlobalDeviceID);

		  hour	 = MCI_HMS_HOUR(dwPosition);
		  minute = MCI_HMS_MINUTE(dwPosition);
		  second = MCI_HMS_SECOND(dwPosition);


		  wsprintf(szPosition,"Hour = %d\nMinute = %d\nSecond = %d",hour,minute,second);
		  MessageBox(NULL,(LPSTR)szPosition,"The Current Position in TMSF",MB_OK);

	     break;

	     case IDM_GET_FRAMES:

	/****************************************************************
	When the user selects Get Frames, the Videodisc device returns the
	current position in the number of frames.	This is
	done through the function GetPositionFrames which calls mciSendCommand
	The function is passed a Device ID which is used to identify
	the global "Videodisc" driver.	Please see the actual function
	declaration header for complete details.
	****************************************************************/


		  dwPosition = GetPositionFrames(hWnd,wGlobalDeviceID);



		  wsprintf(szPosition,"Frames = %lu",dwPosition);
		  MessageBox(NULL,(LPSTR)szPosition,"The Current Position in Frames",MB_OK);



	     break;

	     case IDM_VD_EJECT:

	/****************************************************************
	When the user selects Eject, the Videodisc Ejects the Disk from the
	physical Videodisc player.  This is done through the function VDEject
	which calls mciSendCommand.	The function is passed a Device ID
	which is used to identify the global "Videodisc" driver.  Please see
	the actual function declaration header for complete details.
	****************************************************************/


		       VDEject(hWnd,wGlobalDeviceID);
	     break;



	 case IDM_PLAY_FAST:


	/****************************************************************
	When the user selects play fast, the Videodisc is played through
	the function PlayVD which calls mciSendCommand and is passed the
	MCI_VD_PLAY_FAST flag.
	The function is passed a Device ID which is used to identify the
	global "Videodisc" driver. Please see the actual function declaration
	header for complete details.
	****************************************************************/


			PlayVD(hWnd,wGlobalDeviceID,MCI_VD_PLAY_FAST);
	     break;

     case IDM_PLAY_SLOW:


	/****************************************************************
	When the user selects play slow, the Videodisc is played through
	the function PlayVD which calls mciSendCommand and is passed the
	MCI_VD_PLAY_SLOW flag.
	The function is passed a Device ID which is used to identify the
	global "Videodisc" driver.	Please see the actual function
	declaration header for complete details.
	****************************************************************/


			PlayVD(hWnd,wGlobalDeviceID,MCI_VD_PLAY_SLOW);
	     break;

	case IDM_PLAY_SCAN:


	/****************************************************************
	When the user selects play scan, the Videodisc is played through
	the function PlayVD which calls mciSendCommand and is passed the
	MCI_VD_PLAY_SCAN flag.
	The function is passed a Device ID which is used to identify the
	global "Videodisc" driver. Please see the actual function
	declaration header for complete details.
	****************************************************************/


			PlayVD(hWnd,wGlobalDeviceID,MCI_VD_PLAY_SCAN);
	     break;


	case IDM_PLAY_REVERSE:


	/****************************************************************
	When the user selects play reverse, the Videodisc is played through
	the function PlayVD which calls mciSendCommand and is passed the
	MCI_VD_PLAY_REVERSE flag.
	The function is passed a Device ID which is used to identify the
	global "Videodisc" driver.  Please see the actual function declaration
	header for complete detail.
	****************************************************************/


			PlayVD(hWnd,wGlobalDeviceID,MCI_VD_PLAY_REVERSE);
	break;


       case IDM_PICK_SPEED:


	/****************************************************************
	When the user selects play, the Videodisc is played through
	the function PlayVD which calls mciSendCommand.
	The function is passed a Device ID which is used to identify the
	global "Videodisc" driver.  Please see the actual function declaration
	header for complete details.
	****************************************************************/

		    lpSpeedDlgProc = MakeProcInstance(SpeedDlgProc, hInst);
		    DialogBox(hInst,
			      "speed",
			      hWnd,
			      lpSpeedDlgProc);
		    FreeProcInstance(lpSpeedDlgProc);


	     break;


	     case IDM_EXIT:
		  SendMessage(hWnd,WM_CLOSE,0,0L);
	     break;

	     case ID_HELP:
		if(!WinHelp(hWnd,"help.hlp",HELP_INDEX,0))
		      MessageBox(GetFocus(),"Help Not Found","ERROR",MB_OK);
		break;

	     }
	 break;



    case WM_DESTROY:

	if (wGlobalDeviceID!=0)
	    wGlobalDeviceID=CloseVDDevice(hWnd,wGlobalDeviceID);

        PostQuitMessage( 0 );
        break;

    case WM_PAINT:
        BeginPaint( hWnd, (LPPAINTSTRUCT)&ps );
	PlayVDPaint( ps.hdc );
        EndPaint( hWnd, (LPPAINTSTRUCT)&ps );
        break;

    default:
        return DefWindowProc( hWnd, message, wParam, lParam );
        break;
    }
    return(0L);
}


/****************************************************************************

    FUNCTION  :  OpenVDDevice(HWND)

    PURPOSE   :  This function opens the videodisc device and leaves it open.
		 The device could also be opened by assigning the string
		 "videodisc" as follows.

		 mciopen.lpstrDeviceType="videodisc"

		 Using this format you would only have to specify

		 MCI_OPEN_TYPE for dwflags.

    COMMENTS  :


    HISTORY   :

****************************************************************************/





WORD FAR PASCAL OpenVDDevice(hWnd)

HWND hWnd;
{

     MCI_OPEN_PARMS mciopen;
     DWORD dwRes;
     WORD wGlobalDeviceID;
     DWORD dwFlags;


     mciopen.wDeviceID=0;


     mciopen.lpstrDeviceType=(LPSTR)"videodisc";


     MessageBox(hWnd,"About to open the Device","YES",MB_OK);


     dwFlags= MCI_OPEN_TYPE | MCI_OPEN_SHAREABLE;

     dwRes=mciSendCommand(0,MCI_OPEN,dwFlags,(DWORD)(LPSTR)&mciopen);

     if (dwRes)
	{
	ErrorProc(dwRes);
	return(FALSE);
	}

     wGlobalDeviceID=mciopen.wDeviceID;

     return(wGlobalDeviceID);
     }


/****************************************************************************

    FUNCTION  :  PlayVD(HWND,WORD)

    PURPOSE   :  This function takes a currently open device ID and starts
		 the Videodisc playing.

    COMMENTS  :

    HISTORY   :

****************************************************************************/



void FAR PASCAL PlayVD(hWnd,wGlobalDeviceID,dwMode)

HWND hWnd;
WORD wGlobalDeviceID;
DWORD dwMode;
{

     MCI_OPEN_PARMS mciopen;
     MCI_VD_PLAY_PARMS mciplay;
     MCI_SET_PARMS mciset;
     MCI_STATUS_PARMS mcistatus;
     DWORD dwRes;
     WORD wDeviceID;
     DWORD dwFlags;
     char szWaveMessage[80];




     //set time format to HMS

     mciset.dwTimeFormat = MCI_FORMAT_HMS;
     dwFlags=MCI_SET_TIME_FORMAT;

     dwRes=mciSendCommand(wGlobalDeviceID,MCI_SET,dwFlags,(DWORD)(LPSTR)&mciset);

     if (dwRes)
	{
	  ErrorProc(dwRes);
	  return;
	}


     // play the VD.


     dwFlags=dwMode;

     dwRes=mciSendCommand(wGlobalDeviceID,MCI_PLAY,dwFlags,(DWORD)(LPSTR)&mciplay);

     if (dwRes)
	{
	  ErrorProc(dwRes);
	  return;
	}

     return;
     }



/****************************************************************************

    FUNCTION  :  PlayVDVariableSpeed(HWND,WORD,DWORD,int)

    PURPOSE   :  This function takes a currently open device ID and starts
		 the Videodisc playing at the speed provided by speed.

    COMMENTS  :

    HISTORY   :

****************************************************************************/



void FAR PASCAL PlayVDVariableSpeed(hWnd,wGlobalDeviceID,dwMode,speed)

HWND hWnd;
WORD wGlobalDeviceID;
DWORD dwMode;
int   speed;
{

     MCI_OPEN_PARMS mciopen;
     MCI_VD_PLAY_PARMS mciplay;
     MCI_SET_PARMS mciset;
     MCI_STATUS_PARMS mcistatus;
     DWORD dwRes;
     WORD wDeviceID;
     DWORD dwFlags;
     char szWaveMessage[80];




     //set time format to HMS

     mciset.dwTimeFormat = MCI_FORMAT_HMS;
     dwFlags=MCI_SET_TIME_FORMAT;

     dwRes=mciSendCommand(wGlobalDeviceID,MCI_SET,dwFlags,(DWORD)(LPSTR)&mciset);

     if (dwRes)
	{
	  ErrorProc(dwRes);
	  return;
	}


     // play the videodisc.

       mciplay.dwSpeed=speed;

     dwFlags=dwMode;

     dwRes=mciSendCommand(wGlobalDeviceID,MCI_PLAY,dwFlags,(DWORD)(LPSTR)&mciplay);

     if (dwRes)
	{
	  ErrorProc(dwRes);
	  return;
	}

     return;
     }



/****************************************************************************

    FUNCTION  :  CloseVDDevice(HWND,WORD)

    PURPOSE   :  This function closes a currently open videodisc device.


    COMMENTS  :	 To conserve system resources you should always close a
		 device when you are not using it.

    HISTORY   :

****************************************************************************/



WORD FAR PASCAL CloseVDDevice(hWnd,wGlobalDeviceID)

HWND hWnd;
WORD wGlobalDeviceID;

{
     DWORD dwRes;

     // Close the global device

      dwRes=mciSendCommand(wGlobalDeviceID,MCI_CLOSE,0,NULL);

     if (dwRes)
	{
	ErrorProc(dwRes);
	return(FALSE);
	}


       wGlobalDeviceID=0;

return(wGlobalDeviceID);
}

/****************************************************************************

    FUNCTION  :  StopVD(HWND,WORD)

    PURPOSE   :  This function stops a currently playing Videodisc device.


    COMMENTS  :

    HISTORY   :

****************************************************************************/



void FAR PASCAL StopVD(hWnd,wGlobalDeviceID)

HWND hWnd;
WORD wGlobalDeviceID;


{
     MCI_GENERIC_PARMS mcigeneric;
     DWORD dwRes;
     DWORD dwFlags;


     dwFlags=0;



     dwRes=mciSendCommand(wGlobalDeviceID,MCI_STOP,dwFlags,(DWORD)(LPSTR)&mcigeneric);

     if (dwRes)
	   {
	   ErrorProc(dwRes);
	   return;
	   }


     return;
}

/****************************************************************************

    FUNCTION  :  PauseVD(HWND,WORD)

    PURPOSE   :  This function pauses a currently playing Videodisc device.


    COMMENTS  :  To implement pause you must first test to see whether the
		 Videodisc is playing or paused.  If the status is paused
		 you can restart the Videodisc from the currently paused
		 position by playing.  If the Videodisc is playing then
		 Pause the Videodisc.

    HISTORY   :

****************************************************************************/


BOOL FAR PASCAL PauseVD(hWnd,wGlobalDeviceID)

HWND hWnd;
WORD wGlobalDeviceID;


{
     MCI_GENERIC_PARMS mcigeneric;
     MCI_STATUS_PARMS mcistatus;
     MCI_VD_PLAY_PARMS mciplay;
     DWORD dwFlags;
     DWORD dwRes;

     dwFlags=MCI_STATUS_ITEM;

     mcistatus.dwItem = MCI_STATUS_MODE;

     dwRes=mciSendCommand(wGlobalDeviceID,MCI_STATUS,dwFlags,(DWORD)(LPSTR)&mcistatus);

     if (dwRes)
	   {
	   ErrorProc(dwRes);
	   return(FALSE);
	   }


     if (mcistatus.dwReturn == MCI_MODE_PAUSE)
	  {
	  dwFlags=0;

	  dwRes=mciSendCommand(wGlobalDeviceID,MCI_PLAY,dwFlags,(DWORD)(LPSTR)&mciplay);

	  if (dwRes)
	       {
		ErrorProc(dwRes);
		return(FALSE);
	       }

	  }
      else
	  {
	   dwFlags=0;


	   dwRes=mciSendCommand(wGlobalDeviceID,MCI_PAUSE,dwFlags,(DWORD)(LPSTR)&mcigeneric);

	   if (dwRes)
		 {
		  ErrorProc(dwRes);
		  return(FALSE);
		 }
	  }

return(TRUE);

}

/****************************************************************************

    FUNCTION  :  GoToHMS(HWND,WORD,int,int,int)

    PURPOSE   :  This function goes to the position on the Videodisc
		 specified in HMS format (hour, minute, second).


    COMMENTS  :

    HISTORY   :

****************************************************************************/

DWORD FAR PASCAL GoToHMS(hWnd,wGlobalDeviceID,hour,minute,second)

HWND hWnd;
WORD wGlobalDeviceID;
int hour;
int minute;
int second;

{
     MCI_SEEK_PARMS mciseek;
     MCI_STATUS_PARMS mcistatus;
     MCI_PLAY_PARMS  mciplay;
     DWORD dwFlags;
     DWORD dwRes;
     DWORD dwCurrentTrack;
     DWORD dwNextTrack;
     LPSTR lpTrackString;


     dwFlags=MCI_TO;

     mciseek.dwTo=MCI_MAKE_HMS(hour,minute,second);

     dwRes=mciSendCommand(wGlobalDeviceID,MCI_SEEK,dwFlags,(DWORD)(LPSTR)&mciseek);

     if (dwRes)
	  {
	   ErrorProc(dwRes);
	   return(FALSE);
	  }

	  dwFlags=0;

	  dwRes=mciSendCommand(wGlobalDeviceID,MCI_PLAY,dwFlags,(DWORD)(LPSTR)&mciplay);

	  if (dwRes)
	       {
		ErrorProc(dwRes);
		return(FALSE);
	       }

return(dwNextTrack);

}

/****************************************************************************

    FUNCTION  :  GoToPrevFrame(HWND,WORD)

    PURPOSE   :  This function goes to the previous Frame on the Videodisc.


    COMMENTS  :

    HISTORY   :

****************************************************************************/


BOOL FAR PASCAL GoToPrevFrame(hWnd,wGlobalDeviceID,frameamount)

HWND hWnd;
WORD wGlobalDeviceID;
int frameamount;


{

     MCI_VD_STEP_PARMS mcistep;
     MCI_PLAY_PARMS  mciplay;
     DWORD dwFlags;
     DWORD dwRes;
     DWORD dwCurrentTrack;
     DWORD dwNextTrack;
     LPSTR lpTrackString;

     dwFlags=MCI_VD_STEP_REVERSE;

     mcistep.dwFrames=36;

     dwRes=mciSendCommand(wGlobalDeviceID,MCI_STEP,dwFlags,(DWORD)(LPSTR)&mcistep);

     if (dwRes)
	  {
	   ErrorProc(dwRes);
	   return(FALSE);
	  }



return TRUE;

}

/****************************************************************************

    FUNCTION  :  GoToNextFrame(HWND,WORD)

    PURPOSE   :  This function goes to the next frame on the Videodisc.


    COMMENTS  :

    HISTORY   :

****************************************************************************/


BOOL FAR PASCAL GoToNextFrame(hWnd,wGlobalDeviceID,frameamount)

HWND hWnd;
WORD wGlobalDeviceID;
int frameamount;


{
     MCI_VD_STEP_PARMS mcistep;
     MCI_PLAY_PARMS  mciplay;
     DWORD dwFlags;
     DWORD dwRes;
     DWORD dwCurrentTrack;
     DWORD dwNextTrack;
     LPSTR lpTrackString;

     dwFlags=MCI_VD_STEP_FRAMES;

     mcistep.dwFrames=frameamount;

     dwRes=mciSendCommand(wGlobalDeviceID,MCI_STEP,dwFlags,(DWORD)(LPSTR)&mcistep);

     if (dwRes)
	  {
	   ErrorProc(dwRes);
	   return(FALSE);
	  }


return TRUE;


}

/****************************************************************************

    FUNCTION  :  GetPositionFrames(HWND,WORD)

    PURPOSE   :  This function gets the current videodisc position in frames.


    COMMENTS  :

    HISTORY   :

****************************************************************************/

DWORD FAR PASCAL GetPositionFrames(hWnd,wGlobalDeviceID)

HWND hWnd;
WORD wGlobalDeviceID;


{
     MCI_STATUS_PARMS mcistatus;
     MCI_SET_PARMS    mciset;
     DWORD dwFlags;
     DWORD dwRes;

     mciset.dwTimeFormat = MCI_FORMAT_FRAMES;
     dwFlags=MCI_SET_TIME_FORMAT;

     dwRes=mciSendCommand(wGlobalDeviceID,MCI_SET,dwFlags,(DWORD)(LPSTR)&mciset);

     if (dwRes)
	{
	  ErrorProc(dwRes);
	  return(FALSE);
	}


     dwFlags=MCI_STATUS_ITEM;

     mcistatus.dwItem = MCI_STATUS_POSITION;

     dwRes=mciSendCommand(wGlobalDeviceID,MCI_STATUS,dwFlags,(DWORD)(LPSTR)&mcistatus);

     if (dwRes)
	   {
	   ErrorProc(dwRes);
	   return(FALSE);
	   }


     mciset.dwTimeFormat = MCI_FORMAT_HMS;
     dwFlags=MCI_SET_TIME_FORMAT;

     dwRes=mciSendCommand(wGlobalDeviceID,MCI_SET,dwFlags,(DWORD)(LPSTR)&mciset);

     if (dwRes)
	{
	  ErrorProc(dwRes);
	  return(FALSE);
	}

return(mcistatus.dwReturn);

}

/****************************************************************************

    FUNCTION  :  IsVDFormatCAV(HWND,WORD)

    PURPOSE   :  The function returns a true value if the format of the
		 currently playing videodisc is CAV otherwise it returns
		 false.

    COMMENTS  :

    HISTORY   :

****************************************************************************/

BOOL FAR PASCAL IsVDFormatCAV(hWnd,wGlobalDeviceID)

HWND hWnd;
WORD wGlobalDeviceID;


{
     MCI_STATUS_PARMS mcistatus;
     MCI_SET_PARMS    mciset;
     DWORD dwFlags;
     DWORD dwRes;



     dwFlags=MCI_STATUS_ITEM;

     mcistatus.dwItem = MCI_VD_STATUS_MEDIA_TYPE;

     dwRes=mciSendCommand(wGlobalDeviceID,MCI_STATUS,dwFlags,(DWORD)(LPSTR)&mcistatus);

     if (dwRes)
	   {
	   ErrorProc(dwRes);
	   return(FALSE);
	   }

    if(mcistatus.dwReturn==MCI_VD_MEDIA_CAV)
	  return TRUE;
    else
	  return FALSE;


}


/****************************************************************************

    FUNCTION  :  GetPositionHMS(HWND,WORD)

    PURPOSE   :  This function gets the current track playing on the Videodisc
		 in the MSF (Minute Second Frame) format.


    COMMENTS  :

    HISTORY   :

****************************************************************************/


DWORD FAR PASCAL GetPositionHMS(hWnd,wGlobalDeviceID)

HWND hWnd;
WORD wGlobalDeviceID;


{
     MCI_STATUS_PARMS mcistatus;
     MCI_SET_PARMS mciset;
     DWORD dwFlags;
     DWORD dwRes;


     mciset.dwTimeFormat = MCI_FORMAT_HMS;
     dwFlags=MCI_SET_TIME_FORMAT;

     dwRes=mciSendCommand(wGlobalDeviceID,MCI_SET,dwFlags,(DWORD)(LPSTR)&mciset);

     if (dwRes)
	{
	  ErrorProc(dwRes);
	  return(FALSE);
	}

     dwFlags=MCI_STATUS_ITEM;

     mcistatus.dwItem = MCI_STATUS_POSITION;

     dwRes=mciSendCommand(wGlobalDeviceID,MCI_STATUS,dwFlags,(DWORD)(LPSTR)&mcistatus);

     if (dwRes)
	   {
	   ErrorProc(dwRes);
	   return(FALSE);
	   }




return(mcistatus.dwReturn);

}

/****************************************************************************

    FUNCTION  :  VDEject(HWND,WORD)

    PURPOSE   :  This function Ejects the Videodisc from the physical Videodisc drive.


    COMMENTS  :

    HISTORY   :

****************************************************************************/



void FAR PASCAL VDEject(hWnd,wGlobalDeviceID)

HWND hWnd;
WORD wGlobalDeviceID;


{
     MCI_SET_PARMS mciset;
     DWORD dwFlags;
     DWORD dwRes;


     dwFlags=MCI_SET_DOOR_OPEN;

     dwRes=mciSendCommand(wGlobalDeviceID,MCI_SET,dwFlags,(DWORD)(LPSTR)&mciset);

     if (dwRes)
	{
	  ErrorProc(dwRes);
	  return;
	}

return;

}


/****************************************************************************

    FUNCTION  :  ErrorProc(WORD)

    PURPOSE   :  ErrorProc calls mciGetErrorString to display an error
		 message returned by MCI.

    COMMENTS  :

    HISTORY   :

****************************************************************************/

void FAR PASCAL ErrorProc(dwResult)

DWORD dwResult;

{
     HANDLE hMem;
     LPSTR lpStringBuff;


     hMem=GlobalAlloc(GHND,80);

     if (hMem)
	  {
	   lpStringBuff=GlobalLock(hMem);
	   if (lpStringBuff)
		{
		 if(mciGetErrorString(dwResult,lpStringBuff,80))
		       MessageBox(NULL,lpStringBuff,"ERROR",MB_OK);
		 else
		       MessageBox(NULL,"Generic Error","ERROR",MB_OK);

		 GlobalUnlock(hMem);
		}
	    else
		MessageBox(NULL,"Lock Failed","ERROR",MB_OK);

	    GlobalFree(hMem);
	    }
	else
	    MessageBox(NULL,"Alloc Failed","ERROR",MB_OK);

	return;
}
