//----------------------------------------------------------------------------
//
// strtable.c: VB Access to String Table Data in Resource File
//	(c) George E. Defenbaugh, Jr., 1993
//
//----------------------------------------------------------------------------

#include "strtable.h"

HINSTANCE			ghInstance;			// DLL's instance handle

// remove for production compile
#define DEBUG

//-----------------------------------------------------------------------------
//
// LibMain ()
//
//	Arguments:
//		normal LibMain arguments
//
// Return:
//    TRUE			initialization OK
//		0				initialization failed
//
//----------------------------------------------------------------------------

int FAR PASCAL LibMain (HINSTANCE hInstance, WORD wDataSeg,
		WORD wHeapSize, LPSTR lpszCmdLine)
{
	ghInstance = hInstance;    // save DLL's instance handle
	if (wHeapSize > 0)         // unlock local data segment if it's MOVEABLE
	{
		UnlockData (0);
	}

	return TRUE;
}	// LibMain

int CALLBACK WEP (int nParameter)
{
	 // any cleanup activities go here
	 return TRUE;
}	// WEP

//----------------------------------------------------------------------------
//
// vbGetStringTable ()	VB Access to String Table Data in Resource File
//		A Visual Basic specific function that demonstrates the following capabilities;
//
//		1) Ability of a DLL to access a Visual Basic string via a passed HLSTR parm.
//		2) Ability of a DLL to directly modify a VB string with data from a STRINGTABLE.
//		3)	Ability of a DLL to display debug information based on a parm value.
//
//	Arguments:
//		hlstrTextBox	Copy of VB string descriptor
//		nStrNbr			The number of the string from the StringTable
//		nDebug			Debug flag (0 == none, 1 == thru VB panel, 2 == thru MsgBox
//		hDebugWindow	Window handle of VB control where msgs should appear if nDebug == 1
//
// Return:
//		Length of the string that is placed in the text box
//
//----------------------------------------------------------------------------

int CALLBACK _export vbGetStringTable (HLSTR hlstrTextBox, int nStrNbr, int nDebug, HWND hDebugWindow)
{
	char	   szStrText[MAX_RESC_MSG_SIZE + 1];	// Holds text from LoadString
	int		nLoadStringRtn;
	ERR		errVBSetRtn;
	USHORT	usVBGetRtn;
	LONG		lVBSendRtn;

	// Get the value from the VB text box, identified by the HLSTR parm value
	usVBGetRtn = VBGetHlstr (hlstrTextBox, (LPVOID) szStrText, MAX_RESC_MSG_SIZE);

	if (nDebug != DEBUG_OFF)
	{
		char		szTemp[MAX_RESC_MSG_SIZE];				//	For debugging
		wsprintf (szTemp, "Return Code is: %d\nString is: %s", usVBGetRtn, (LPSTR) szStrText);
		if (nDebug == DEBUG_MSGBOX)
		{
			MessageBox (NULL, szTemp, "After VBGetHlstr", MB_OK | MB_ICONINFORMATION | MB_TASKMODAL);
		}
		else
		{
			lVBSendRtn = SendMessage (hDebugWindow, WM_SETTEXT, 0, (LONG)(LPSTR) szTemp);		
			MessageBox (NULL, "Press OK after you read the debug message", "", MB_OK | MB_ICONINFORMATION | MB_TASKMODAL);
		}
	}

	// Get the appropriate value from the StringTable
	nLoadStringRtn = LoadString (ghInstance, nStrNbr, szStrText, MAX_RESC_MSG_SIZE);

	if (nDebug != DEBUG_OFF)
	{
		char		szTemp[MAX_RESC_MSG_SIZE];				//	For debugging
		wsprintf (szTemp, "String is: %s", (LPSTR) szStrText);
		if (nDebug == DEBUG_MSGBOX)
		{
			MessageBox (NULL, szTemp, "After LoadString", MB_OK | MB_ICONINFORMATION | MB_TASKMODAL);
		}
		else
		{
			lVBSendRtn = SendMessage (hDebugWindow, WM_SETTEXT, 0, (LONG)(LPSTR) szTemp);		
			MessageBox (NULL, "Press OK after you read the debug message", "", MB_OK | MB_ICONINFORMATION | MB_TASKMODAL);
		}
	}

	// If LoadString failed, let them know
	if (nLoadStringRtn == 0)
	{
		strcpy (szStrText, "String Does Not Exist");
	}

	// Move the string back into VB data space
	errVBSetRtn = VBSetHlstr ((LPHLSTR) &hlstrTextBox, (LPVOID) szStrText, (USHORT) strlen (szStrText));

	if (nDebug != DEBUG_OFF)
	{
		char		szTemp[MAX_RESC_MSG_SIZE];
		wsprintf (szTemp, "Return Code is: %d", errVBSetRtn);
		if (nDebug == DEBUG_MSGBOX)
		{
			MessageBox (NULL, szTemp, "After VBSetHlstr", MB_OK | MB_ICONINFORMATION | MB_TASKMODAL);
		}
		else
		{
			lVBSendRtn = SendMessage (hDebugWindow, WM_SETTEXT, 0, (LONG)(LPSTR) szTemp);		
			MessageBox (NULL, "Press OK after you read the debug message", "", MB_OK | MB_ICONINFORMATION | MB_TASKMODAL);
		}
	}

	// Return the length of the text string
	return (strlen (szStrText));
}	// vbGetStringTable
