#-----------------------------------------------------------------------------
#  PROJECT DEPENDENCY STRUCTURE
#  
#  .DLL BUILD
#  
#     strtable.c
#        strtable.h
#				vbapi.h
#
#  .RES BUILD
#
#     strtable.rc
#        strtable.h
#				vbapi.h
#
#	SPECIAL LIBRARIES
#
#		vbapi.lib
#
#-----------------------------------------------------------------------------


# Turn on auto dependency

.AUTODEPEND


#  *Translator Definitions*

TARGET = $(PROJECT)
SOURCE = $(PROJECT).c

MODEL = large
CC = bcc
RC = brc
#RC = rc
TASM = tasm
TLIB = tlib
TLINK = tlink
#TLINK_OPTIONS = /v/x/c/P         # /v  - includes debuggin info
TLINK_OPTIONS = /v/x/P         # /v  - includes debuggin info
											# /x  - produces no map
											# /c  - case sensitive public externals
											# /P  - code segment packing

COMPILE_TYPE = -WDE
OBJ_TYPE = d
TLINK_TYPE = /Twd

#  LIB and INCLUDE will be environment variables
#     LIB = D:\BCPP31\LIB
#     INCLUDE = D:\BCPP31\INCLUDE
#  PROJECT comes from batch file ($(PROJECT).cfg is configuration file)
#  MAKEFILE comes from batch file ($(MAKEFILE).mak is this file)

!if $(MODEL) == small
MMCOMPILE = -ms
MMLIB = cws
MMOBJ = c0$(OBJ_TYPE)s
!elif $(MODEL) == medium
MMCOMPILE = -mm
MMLIB = cwm
MMOBJ = c0$(OBJ_TYPE)m
!elif $(MODEL) == large
MMCOMPILE = -ml
MMLIB = cwl
MMOBJ = c0$(OBJ_TYPE)l
!elif $(MODEL) == compact
MMCOMPILE = -mc
MMLIB = cwc
MMOBJ = c0$(OBJ_TYPE)c
!else
!error MODEL doesn't have an allowable value
!endif


#  *Implicit Rules*

#  Implicit rule of .c.obj means "any file with extension of .c can be
#     translated into file of same name with .obj extension with this sequence
#     of commands". Make uses an implicit rule if it can't find any explicit
#     rules for a given target or if an explicit rule with no commands exists
#     for the target.

#  Braces denote batching: Execution of command is delayed until _next_
#     command. If next command is same except part in braces, then items are
#     'batched' together and command is done once.
#  $< is full path of dependent when used in implicit rule
#  We always plan on using the configuration file of the target

.c.obj:
	$(CC) +$(PROJECT).cfg {$< }

.cpp.obj:
	$(CC) +$(PROJECT).cfg {$< }


#  *List Macros*

DLL_DEPENDENCIES =  \
	$(SOURCE:.c=.obj) \
	$(PROJECT).cfg \
	$(TARGET).def \
	$(TARGET).res \
	$(TARGET).h

RES_DEPENDENCIES =  \
	$(PROJECT).cfg \
	$(TARGET).rc \
	$(TARGET).h


#  *Explicit Rules*

#  MAKE builds the _first_ explicit rule that it encounters. This explains why
#     nothing effectively happens when the .cfg:.mak rule is moved to the top
#     of this file.
#  The ALL: target forces MAKE to first resolve the .cfg dependency, and then
#     to build the .dll file. Apparently including .cfg as a component in the
#     body of the .c.obj implied rule is not sufficient to cause .cfg to be
#     built before being used in the compile. Mere inclusion in the command
#     line of the rule body does not force dependency resolution. HOWEVER,
#     the .cfg build _might_ have been forced ahead of the execution of the
#     compiler _if_ .cfg had appeared ahead of the .c line in the definition
#     of DLL_DEPENDENCIES.
#  After linking, the Resource Compiler _must_ be used to add
#     resources to the image. It also marks the image as Windows 3.x compatible.
#  Even if you have no resources you must run the Resource Compiler.

#  Resolution of the next line drives everything else that is built:

ALL: $(PROJECT).cfg $(TARGET).DLL   # resolution of this line drives everything

$(TARGET).DLL: $(DLL_DEPENDENCIES)
	$(TLINK) $(TLINK_OPTIONS)$(TLINK_TYPE) /L$(LIB) @&&|
$(MMOBJ).obj+        # object files
$(SOURCE:.c=.obj)
$(TARGET)            # executable file
$(TARGET).map			# blank response, no map file
vbapi.lib+				# libraries
mathws.lib+
import.lib+
$(MMLIB).lib
$(TARGET).def
|
	$(RC) -v $(TARGET).res $(TARGET).DLL

$(TARGET).res: $(RES_DEPENDENCIES)
	$(RC) -r -v -I$(INCLUDE) $(TARGET).rc


#  *Compiler Configuration File*

$(PROJECT).cfg: $(MAKEFILE).mak
	copy &&|
-c                   # compile only
-d                   # merge duplicate literal strings
-f-                  # no floating point included; don't include libraries
-3                   # 80386
-O2						# fastest code
$(MMCOMPILE)         # see comments where MODEL is defined
-v-						# debugging off
$(COMPILE_TYPE)      # default of DLL with only _export functions exported
-w-par               # ignore warning that parameter is never used
-w-eff					# ignore warning that code has no effect (occurs with Global... macros)
-I$(INCLUDE)
-L$(LIB)
#-P						# compile as C++ to help later
-P-c						# compile based on extension, with default of C
| $(PROJECT).cfg
	rm $(SOURCE:.c=.obj)
